@@ -6,65 +6,33 @@
 
   Bios Handler (Trap #13)
 
-  We intercept and direct some Bios calls to handle input/output to RS-232
-  or the printer etc...
+  We intercept some Bios calls for debugging
 */
-const char Bios_rcsid[] = "Hatari $Id: bios.c,v 1.10 2007/01/16 18:42:59 thothy Exp $";
+const char Bios__fileid[] = "Hatari bios.c : " __DATE__ " " __TIME__;
 
 #include "main.h"
 #include "configuration.h"
 #include "floppy.h"
 #include "log.h"
 #include "m68000.h"
-#include "misc.h"
 #include "printer.h"
 #include "rs232.h"
 #include "stMemory.h"
 #include "bios.h"
 
 
-#define BIOS_DEBUG 0
-
-
 /*-----------------------------------------------------------------------*/
 /**
  * BIOS Return input device status
  * Call 1
  */
-static BOOL Bios_Bconstat(Uint32 Params)
+static bool Bios_Bconstat(Uint32 Params)
 {
 	Uint16 Dev;
 
 	Dev = STMemory_ReadWord(Params+SIZE_WORD);
 
-	switch(Dev)
-	{
-	 case 0:                            /* PRT: Centronics */
-		if (ConfigureParams.Printer.bEnablePrinting)
-		{
-			Regs[REG_D0] = 0;               /* No characters ready (cannot read from printer) */
-			return TRUE;
-		}
-		else
-		{
-			return FALSE;
-		}
-		break;
-	 case 1:                            /* AUX: RS-232 */
-		if (ConfigureParams.RS232.bEnableRS232)
-		{
-			if (RS232_GetStatus())
-				Regs[REG_D0] = -1;      /* Chars waiting */
-			else
-				Regs[REG_D0] = 0;
-			return TRUE;
-		}
-		else
-		{
-			return FALSE;
-		}
-		break;
-	}
+	HATARI_TRACE(HATARI_TRACE_OS_BIOS, "BIOS Bconstat(%i)\n", Dev);
 
 	return FALSE;
 }
@@ -75,39 +43,13 @@ static BOOL Bios_Bconstat(Uint32 Params)
  * BIOS Read character from device
  * Call 2
  */
-static BOOL Bios_Bconin(Uint32 Params)
+static bool Bios_Bconin(Uint32 Params)
 {
 	Uint16 Dev;
-	unsigned char Char;
 
 	Dev = STMemory_ReadWord(Params+SIZE_WORD);
 
-	switch(Dev)
-	{
-	 case 0:                            /* PRT: Centronics */
-		if (ConfigureParams.Printer.bEnablePrinting)
-		{
-			Regs[REG_D0] = 0;           /* Force NULL character (cannot read from printer) */
-			return TRUE;
-		}
-		else
-		{
-			return FALSE;
-		}
-		break;
-	 case 1:                            /* AUX: RS-232 */
-		if (ConfigureParams.RS232.bEnableRS232)
-		{
-			RS232_ReadBytes(&Char, 1);
-			Regs[REG_D0] = Char;
-			return TRUE;
-		}
-		else
-		{
-			return FALSE;
-		}
-		break;
-	}
+	HATARI_TRACE(HATARI_TRACE_OS_BIOS, "BIOS Bconin(%i)\n", Dev);
 
 	return FALSE;
 }
@@ -118,7 +60,7 @@ static BOOL Bios_Bconin(Uint32 Params)
  * BIOS Write character to device
  * Call 3
  */
-static BOOL Bios_Bconout(Uint32 Params)
+static bool Bios_Bconout(Uint32 Params)
 {
 	Uint16 Dev;
 	unsigned char Char;
@@ -126,31 +68,7 @@ static BOOL Bios_Bconout(Uint32 Params)
 	Dev = STMemory_ReadWord(Params+SIZE_WORD);
 	Char = STMemory_ReadWord(Params+SIZE_WORD+SIZE_WORD);
 
-	switch(Dev)
-	{
-	 case 0:                            /* PRT: Centronics */
-		if (ConfigureParams.Printer.bEnablePrinting)
-		{
-			Printer_TransferByteTo(Char);
-			return TRUE;
-		}
-		else
-		{
-			return FALSE;
-		}
-		break;
-	 case 1:                            /* AUX: RS-232 */
-		if (ConfigureParams.RS232.bEnableRS232)
-		{
-			RS232_TransferBytesTo(&Char, 1);
-			return TRUE;
-		}
-		else
-		{
-			return FALSE;
-		}
-		break;
-	}
+	HATARI_TRACE(HATARI_TRACE_OS_BIOS, "BIOS Bconout(%i, 0x%02x)\n", Dev, Char);
 
 	return FALSE;
 }
@@ -161,9 +79,8 @@ static BOOL Bios_Bconout(Uint32 Params)
  * BIOS Read/Write disk sector
  * Call 4
  */
-static BOOL Bios_RWabs(Uint32 Params)
+static bool Bios_RWabs(Uint32 Params)
 {
-#if BIOS_DEBUG
 	Uint32 pBuffer;
 	Uint16 RWFlag, Number, RecNo, Dev;
 
@@ -174,8 +91,8 @@ static BOOL Bios_RWabs(Uint32 Params)
 	RecNo = STMemory_ReadWord(Params+SIZE_WORD+SIZE_WORD+SIZE_LONG+SIZE_WORD);
 	Dev = STMemory_ReadWord(Params+SIZE_WORD+SIZE_WORD+SIZE_LONG+SIZE_WORD+SIZE_WORD);
 
-	Log_Printf(LOG_DEBUG, "RWABS %s,%d,0x%X,%d,%d\n", EmulationDrives[Dev].szFileName,RWFlag, (char *)STRAM_ADDR(pBuffer), RecNo, Number);
-#endif
+	HATARI_TRACE(HATARI_TRACE_OS_BIOS, "BIOS RWabs %i,%d,0x%lX,%d,%d\n",
+			Dev, RWFlag, STRAM_ADDR(pBuffer), RecNo, Number);
 
 	return FALSE;
 }
@@ -186,37 +103,13 @@ static BOOL Bios_RWabs(Uint32 Params)
  * BIOS Return output device status
  * Call 8
  */
-static BOOL Bios_Bcostat(Uint32 Params)
+static bool Bios_Bcostat(Uint32 Params)
 {
 	Uint16 Dev;
 
 	Dev = STMemory_ReadWord(Params+SIZE_WORD);
 
-	switch(Dev)
-	{
-	 case 0:                            /* PRT: Centronics */
-		if (ConfigureParams.Printer.bEnablePrinting)
-		{
-			Regs[REG_D0] = -1;          /* Device ready */
-			return TRUE;
-		}
-		else
-		{
-			return FALSE;
-		}
-		break;
-	 case 1:                            /* AUX: RS-232 */
-		if (ConfigureParams.RS232.bEnableRS232)
-		{
-			Regs[REG_D0] = -1;          /* Device ready */
-			return TRUE;
-		}
-		else
-		{
-			return FALSE;
-		}
-		break;
-	}
+	HATARI_TRACE(HATARI_TRACE_OS_BIOS, "BIOS Bcostat(%i)\n", Dev);
 
 	return FALSE;
 }
@@ -227,7 +120,7 @@ static BOOL Bios_Bcostat(Uint32 Params)
  * Check Bios call and see if we need to re-direct to our own routines
  * Return TRUE if we've handled the exception, else return FALSE to let TOS attempt it
  */
-BOOL Bios(void)
+bool Bios(void)
 {
 	Uint32 Params;
 	Uint16 BiosCall;
@@ -236,8 +129,6 @@ BOOL Bios(void)
 	Params = Regs[REG_A7];
 	BiosCall = STMemory_ReadWord(Params);
 
-	/* Debug_File("BIOS %d\n",BiosCall); */
-
 	/* Intercept? */
 	switch(BiosCall)
 	{
@@ -252,6 +143,7 @@ BOOL Bios(void)
 	 case 0x8:
 		return Bios_Bcostat(Params);
 	 default:           /* Call as normal! */
+		HATARI_TRACE ( HATARI_TRACE_OS_BIOS, "BIOS %d\n", BiosCall );
 		return FALSE;
 	}
 }
