@@ -54,62 +54,15 @@
 /  a friend, but please do not charge him....
 /
 /---------------------------------------------------------------------*/
-const char CfgOpts_rcsid[] = "Hatari $Id: cfgopts.c,v 1.15 2007/12/18 20:35:04 thothy Exp $";
-
-#include <config.h>
+const char CfgOpts_fileid[] = "Hatari cfgopts.c : " __DATE__ " " __TIME__;
 
 #include <stdio.h>
 #include <stdlib.h>
-#include <string.h>
 #include <unistd.h>
 
-#if HAVE_STRINGS_H
-#include <strings.h>
-#endif
-
 #include "main.h"
 #include "cfgopts.h"
-
-
-
-/**
- *  --- Remove spaces from beginning and end of a string ---
- */
-static char *trim(char *buffer)
-{
-	const char SPACE = ' ';
-	const char TABULA = '\t';
-
-	if (buffer != NULL)
-	{
-		int i, linelen;
-
-		linelen = strlen(buffer);
-
-		for (i = 0; i < linelen; i++)
-		{
-			if (buffer[i] != SPACE && buffer[i] != TABULA)
-				break;
-		}
-
-		if (i > 0 && i < linelen)
-		{
-			linelen -= i;
-			memmove(buffer, buffer + i, linelen);
-		}
-
-		for (i = linelen; i > 0; i--)
-		{
-			int j = i - 1;
-			if (buffer[j] != SPACE && buffer[j] != TABULA)
-				break;
-		}
-
-		buffer[i] = '\0';
-	}
-
-	return buffer;
-}
+#include "str.h"
 
 
 /**
@@ -141,7 +94,7 @@ int input_config(const char *filename, const struct Config_Tag configs[], const
 	{
 		do
 		{
-			fptr = trim(fgets(line, sizeof(line), file));  /* get input line */
+			fptr = Str_Trim(fgets(line, sizeof(line), file));  /* get input line */
 		}
 		while ( memcmp(line,header,strlen(header)) && !feof(file));
 	}
@@ -149,7 +102,7 @@ int input_config(const char *filename, const struct Config_Tag configs[], const
 	if ( !feof(file) )
 		do
 		{
-			fptr = trim(fgets(line, sizeof(line), file));   /* get input line */
+			fptr = Str_Trim(fgets(line, sizeof(line), file));   /* get input line */
 			if (fptr == NULL)
 				continue;
 			lineno++;
@@ -157,10 +110,10 @@ int input_config(const char *filename, const struct Config_Tag configs[], const
 				continue;                               /* skip comments */
 			if (line[0] == '[')
 				continue;                               /* skip next header */
-			tok = trim(strtok(line, "=\n\r"));          /* get first token */
+			tok = Str_Trim(strtok(line, "=\n\r"));          /* get first token */
 			if (tok != NULL)
 			{
-				next = trim(strtok(NULL, "=\n\r"));     /* get actual config information */
+				next = Str_Trim(strtok(NULL, "=\n\r"));     /* get actual config information */
 				for (ptr = configs; ptr->buf; ++ptr)    /* scan for token */
 				{
 					if (!strcmp(tok, ptr->code))        /* got a match? */
@@ -169,9 +122,9 @@ int input_config(const char *filename, const struct Config_Tag configs[], const
 						{
 						 case Bool_Tag:
 							if (!strcasecmp(next,"FALSE"))
-								*((BOOL *)(ptr->buf)) = FALSE;
+								*((bool *)(ptr->buf)) = false;
 							else if (!strcasecmp(next,"TRUE"))
-								*((BOOL *)(ptr->buf)) = TRUE;
+								*((bool *)(ptr->buf)) = true;
 							++count;
 							break;
 
@@ -240,7 +193,7 @@ static int write_token(FILE *outfile, const struct Config_Tag *ptr)
 	switch (ptr->type)    /* check type */
 	{
 	 case Bool_Tag:
-		fprintf(outfile,"%s\n", *((BOOL *)(ptr->buf)) ? "TRUE" : "FALSE");
+		fprintf(outfile,"%s\n", *((bool *)(ptr->buf)) ? "TRUE" : "FALSE");
 		break;
 
 	 case Char_Tag:
@@ -326,7 +279,7 @@ int update_config(const char *filename, const struct Config_Tag configs[], const
 	FILE *cfgfile, *tempfile;
 	char *fptr, *tok, *next;
 	char line[1024];
-	BOOL bUseTempCfg = FALSE;
+	bool bUseTempCfg = false;
 	const char *sTempCfgName = "_temp_.cfg";
 
 	cfgfile = fopen(filename, "r");
@@ -361,10 +314,10 @@ int update_config(const char *filename, const struct Config_Tag configs[], const
 		int headerlen = strlen(header);
 		do
 		{
-			fptr = trim(fgets(line, sizeof(line), cfgfile));  /* get input line */
+			fptr = Str_Trim(fgets(line, sizeof(line), cfgfile));  /* get input line */
 			if (feof(cfgfile))
 				break;
-			fprintf(tempfile, "%s", line);
+			fprintf(tempfile, "%s\n", line);
 		}
 		while(memcmp(line, header, headerlen));
 	}
@@ -392,13 +345,13 @@ int update_config(const char *filename, const struct Config_Tag configs[], const
 
 		for(;;)
 		{
-			fptr = trim(fgets(line, sizeof(line), cfgfile));  /* get input line */
+			fptr = Str_Trim(fgets(line, sizeof(line), cfgfile));  /* get input line */
 			if (fptr == NULL)
 				break;
 			lineno++;
 			if (line[0] == '#')
 			{
-				fprintf(tempfile, "%s", line);
+				fprintf(tempfile, "%s\n", line);
 				continue;                                 /* skip comments */
 			}
 			if (line[0] == '[' || feof(cfgfile))
@@ -406,7 +359,7 @@ int update_config(const char *filename, const struct Config_Tag configs[], const
 				break;
 			}
 
-			tok = trim(strtok(line, "=\n\r"));           /* get first token */
+			tok = Str_Trim(strtok(line, "=\n\r"));           /* get first token */
 			if (tok != NULL)
 			{
 				int i = 0;
@@ -450,14 +403,14 @@ int update_config(const char *filename, const struct Config_Tag configs[], const
 		}
 
 		if (!feof(cfgfile) && fptr != NULL)
-			fprintf(tempfile, "\n%s", line);
+			fprintf(tempfile, "\n%s\n", line);
 
 		for(;;)
 		{
-			fptr = trim(fgets(line, sizeof(line), cfgfile));  /* get input line */
+			fptr = Str_Trim(fgets(line, sizeof(line), cfgfile));  /* get input line */
 			if (feof(cfgfile))
 				break;
-			fprintf(tempfile, "%s", line);
+			fprintf(tempfile, "%s\n", line);
 		}
 	}
 
