@@ -9,7 +9,7 @@
   The configuration file is now stored in an ASCII format to allow the user
   to edit the file manually.
 */
-const char Configuration_rcsid[] = "Hatari $Id: configuration.c,v 1.84 2008/03/11 20:11:08 eerot Exp $";
+const char Configuration_fileid[] = "Hatari configuration.c : " __DATE__ " " __TIME__;
 
 #include <SDL_keysym.h>
 
@@ -27,7 +27,6 @@ const char Configuration_rcsid[] = "Hatari $Id: configuration.c,v 1.84 2008/03/1
 #include "video.h"
 
 
-BOOL bFirstTimeInstall = FALSE;             /* Has been run before? Used to set default joysticks etc... */
 CNF_PARAMS ConfigureParams;                 /* List of configuration for the emulator */
 char sConfigFileName[FILENAME_MAX];         /* Stores the name of the configuration file */
 
@@ -36,6 +35,7 @@ char sConfigFileName[FILENAME_MAX];         /* Stores the name of the configurat
 static const struct Config_Tag configs_Log[] =
 {
 	{ "sLogFileName", String_Tag, ConfigureParams.Log.sLogFileName },
+	{ "sTraceFileName", String_Tag, ConfigureParams.Log.sTraceFileName },
 	{ "nTextLogLevel", Int_Tag, &ConfigureParams.Log.nTextLogLevel },
 	{ "nAlertDlgLogLevel", Int_Tag, &ConfigureParams.Log.nAlertDlgLogLevel },
 	{ "bConfirmQuit", Bool_Tag, &ConfigureParams.Log.bConfirmQuit },
@@ -46,8 +46,8 @@ static const struct Config_Tag configs_Log[] =
 /* Used to load/save screen options */
 static const struct Config_Tag configs_Screen[] =
 {
-	{ "MonitorType", Int_Tag, &ConfigureParams.Screen.MonitorType },
-	{ "FrameSkips", Int_Tag, &ConfigureParams.Screen.FrameSkips },
+	{ "nMonitorType", Int_Tag, &ConfigureParams.Screen.nMonitorType },
+	{ "nFrameSkips", Int_Tag, &ConfigureParams.Screen.nFrameSkips },
 	{ "bFullScreen", Bool_Tag, &ConfigureParams.Screen.bFullScreen },
 	{ "bAllowOverscan", Bool_Tag, &ConfigureParams.Screen.bAllowOverscan },
 	{ "nSpec512Threshold", Int_Tag, &ConfigureParams.Screen.nSpec512Threshold },
@@ -57,11 +57,14 @@ static const struct Config_Tag configs_Screen[] =
 	{ "nVdiWidth", Int_Tag, &ConfigureParams.Screen.nVdiWidth },
 	{ "nVdiHeight", Int_Tag, &ConfigureParams.Screen.nVdiHeight },
 	{ "nVdiColors", Int_Tag, &ConfigureParams.Screen.nVdiColors },
+	{ "bShowStatusbar", Bool_Tag, &ConfigureParams.Screen.bShowStatusbar },
+	{ "bShowDriveLed", Bool_Tag, &ConfigureParams.Screen.bShowDriveLed },
 	{ "bCaptureChange", Bool_Tag, &ConfigureParams.Screen.bCaptureChange },
-	{ "nFramesPerSecond", Int_Tag, &ConfigureParams.Screen.nFramesPerSecond },
+	{ "nWindowBorderPixelsTop", Int_Tag, &ConfigureParams.Screen.nWindowBorderPixelsTop },
 	{ "nWindowBorderPixelsLeft", Int_Tag, &ConfigureParams.Screen.nWindowBorderPixelsLeft },
 	{ "nWindowBorderPixelsRight", Int_Tag, &ConfigureParams.Screen.nWindowBorderPixelsRight },
 	{ "nWindowBorderPixelsBottom", Int_Tag, &ConfigureParams.Screen.nWindowBorderPixelsBottom },
+	{ "nFullScreenBorderPixelsTop", Int_Tag, &ConfigureParams.Screen.nFullScreenBorderPixelsTop },
 	{ "nFullScreenBorderPixelsLeft", Int_Tag, &ConfigureParams.Screen.nFullScreenBorderPixelsLeft },
 	{ "nFullScreenBorderPixelsRight", Int_Tag, &ConfigureParams.Screen.nFullScreenBorderPixelsRight },
 	{ "nFullScreenBorderPixelsBottom", Int_Tag, &ConfigureParams.Screen.nFullScreenBorderPixelsBottom },
@@ -172,13 +175,15 @@ static const struct Config_Tag configs_ShortCutWithMod[] =
 	{ "keyScreenShot", Int_Tag, &ConfigureParams.Shortcut.withModifier[SHORTCUT_SCREENSHOT] },
 	{ "keyBossKey",    Int_Tag, &ConfigureParams.Shortcut.withModifier[SHORTCUT_BOSSKEY] },
 	{ "keyCursorEmu",  Int_Tag, &ConfigureParams.Shortcut.withModifier[SHORTCUT_CURSOREMU] },
-	{ "keyMaxSpeed",   Int_Tag, &ConfigureParams.Shortcut.withModifier[SHORTCUT_MAXSPEED] },
+	{ "keyFastForward",Int_Tag, &ConfigureParams.Shortcut.withModifier[SHORTCUT_FASTFORWARD] },
 	{ "keyRecAnim",    Int_Tag, &ConfigureParams.Shortcut.withModifier[SHORTCUT_RECANIM] },
 	{ "keyRecSound",   Int_Tag, &ConfigureParams.Shortcut.withModifier[SHORTCUT_RECSOUND] },
 	{ "keySound",      Int_Tag, &ConfigureParams.Shortcut.withModifier[SHORTCUT_SOUND] },
+	{ "keyDebug",      Int_Tag, &ConfigureParams.Shortcut.withModifier[SHORTCUT_PAUSE] },
 	{ "keyQuit",       Int_Tag, &ConfigureParams.Shortcut.withModifier[SHORTCUT_QUIT] },
 	{ "keyLoadMem",    Int_Tag, &ConfigureParams.Shortcut.withModifier[SHORTCUT_LOADMEM] },
 	{ "keySaveMem",    Int_Tag, &ConfigureParams.Shortcut.withModifier[SHORTCUT_SAVEMEM] },
+	{ "keyInsertDiskA",Int_Tag, &ConfigureParams.Shortcut.withModifier[SHORTCUT_INSERTDISKA] },
 	{ NULL , Error_Tag, NULL }
 };
 
@@ -193,13 +198,15 @@ static const struct Config_Tag configs_ShortCutWithoutMod[] =
 	{ "keyScreenShot", Int_Tag, &ConfigureParams.Shortcut.withoutModifier[SHORTCUT_SCREENSHOT] },
 	{ "keyBossKey",    Int_Tag, &ConfigureParams.Shortcut.withoutModifier[SHORTCUT_BOSSKEY] },
 	{ "keyCursorEmu",  Int_Tag, &ConfigureParams.Shortcut.withoutModifier[SHORTCUT_CURSOREMU] },
-	{ "keyMaxSpeed",   Int_Tag, &ConfigureParams.Shortcut.withoutModifier[SHORTCUT_MAXSPEED] },
+	{ "keyFastForward",Int_Tag, &ConfigureParams.Shortcut.withoutModifier[SHORTCUT_FASTFORWARD] },
 	{ "keyRecAnim",    Int_Tag, &ConfigureParams.Shortcut.withoutModifier[SHORTCUT_RECANIM] },
 	{ "keyRecSound",   Int_Tag, &ConfigureParams.Shortcut.withoutModifier[SHORTCUT_RECSOUND] },
 	{ "keySound",      Int_Tag, &ConfigureParams.Shortcut.withoutModifier[SHORTCUT_SOUND] },
+	{ "keyDebug",      Int_Tag, &ConfigureParams.Shortcut.withoutModifier[SHORTCUT_PAUSE] },
 	{ "keyQuit",       Int_Tag, &ConfigureParams.Shortcut.withoutModifier[SHORTCUT_QUIT] },
 	{ "keyLoadMem",    Int_Tag, &ConfigureParams.Shortcut.withoutModifier[SHORTCUT_LOADMEM] },
 	{ "keySaveMem",    Int_Tag, &ConfigureParams.Shortcut.withoutModifier[SHORTCUT_SAVEMEM] },
+	{ "keyInsertDiskA",Int_Tag, &ConfigureParams.Shortcut.withoutModifier[SHORTCUT_INSERTDISKA] },
 	{ NULL , Error_Tag, NULL }
 };
 
@@ -229,6 +236,10 @@ static const struct Config_Tag configs_Floppy[] =
 {
 	{ "bAutoInsertDiskB", Bool_Tag, &ConfigureParams.DiskImage.bAutoInsertDiskB },
 	{ "nWriteProtection", Int_Tag, &ConfigureParams.DiskImage.nWriteProtection },
+	{ "szDiskAZipPath", String_Tag, ConfigureParams.DiskImage.szDiskZipPath[0] },
+	{ "szDiskAFileName", String_Tag, ConfigureParams.DiskImage.szDiskFileName[0] },
+	{ "szDiskBZipPath", String_Tag, ConfigureParams.DiskImage.szDiskZipPath[1] },
+	{ "szDiskBFileName", String_Tag, ConfigureParams.DiskImage.szDiskFileName[1] },
 	{ "szDiskImageDirectory", String_Tag, ConfigureParams.DiskImage.szDiskImageDirectory },
 	{ NULL , Error_Tag, NULL }
 };
@@ -277,7 +288,8 @@ static const struct Config_Tag configs_Printer[] =
 static const struct Config_Tag configs_Midi[] =
 {
 	{ "bEnableMidi", Bool_Tag, &ConfigureParams.Midi.bEnableMidi },
-	{ "szMidiOutFileName", String_Tag, ConfigureParams.Midi.szMidiOutFileName },
+	{ "sMidiInFileName", String_Tag, ConfigureParams.Midi.sMidiInFileName },
+	{ "sMidiOutFileName", String_Tag, ConfigureParams.Midi.sMidiOutFileName },
 	{ NULL , Error_Tag, NULL }
 };
 
@@ -293,7 +305,7 @@ static const struct Config_Tag configs_System[] =
 	{ "bRealTimeClock", Bool_Tag, &ConfigureParams.System.bRealTimeClock },
 	{ "bPatchTimerD", Bool_Tag, &ConfigureParams.System.bPatchTimerD },
 	{ "bSlowFDC", Bool_Tag, &ConfigureParams.System.bSlowFDC },
-	{ "nMinMaxSpeed", Int_Tag, &ConfigureParams.System.nMinMaxSpeed },
+	{ "bFastForward", Bool_Tag, &ConfigureParams.System.bFastForward },
 	{ NULL , Error_Tag, NULL }
 };
 
@@ -311,21 +323,24 @@ void Configuration_SetDefault(void)
 	psHomeDir = Paths_GetHatariHome();
 	psWorkingDir = Paths_GetWorkingDir();
 
-	/* Assume first-time install */
-	bFirstTimeInstall = TRUE;
-
 	/* Clear parameters */
 	memset(&ConfigureParams, 0, sizeof(CNF_PARAMS));
 
 	/* Set defaults for logging */
 	strcpy(ConfigureParams.Log.sLogFileName, "stderr");
-	ConfigureParams.Log.nTextLogLevel = LOG_INFO;
-	ConfigureParams.Log.nAlertDlgLogLevel = LOG_INFO;
+	strcpy(ConfigureParams.Log.sTraceFileName, "stderr");
+	ConfigureParams.Log.nTextLogLevel = LOG_TODO;
+	ConfigureParams.Log.nAlertDlgLogLevel = LOG_ERROR;
 	ConfigureParams.Log.bConfirmQuit = TRUE;
 
 	/* Set defaults for floppy disk images */
 	ConfigureParams.DiskImage.bAutoInsertDiskB = TRUE;
 	ConfigureParams.DiskImage.nWriteProtection = WRITEPROT_OFF;
+	for (i = 0; i < 2; i++)
+	{
+		ConfigureParams.DiskImage.szDiskZipPath[i][0] = '\0';
+		ConfigureParams.DiskImage.szDiskFileName[i][0] = '\0';
+	}
 	strcpy(ConfigureParams.DiskImage.szDiskImageDirectory, psWorkingDir);
 	File_AddSlashToEndFileName(ConfigureParams.DiskImage.szDiskImageDirectory);
 
@@ -333,7 +348,7 @@ void Configuration_SetDefault(void)
 	ConfigureParams.HardDisk.bBootFromHardDisk = FALSE;
 	ConfigureParams.HardDisk.nHardDiskDir = DRIVE_C;
 	ConfigureParams.HardDisk.bUseHardDiskDirectories = FALSE;
-	for (i=0; i<MAX_HARDDRIVES; i++)
+	for (i = 0; i < MAX_HARDDRIVES; i++)
 	{
 		strcpy(ConfigureParams.HardDisk.szHardDiskDirectories[i], psWorkingDir);
 		File_CleanFileName(ConfigureParams.HardDisk.szHardDiskDirectories[i]);
@@ -357,15 +372,17 @@ void Configuration_SetDefault(void)
 	}
 	ConfigureParams.Joysticks.Joy[1].nJoyId = 0;    /* ST Joystick #1 is default joystick */
 	ConfigureParams.Joysticks.Joy[0].nJoyId = 1;
+	ConfigureParams.Joysticks.Joy[1].nJoystickMode = JOYSTICK_REALSTICK;
 
 	/* Set defaults for Keyboard */
-	ConfigureParams.Keyboard.bDisableKeyRepeat = TRUE;
+	ConfigureParams.Keyboard.bDisableKeyRepeat = FALSE;
 	ConfigureParams.Keyboard.nKeymapType = KEYMAP_SYMBOLIC;
 	strcpy(ConfigureParams.Keyboard.szMappingFileName, "");
   
 	/* Set defaults for Shortcuts */
 	ConfigureParams.Shortcut.withoutModifier[SHORTCUT_OPTIONS] = SDLK_F12;
 	ConfigureParams.Shortcut.withoutModifier[SHORTCUT_FULLSCREEN] = SDLK_F11;
+	ConfigureParams.Shortcut.withoutModifier[SHORTCUT_PAUSE] = SDLK_PAUSE;
   
 	ConfigureParams.Shortcut.withModifier[SHORTCUT_OPTIONS] = SDLK_o;
 	ConfigureParams.Shortcut.withModifier[SHORTCUT_FULLSCREEN] = SDLK_f;
@@ -375,14 +392,15 @@ void Configuration_SetDefault(void)
 	ConfigureParams.Shortcut.withModifier[SHORTCUT_SCREENSHOT] = SDLK_g;
 	ConfigureParams.Shortcut.withModifier[SHORTCUT_BOSSKEY] = SDLK_i;
 	ConfigureParams.Shortcut.withModifier[SHORTCUT_CURSOREMU] = SDLK_j;
-	ConfigureParams.Shortcut.withModifier[SHORTCUT_MAXSPEED] = SDLK_x;
+	ConfigureParams.Shortcut.withModifier[SHORTCUT_FASTFORWARD] = SDLK_x;
 	ConfigureParams.Shortcut.withModifier[SHORTCUT_RECANIM] = SDLK_a;
 	ConfigureParams.Shortcut.withModifier[SHORTCUT_RECSOUND] = SDLK_y;
 	ConfigureParams.Shortcut.withModifier[SHORTCUT_SOUND] = SDLK_s;
 	ConfigureParams.Shortcut.withModifier[SHORTCUT_QUIT] = SDLK_q;
 	ConfigureParams.Shortcut.withModifier[SHORTCUT_LOADMEM] = SDLK_l;
 	ConfigureParams.Shortcut.withModifier[SHORTCUT_SAVEMEM] = SDLK_k;
-	
+	ConfigureParams.Shortcut.withModifier[SHORTCUT_INSERTDISKA] = SDLK_d;
+
 	/* Set defaults for Memory */
 	ConfigureParams.Memory.nMemorySize = 1;     /* 1 MiB */
 	ConfigureParams.Memory.bAutoSave = FALSE;
@@ -404,28 +422,32 @@ void Configuration_SetDefault(void)
 
 	/* Set defaults for MIDI */
 	ConfigureParams.Midi.bEnableMidi = FALSE;
-	strcpy(ConfigureParams.Midi.szMidiOutFileName, "/dev/midi00");
+	strcpy(ConfigureParams.Midi.sMidiInFileName, "/dev/snd/midiC1D0");
+	strcpy(ConfigureParams.Midi.sMidiOutFileName, "/dev/snd/midiC1D0");
 
 	/* Set defaults for Screen */
 	ConfigureParams.Screen.bFullScreen = FALSE;
-	ConfigureParams.Screen.FrameSkips = 0;
+	ConfigureParams.Screen.nFrameSkips = AUTO_FRAMESKIP_LIMIT;
 	ConfigureParams.Screen.bAllowOverscan = TRUE;
 	ConfigureParams.Screen.nSpec512Threshold = 16;
 	ConfigureParams.Screen.nForceBpp = 0;
 	ConfigureParams.Screen.bZoomLowRes = FALSE;
-	ConfigureParams.Screen.MonitorType = MONITOR_TYPE_RGB;
+	ConfigureParams.Screen.nMonitorType = MONITOR_TYPE_RGB;
 	ConfigureParams.Screen.bUseExtVdiResolutions = FALSE;
 	ConfigureParams.Screen.nVdiWidth = 640;
 	ConfigureParams.Screen.nVdiHeight = 480;
 	ConfigureParams.Screen.nVdiColors = GEMCOLOR_16;
+	ConfigureParams.Screen.bShowStatusbar = TRUE;
+	ConfigureParams.Screen.bShowDriveLed = TRUE;
 	ConfigureParams.Screen.bCaptureChange = FALSE;
-	ConfigureParams.Screen.nFramesPerSecond = 25;
+	ConfigureParams.Screen.nWindowBorderPixelsTop = OVERSCAN_TOP;
 	ConfigureParams.Screen.nWindowBorderPixelsLeft = 48;
 	ConfigureParams.Screen.nWindowBorderPixelsRight = 48;
-	ConfigureParams.Screen.nWindowBorderPixelsBottom = OVERSCAN_BOTTOM;
+	ConfigureParams.Screen.nWindowBorderPixelsBottom = MAX_OVERSCAN_BOTTOM-2;
+	ConfigureParams.Screen.nFullScreenBorderPixelsTop = OVERSCAN_TOP;
 	ConfigureParams.Screen.nFullScreenBorderPixelsLeft = 32;
 	ConfigureParams.Screen.nFullScreenBorderPixelsRight = 32;
-	ConfigureParams.Screen.nFullScreenBorderPixelsBottom = OVERSCAN_BOTTOM;
+	ConfigureParams.Screen.nFullScreenBorderPixelsBottom = MAX_OVERSCAN_BOTTOM-2;
 
 	/* Set defaults for Sound */
 	ConfigureParams.Sound.bEnableSound = TRUE;
@@ -448,7 +470,7 @@ void Configuration_SetDefault(void)
 	ConfigureParams.System.nDSPType = DSP_TYPE_NONE;
 	ConfigureParams.System.bPatchTimerD = TRUE;
 	ConfigureParams.System.bRealTimeClock = TRUE;
-	ConfigureParams.System.nMinMaxSpeed = MINMAXSPEED_MIN;
+	ConfigureParams.System.bFastForward = FALSE;
 	ConfigureParams.System.bSlowFDC = FALSE;
 
 	/* Initialize the configuration file name */
@@ -469,13 +491,13 @@ void Configuration_SetDefault(void)
  * Copy details from configuration structure into global variables for system,
  * clean file names, etc...  Called from main.c and dialog.c files.
  */
-void Configuration_Apply(BOOL bReset)
+void Configuration_Apply(bool bReset)
 {
 	if (bReset)
 	{
 		/* Set resolution change */
 		bUseVDIRes = ConfigureParams.Screen.bUseExtVdiResolutions;
-		bUseHighRes = ((!bUseVDIRes) && ConfigureParams.Screen.MonitorType == MONITOR_TYPE_MONO)
+		bUseHighRes = ((!bUseVDIRes) && ConfigureParams.Screen.nMonitorType == MONITOR_TYPE_MONO)
 			|| (bUseVDIRes && ConfigureParams.Screen.nVdiColors == GEMCOLOR_2);
 		if (bUseHighRes)
 		{
@@ -488,11 +510,15 @@ void Configuration_Apply(BOOL bReset)
 			                  ConfigureParams.Screen.nVdiHeight);
 		}
 	}
-
+	if (ConfigureParams.Screen.nFrameSkips < AUTO_FRAMESKIP_LIMIT)
+	{
+		nFrameSkips = ConfigureParams.Screen.nFrameSkips;
+	}
 	/* Set playback frequency */
 	if (ConfigureParams.Sound.bEnableSound)
+	{
 		Audio_SetOutputAudioFreq(ConfigureParams.Sound.nPlaybackQuality);
-
+	}
 	/* CPU settings */
 	if (ConfigureParams.System.nCpuFreq < 12)
 	{
@@ -523,12 +549,14 @@ void Configuration_Apply(BOOL bReset)
 	File_MakeAbsoluteName(ConfigureParams.Sound.szYMCaptureFileName);
 	if (strlen(ConfigureParams.Keyboard.szMappingFileName) > 0)
 		File_MakeAbsoluteName(ConfigureParams.Keyboard.szMappingFileName);
-	File_MakeAbsoluteName(ConfigureParams.RS232.szOutFileName);
-	File_MakeAbsoluteName(ConfigureParams.RS232.szInFileName);
 	
 	/* make path names absolute, but handle special file names */
 	File_MakeAbsoluteSpecialName(ConfigureParams.Log.sLogFileName);
-	File_MakeAbsoluteSpecialName(ConfigureParams.Midi.szMidiOutFileName);
+	File_MakeAbsoluteSpecialName(ConfigureParams.Log.sTraceFileName);
+	File_MakeAbsoluteSpecialName(ConfigureParams.RS232.szInFileName);
+	File_MakeAbsoluteSpecialName(ConfigureParams.RS232.szOutFileName);
+	File_MakeAbsoluteSpecialName(ConfigureParams.Midi.sMidiInFileName);
+	File_MakeAbsoluteSpecialName(ConfigureParams.Midi.sMidiOutFileName);
 	File_MakeAbsoluteSpecialName(ConfigureParams.Printer.szPrintToFileName);
 }
 
@@ -567,8 +595,6 @@ void Configuration_Load(const char *psFileName)
 		return;
 	}
 
-	bFirstTimeInstall = FALSE;
-
 	Configuration_LoadSection(psFileName, configs_Log, "[Log]");
 	Configuration_LoadSection(psFileName, configs_Screen, "[Screen]");
 	Configuration_LoadSection(psFileName, configs_Joystick0, "[Joystick0]");
@@ -617,7 +643,7 @@ void Configuration_Save(void)
 {
 	if (Configuration_SaveSection(sConfigFileName, configs_Log, "[Log]") < 0)
 	{
-		fprintf(stderr, "Error saving config file.\n");
+		Log_AlertDlg(LOG_ERROR, "Error saving config file.");
 		return;
 	}
 	Configuration_SaveSection(sConfigFileName, configs_Screen, "[Screen]");
@@ -647,7 +673,7 @@ void Configuration_Save(void)
  * Save/restore snapshot of configuration variables
  * ('MemorySnapShot_Store' handles type)
  */
-void Configuration_MemorySnapShot_Capture(BOOL bSave)
+void Configuration_MemorySnapShot_Capture(bool bSave)
 {
 	MemorySnapShot_Store(ConfigureParams.Rom.szTosImageFileName, sizeof(ConfigureParams.Rom.szTosImageFileName));
 	MemorySnapShot_Store(ConfigureParams.Rom.szCartridgeImageFileName, sizeof(ConfigureParams.Rom.szCartridgeImageFileName));
@@ -659,7 +685,7 @@ void Configuration_MemorySnapShot_Capture(BOOL bSave)
 	MemorySnapShot_Store(&ConfigureParams.HardDisk.bUseHardDiskImage, sizeof(ConfigureParams.HardDisk.bUseHardDiskImage));
 	MemorySnapShot_Store(ConfigureParams.HardDisk.szHardDiskImage, sizeof(ConfigureParams.HardDisk.szHardDiskImage));
 
-	MemorySnapShot_Store(&ConfigureParams.Screen.MonitorType, sizeof(ConfigureParams.Screen.MonitorType));
+	MemorySnapShot_Store(&ConfigureParams.Screen.nMonitorType, sizeof(ConfigureParams.Screen.nMonitorType));
 	MemorySnapShot_Store(&ConfigureParams.Screen.bUseExtVdiResolutions, sizeof(ConfigureParams.Screen.bUseExtVdiResolutions));
 	MemorySnapShot_Store(&ConfigureParams.Screen.nVdiWidth, sizeof(ConfigureParams.Screen.nVdiWidth));
 	MemorySnapShot_Store(&ConfigureParams.Screen.nVdiHeight, sizeof(ConfigureParams.Screen.nVdiHeight));
