@@ -8,7 +8,7 @@
   pressed, the emulator is (hopefully) halted and this little CLI can be used
   (in the terminal box) for debugging tasks like memory and register dumps.
 */
-const char DebugUI_rcsid[] = "Hatari $Id: debugui.c,v 1.18 2007/12/23 18:54:50 thothy Exp $";
+const char DebugUI_fileid[] = "Hatari debugui.c : " __DATE__ " " __TIME__;
 
 #include <ctype.h>
 #include <stdio.h>
@@ -21,9 +21,12 @@ const char DebugUI_rcsid[] = "Hatari $Id: debugui.c,v 1.18 2007/12/23 18:54:50 t
 #endif
 
 #include "main.h"
+#include "change.h"
 #include "configuration.h"
+#include "file.h"
 #include "reset.h"
 #include "m68000.h"
+#include "str.h"
 #include "stMemory.h"
 #include "sound.h"
 #include "tos.h"
@@ -31,66 +34,18 @@ const char DebugUI_rcsid[] = "Hatari $Id: debugui.c,v 1.18 2007/12/23 18:54:50 t
 
 #include "hatari-glue.h"
 
-
-#define DEBUG_QUIT     0
-#define DEBUG_CMD      1
-
 #define MEMDUMP_COLS   16      /* memdump, number of bytes per row */
 #define MEMDUMP_ROWS   4       /* memdump, number of rows */
 #define NON_PRINT_CHAR '.'     /* character to display for non-printables */
 #define DISASM_INSTS   5       /* disasm - number of instructions */
 
-static BOOL bMemDump;          /* has memdump been called? */
+static bool bMemDump;          /* has memdump been called? */
 static unsigned long memdump_addr; /* memdump address */
 static unsigned long disasm_addr;  /* disasm address */
 
-static FILE *debugLogFile;
-static FILE *debug_stdout;
+static FILE *debugOutput;
 
 
-/**
- * convert string to lowercase
- */
-static void string_tolower(char *str)
-{
-	int i=0;
-	while(str[i] != '\0')
-	{
-		if(isupper((unsigned)str[i]))
-			str[i] = tolower(str[i]);
-		i++;
-	}
-}
-
-/**
- * truncate string at first unprintable char (e.g. newline)
- */
-static void string_trunc(char *str)
-{
-	int i=0;
-	while (str[i] != '\0')
-	{
-		if (!isprint((unsigned)str[i]))
-			str[i] = '\0';
-		i++;
-	}
-}
-
-/**
- * check if string is valid hex number.
- */
-static BOOL isHex(char *str)
-{
-	int i=0;
-	while (str[i] != '\0' && str[i] != ' ')
-	{
-		if (!isxdigit((unsigned)str[i]))
-			return FALSE;
-		i++;
-	}
-	return TRUE;
-}
-
 
 /*-----------------------------------------------------------------------*/
 /**
@@ -99,9 +54,9 @@ static BOOL isHex(char *str)
  * -2 if a range, but not a valid one.
  * 0 if OK.
  */
-static BOOL getRange(char *str, unsigned long *lower, unsigned long *upper)
+static int getRange(char *str, unsigned long *lower, unsigned long *upper)
 {
-	BOOL fDash = FALSE;
+	bool fDash = FALSE;
 	int i=0;
 
 	while (str[i] != '\0')
@@ -127,14 +82,34 @@ static BOOL getRange(char *str, unsigned long *lower, unsigned long *upper)
 
 /*-----------------------------------------------------------------------*/
 /**
- * Open a log file.
+ * Open given log file.
  */
-static void DebugUI_OpenLog(const char *arg)
+static void DebugUI_SetLogFile(const char *logpath)
 {
-	debugLogFile = fopen(arg, "w");
-	if (debugLogFile == NULL)
-		fprintf(stderr, "Can't open file: %s\n", arg);
-	debug_stdout = debugLogFile;
+	File_Close(debugOutput);
+	debugOutput = File_Open(logpath, "w");
+	if (debugOutput)
+		fprintf(stderr, "Debug log '%s' opened\n", logpath);
+	else
+		debugOutput = stderr;
+}
+
+
+/*-----------------------------------------------------------------------*/
+/**
+ * Close a log file if open, and set it to default stream.
+ */
+static void DebugUI_SetLogDefault(void)
+{
+	if (debugOutput != stderr)
+	{
+		if (debugOutput)
+		{
+			File_Close(debugOutput);
+			fprintf(stderr, "Debug log closed.\n");
+		}
+		debugOutput = stderr;
+	}
 }
 
 
@@ -189,7 +164,7 @@ static void DebugUI_SaveBin(char *args)
 
 	if (sscanf(args, "%s%s%lx%lx", dummy, filename, &address, &bytes) != 4)
 	{
-		fprintf(stderr, "  Invalid arguments!");
+		fprintf(stderr, "  Invalid arguments!\n");
 		return;
 	}
 	address &= 0x00FFFFFF;
@@ -217,7 +192,8 @@ static void DebugUI_RegDump(void)
 {
 	uaecptr nextpc;
 	/* use the UAE function instead */
-	m68k_dumpstate(debug_stdout, &nextpc);
+	m68k_dumpstate(debugOutput, &nextpc);
+	fflush(debugOutput);
 }
 
 
@@ -225,12 +201,12 @@ static void DebugUI_RegDump(void)
 /**
  * Dissassemble - arg = starting address, or PC.
  */
-static void DebugUI_DisAsm(char *arg, BOOL cont)
+static void DebugUI_DisAsm(char *arg, bool cont)
 {
 	int i,j;
 	unsigned long disasm_upper;
 	uaecptr nextpc;
-	BOOL isRange = FALSE;
+	bool isRange = FALSE;
 
 	if (cont != TRUE)
 	{
@@ -238,7 +214,7 @@ static void DebugUI_DisAsm(char *arg, BOOL cont)
 
 		if (j == -1)
 		{ /* single address, not a range */
-			if (!isHex(arg))
+			if (!Str_IsHex(arg))
 			{
 				fprintf(stderr,"Invalid address!\n");
 				return;
@@ -271,17 +247,19 @@ static void DebugUI_DisAsm(char *arg, BOOL cont)
 	/* output a single block. */
 	if (isRange == FALSE)
 	{
-		m68k_disasm(debug_stdout, (uaecptr)disasm_addr, &nextpc, DISASM_INSTS);
+		m68k_disasm(debugOutput, (uaecptr)disasm_addr, &nextpc, DISASM_INSTS);
 		disasm_addr = nextpc;
+		fflush(debugOutput);
 		return;
 	}
 
 	/* output a range */
 	while (disasm_addr < disasm_upper)
 	{
-		m68k_disasm(debug_stdout, (uaecptr)disasm_addr, &nextpc, 1);
+		m68k_disasm(debugOutput, (uaecptr)disasm_addr, &nextpc, 1);
 		disasm_addr = nextpc;
 	}
+	fflush(debugOutput);
 }
 
 
@@ -292,7 +270,7 @@ static void DebugUI_DisAsm(char *arg, BOOL cont)
 static void DebugUI_RegSet(char *arg)
 {
 	int i;
-	BOOL s = FALSE;
+	bool s = FALSE;
 	char reg[4];
 	long value;
 
@@ -416,22 +394,20 @@ static void DebugUI_RegSet(char *arg)
 /**
  * Do a memory dump, args = starting address.
  */
-static void DebugUI_MemDump(char *arg, BOOL cont)
+static void DebugUI_MemDump(char *arg, bool cont)
 {
 	int i,j;
 	char c;
-	BOOL isRange = FALSE;
+	bool isRange = FALSE;
 	unsigned long memdump_upper;
 
-
-
 	if (cont != TRUE)
 	{
 		j = getRange(arg, &memdump_addr, &memdump_upper);
 
 		if (j == -1)
 		{ /* single address, not a range */
-			if (!isHex(arg))
+			if (!Str_IsHex(arg))
 			{
 				bMemDump = FALSE;
 				fprintf(stderr, "Invalid address!\n");
@@ -465,37 +441,39 @@ static void DebugUI_MemDump(char *arg, BOOL cont)
 	{
 		for (j=0;j<MEMDUMP_ROWS;j++)
 		{
-			fprintf(debug_stdout, "%6.6lX: ", memdump_addr); /* print address */
+			fprintf(debugOutput, "%6.6lX: ", memdump_addr); /* print address */
 			for (i = 0; i < MEMDUMP_COLS; i++)               /* print hex data */
-				fprintf(debug_stdout, "%2.2x ", STMemory_ReadByte(memdump_addr++));
-			fprintf(debug_stdout, "  ");                     /* print ASCII data */
+				fprintf(debugOutput, "%2.2x ", STMemory_ReadByte(memdump_addr++));
+			fprintf(debugOutput, "  ");                     /* print ASCII data */
 			for (i = 0; i < MEMDUMP_COLS; i++)
 			{
 				c = STMemory_ReadByte(memdump_addr-MEMDUMP_COLS+i);
 				if (!isprint((unsigned)c))
 					c = NON_PRINT_CHAR;         /* non-printable as dots */
-				fprintf(debug_stdout,"%c", c);
+				fprintf(debugOutput,"%c", c);
 			}
-			fprintf(debug_stdout, "\n");        /* newline */
+			fprintf(debugOutput, "\n");        /* newline */
 		}
+		fflush(debugOutput);
 		return;
 	} /* not a range */
 
 	while (memdump_addr < memdump_upper)
 	{
-		fprintf(debug_stdout, "%6.6lX: ", memdump_addr); /* print address */
+		fprintf(debugOutput, "%6.6lX: ", memdump_addr); /* print address */
 		for (i = 0; i < MEMDUMP_COLS; i++)               /* print hex data */
-			fprintf(debug_stdout, "%2.2x ", STMemory_ReadByte(memdump_addr++));
-		fprintf(debug_stdout, "  ");                     /* print ASCII data */
+			fprintf(debugOutput, "%2.2x ", STMemory_ReadByte(memdump_addr++));
+		fprintf(debugOutput, "  ");                     /* print ASCII data */
 		for (i = 0; i < MEMDUMP_COLS; i++)
 		{
 			c = STMemory_ReadByte(memdump_addr-MEMDUMP_COLS+i);
 			if(!isprint((unsigned)c))
 				c = NON_PRINT_CHAR;             /* non-printable as dots */
-			fprintf(debug_stdout,"%c", c);
+			fprintf(debugOutput,"%c", c);
 		}
-		fprintf(debug_stdout, "\n");            /* newline */
+		fprintf(debugOutput, "\n");            /* newline */
 	} /* while */
+	fflush(debugOutput);
 } /* end of memdump */
 
 
@@ -514,7 +492,7 @@ static void DebugUI_MemWrite(char *arg)
 	numBytes = 0;
 	i = 0;
 
-	string_trunc(arg);
+	Str_Trunc(arg);
 	while (arg[i] == ' ')
 		i++; /* skip spaces */
 	while (arg[i] != ' ')
@@ -591,7 +569,7 @@ static void DebugUI_Help(void)
 	        "   Output of reg & mem dumps and disassembly will be written to the log\n"
 	        " l filename address - load a file into memory starting at address. \n"
 	        " s filename address length - dump length bytes from memory to a file. \n"
-	        " o - disable debug mode\n\n"
+	        " o [command line] - set Hatari command line options\n\n"
 	        " q - quit emulator\n"
 	        " c - continue emulation\n\n"
 	        " Adresses may be given as a range e.g. fc0000-fc0100\nAll values in hexadecimal.\n"
@@ -599,45 +577,32 @@ static void DebugUI_Help(void)
 	        "\n");
 }
 
+
 /*-----------------------------------------------------------------------*/
 /**
- * Get a UI command, return it.
+ * Parse and return debug command.
  */
-static int DebugUI_Getcommand(void)
+int DebugUI_ParseCommand(char *input)
 {
 	char command[255], arg[255];
 	static char lastcommand = 0;
-	char *pInput;
-	int i;
-	int retval;
+	int i, retval;
 
-#if HAVE_LIBREADLINE
-	pInput = readline("> ");
-	if (!pInput)
-		return DEBUG_QUIT;
-	if (pInput[0] != 0)
-		add_history(pInput);
-#else
-	fprintf(stderr, "> ");
-	pInput = malloc(256);
-	if (!pInput)
-		return DEBUG_QUIT;
-	pInput[0] = '\0';
-	fgets(pInput, 256, stdin);
-#endif
-
-	command[0] = lastcommand;          /* Used for 'm' and 'd' to continue at last pos */
+	/* Used for 'm' and 'd' to continue at last pos */
+	command[0] = lastcommand;
 	command[1] = 0;
 	arg[0] = 0;
-	i = sscanf(pInput, "%s%s", command, arg);
-	string_tolower(command);
+	i = sscanf(input, "%s%s", command, arg);
+	Str_ToLower(command);
 
 	if (i == 0)
 	{
 		fprintf(stderr, "  Unknown command.\n");
-		free(pInput);
 		return DEBUG_CMD;
 	}
+	if (!debugOutput) {
+		DebugUI_SetLogDefault();
+	}
 
 	lastcommand = 0;
 	retval = DEBUG_CMD;                /* Default return value */
@@ -660,8 +625,7 @@ static int DebugUI_Getcommand(void)
 		break;
 
 	 case 'o':
-		bEnableDebug = FALSE;
-		fprintf(stderr, "  Debug mode disabled.\n");
+		Change_ApplyCommandline(input+1);
 		break;
 
 	 case 'd':
@@ -687,25 +651,16 @@ static int DebugUI_Getcommand(void)
 
 	 case 'f':
 		if (i < 2)
-		{  /* no arg? */
-			if (debugLogFile == NULL)
-				fprintf(stderr, "No log file open.\n");
-			else
-			{
-				fclose(debugLogFile);
-				debug_stdout = stderr;
-				fprintf(stderr, "Log closed.\n");
-			}
-		}
+			DebugUI_SetLogDefault();
 		else
-			DebugUI_OpenLog(arg);
+			DebugUI_SetLogFile(arg);
 		break;
 
 	 case 'w':
 		if (i < 2)    /* not enough args? */
 			fprintf(stderr, "  Usage: w address bytes\n");
 		else
-			DebugUI_MemWrite(pInput);
+			DebugUI_MemWrite(input);
 		break;
 
 	 case 'r':
@@ -719,14 +674,14 @@ static int DebugUI_Getcommand(void)
 		if (i < 2)    /* not enough args? */
 			fprintf(stderr,"  Usage: l filename address\n");
 		else
-			DebugUI_LoadBin(pInput);
+			DebugUI_LoadBin(input);
 		break;
 
 	 case 's':
 		if (i < 2)    /* not enough args? */
 			fprintf(stderr,"  Usage: s filename address bytes\n");
 		else
-			DebugUI_SaveBin(pInput);
+			DebugUI_SaveBin(input);
 		break;
 
 	 default:
@@ -735,8 +690,35 @@ static int DebugUI_Getcommand(void)
 		break;
 	}
 
-	free(pInput);
+	return retval;
+}
 
+/*-----------------------------------------------------------------------*/
+/**
+ * Get a UI command, parse and return it.
+ */
+static int DebugUI_GetCommand(void)
+{
+	char *input;
+	int retval;
+
+#if HAVE_LIBREADLINE
+	input = readline("> ");
+	if (!input)
+		return DEBUG_QUIT;
+	if (input[0] != 0)
+		add_history(input);
+#else
+	fprintf(stderr, "> ");
+	input = malloc(256);
+	if (!input)
+		return DEBUG_QUIT;
+	input[0] = '\0';
+	fgets(input, 256, stdin);
+#endif
+	retval = DebugUI_ParseCommand(input);
+
+	free(input);
 	return retval;
 }
 
@@ -747,18 +729,13 @@ static int DebugUI_Getcommand(void)
  */
 void DebugUI(void)
 {
-	debugLogFile = NULL;
-	debug_stdout = stderr;  /* output to screen, until log file opened */
-
 	bMemDump = FALSE;
 	disasm_addr = 0;
 
-
 	fprintf(stderr, "\nYou have entered debug mode. Type c to continue emulation, h for help."
 	                "\n----------------------------------------------------------------------\n");
-	while (DebugUI_Getcommand() != DEBUG_QUIT)
+	while (DebugUI_GetCommand() != DEBUG_QUIT)
 		;
-	if (debugLogFile != NULL)
-		fclose(debugLogFile);
 	fprintf(stderr,"Returning to emulation...\n------------------------------\n\n");
+	DebugUI_SetLogDefault();
 }
