@@ -14,8 +14,32 @@
 */
 
 /* 2007/11/06   [NP]    Add calls to HATARI_TRACE and set FDC_DELAY_HBL=180		*/
+/* 2008/05/03	[NP]	Add more traces to all commandes.				*/
+/*			FIXME : after a type II read sector with multi bit 'm' on, we	*/
+/*			should update FDCSectorRegister to be 'max sector for current	*/
+/*			track'+1 and set command status	to 'Record Not Found'.		*/
+/*			Also,  if multi bit is set and sector count is less than number	*/
+/*			of sectors in the track, then the FDC reads the whole track	*/
+/*			anyway, setting RNF at the end, but the DMA stops transferring	*/
+/*			data once DMA sector count $ff8604 reaches 0.			*/
+/*			Timings for read sector with multi bit are not good and prevent	*/
+/*			some programs from working (eg Super Monaco GP on Superior 65)	*/
+/*			because intrq bit 5 in $fffa01 seems to be cleared too late,	*/
+/*			the command takes more time than a real ST to complete.		*/
+/* 2008/10/02	[NP]	FDCTrackRegister, FDCSectorRegister and FDCDataRegister are	*/
+/*			stored on 8 bits in the WD 1772. When writing to $ff8604 to	*/
+/*			access these registers, we must keep only the lower 8 bits.	*/
+/*			(fix High Fidelity Dreams by Aura, writes $fb07 in sector reg).	*/
+/*			TODO : FDCxxxxRegister should use Uint8, not Sint16/Uint16.	*/
+/* 2008/12/15	[NP]	Although the Read Address command is not supported yet, add a	*/
+/*			function FDC_UpdateReadAddressCmd that does nothing, but	*/
+/*			completes with no DMA error and clear bit 6 of $fffa01 (fixes	*/
+/*			loader routine used in various Pompey Pirates compilations (23,	*/
+/*			27, ...) that uses the read address command only to update the	*/
+/*			status register and get the state of the write protection).	*/
 
-const char FDC_rcsid[] = "Hatari $Id: fdc.c,v 1.35 2008/03/13 23:17:27 thothy Exp $";
+
+const char FDC_fileid[] = "Hatari fdc.c : " __DATE__ " " __TIME__;
 
 #include "main.h"
 #include "configuration.h"
@@ -28,10 +52,9 @@ const char FDC_rcsid[] = "Hatari $Id: fdc.c,v 1.35 2008/03/13 23:17:27 thothy Ex
 #include "m68000.h"
 #include "memorySnapShot.h"
 #include "mfp.h"
-#include "misc.h"
 #include "psg.h"
 #include "stMemory.h"
-#include "trace.h"
+#include "video.h"
 
 
 /*
@@ -148,7 +171,7 @@ static Sint16 FDCTrackRegister, FDCSectorRegister, FDCDataRegister;
 static int FDCEmulationCommand;                                 /* FDC emulation command currently being exceuted */
 static int FDCEmulationRunning;                                 /* Running command under above */
 static int FDCStepDirection;                                    /* +Track on 'Step' command */
-static BOOL bDMAWaiting;                                        /* Is DMA waiting to copy? */
+static bool bDMAWaiting;                                        /* Is DMA waiting to copy? */
 static int bMotorOn;                                            /* Is motor on? */
 static int MotorSlowingCount;                                   /* Counter used to slow motor before stopping */
 
@@ -194,7 +217,7 @@ void FDC_Reset(void)
 /**
  * Save/Restore snapshot of local variables('MemorySnapShot_Store' handles type)
  */
-void FDC_MemorySnapShot_Capture(BOOL bSave)
+void FDC_MemorySnapShot_Capture(bool bSave)
 {
 	/* Save/Restore details */
 	MemorySnapShot_Store(&DiskControllerStatus_ff8604rd, sizeof(DiskControllerStatus_ff8604rd));
@@ -273,8 +296,9 @@ void FDC_ResetDMAStatus(void)
 	FDCSectorCountRegister = 0;
 	FDC_SetDMAStatus(FALSE);          /* Set no error */
 
+	/* Reset HDC command status */
 	HDCSectorCount = 0;
-	HDCCommand.byteCount = 0;         /* Reset HDC command status */
+	/*HDCCommand.byteCount = 0;*/  /* Not done on real ST? */
 	HDCCommand.returnCode = 0;
 }
 
@@ -282,7 +306,7 @@ void FDC_ResetDMAStatus(void)
 /*-----------------------------------------------------------------------*/
 /**
  * Set DMA Status (RD 0xff8606)
- * 
+ *
  * NOTE FDC Doc's are incorrect - Bit 0 is '0' on error (See TOS floprd, Ninja III etc...)
  * Look like Atari(yet again) connected the hardware up differently to the spec'
  *
@@ -290,7 +314,7 @@ void FDC_ResetDMAStatus(void)
  * Bit 1 - _Sector Count Zero Status (0=Sector Count Zero)
  * Bit 2 - _Data Request Inactive Status
  */
-void FDC_SetDMAStatus(BOOL bError)
+void FDC_SetDMAStatus(bool bError)
 {
 	DMAStatus_ff8606rd &= 0x1;        /* Clear(except for error) */
 
@@ -327,7 +351,7 @@ void FDC_DmaStatus_ReadWord(void)
 
 /*-----------------------------------------------------------------------*/
 /**
- * 
+ *
  */
 static void FDC_UpdateDiskDrive(void)
 {
@@ -426,7 +450,7 @@ void FDC_GpipRead(void)
 		{
 			/* Restart FDC update interrupt to occur right after a few cycles */
 			Int_RemovePendingInterrupt(INTERRUPT_FDC);
-			Int_AddRelativeInterrupt(4, INT_CPU_CYCLE, INTERRUPT_FDC, 0);
+			Int_AddRelativeInterrupt(4, INT_CPU_CYCLE, INTERRUPT_FDC);
 		}
 	}
 	else
@@ -490,6 +514,10 @@ void FDC_InterruptHandler_Update(void)
 		 case FDCEMU_CMD_WRITEMULTIPLESECTORS:
 			FDC_UpdateWriteSectorsCmd();
 			break;
+
+		 case FDCEMU_CMD_READADDRESS:
+			FDC_UpdateReadAddressCmd();
+			break;
 		}
 
 		/* Set disk controller status (RD 0xff8604) */
@@ -763,6 +791,33 @@ void FDC_UpdateWriteSectorsCmd(void)
 }
 
 
+/*-----------------------------------------------------------------------*/
+/**
+ * Run 'READ ADDRESS' command
+ */
+void FDC_UpdateReadAddressCmd(void)
+{
+	/* Which command is running? */
+	switch (FDCEmulationRunning)
+	{
+	 case FDCEMU_RUN_READADDRESS:
+		/* not implemented, just return with no error */
+		FDCEmulationRunning = FDCEMU_RUN_READADDRESS_COMPLETE;
+		break;
+	 case FDCEMU_RUN_READADDRESS_COMPLETE:
+		/* Acknowledge interrupt, move along there's nothing more to see */
+		FDC_AcknowledgeInterrupt();
+		/* Set error */
+		FDC_SetDMAStatus(FALSE);            /* No DMA error */
+		/* Done */
+		FDCEmulationCommand = FDCEMU_CMD_NULL;
+		/* Turn motor off */
+		FDC_TurnMotorOff();
+		break;
+	}
+}
+
+
 /*-----------------------------------------------------------------------*/
 /**
  * Type I Commands
@@ -774,6 +829,9 @@ void FDC_UpdateWriteSectorsCmd(void)
 /*-----------------------------------------------------------------------*/
 static void FDC_TypeI_Restore(void)
 {
+	HATARI_TRACE ( HATARI_TRACE_FDC , "fdc type I restore VBL=%d video_cyc=%d pc=%x\n" ,
+		nVBLs , Cycles_GetCounter(CYCLES_COUNTER_VIDEO) , M68000_GetPC() );
+
 	/* Set emulation to seek to track zero */
 	FDCEmulationCommand = FDCEMU_CMD_RESTORE;
 	FDCEmulationRunning = FDCEMU_RUN_RESTORE_SEEKTOTRACKZERO;
@@ -785,6 +843,9 @@ static void FDC_TypeI_Restore(void)
 /*-----------------------------------------------------------------------*/
 static void FDC_TypeI_Seek(void)
 {
+	HATARI_TRACE ( HATARI_TRACE_FDC , "fdc type I seek track=0x%x VBL=%d video_cyc=%d pc=%x\n" ,
+		FDCDataRegister , nVBLs , Cycles_GetCounter(CYCLES_COUNTER_VIDEO) , M68000_GetPC() );
+
 	/* Set emulation to seek to chosen track */
 	FDCEmulationCommand = FDCEMU_CMD_SEEK;
 	FDCEmulationRunning = FDCEMU_RUN_SEEK_TOTRACK;
@@ -796,6 +857,9 @@ static void FDC_TypeI_Seek(void)
 /*-----------------------------------------------------------------------*/
 static void FDC_TypeI_Step(void)
 {
+	HATARI_TRACE ( HATARI_TRACE_FDC , "fdc type I step %d VBL=%d video_cyc=%d pc=%x\n" ,
+		FDCStepDirection , nVBLs , Cycles_GetCounter(CYCLES_COUNTER_VIDEO) , M68000_GetPC() );
+
 	/* Set emulation to step(same direction as last seek executed, eg 'FDCStepDirection') */
 	FDCEmulationCommand = FDCEMU_CMD_STEP;
 	FDCEmulationRunning = FDCEMU_RUN_STEP_ONCE;
@@ -807,6 +871,9 @@ static void FDC_TypeI_Step(void)
 /*-----------------------------------------------------------------------*/
 static void FDC_TypeI_StepIn(void)
 {
+	HATARI_TRACE ( HATARI_TRACE_FDC , "fdc type I step in VBL=%d video_cyc=%d pc=%x\n" ,
+		nVBLs , Cycles_GetCounter(CYCLES_COUNTER_VIDEO) , M68000_GetPC() );
+
 	/* Set emulation to step in(Set 'FDCStepDirection') */
 	FDCEmulationCommand = FDCEMU_CMD_STEPIN;
 	FDCEmulationRunning = FDCEMU_RUN_STEPIN_ONCE;
@@ -819,6 +886,9 @@ static void FDC_TypeI_StepIn(void)
 /*-----------------------------------------------------------------------*/
 static void FDC_TypeI_StepOut(void)
 {
+	HATARI_TRACE ( HATARI_TRACE_FDC , "fdc type I step out VBL=%d video_cyc=%d pc=%x\n" ,
+		nVBLs , Cycles_GetCounter(CYCLES_COUNTER_VIDEO) , M68000_GetPC() );
+
 	/* Set emulation to step out(Set 'FDCStepDirection') */
 	FDCEmulationCommand = FDCEMU_CMD_STEPOUT;
 	FDCEmulationRunning = FDCEMU_RUN_STEPOUT_ONCE;
@@ -839,6 +909,9 @@ static void FDC_TypeI_StepOut(void)
 /*-----------------------------------------------------------------------*/
 static void FDC_TypeII_ReadSector(void)
 {
+	HATARI_TRACE ( HATARI_TRACE_FDC , "fdc type II read sector %d VBL=%d video_cyc=%d pc=%x\n" ,
+		FDCSectorRegister , nVBLs , Cycles_GetCounter(CYCLES_COUNTER_VIDEO) , M68000_GetPC() );
+
 	/* Set emulation to read a single sector */
 	FDCEmulationCommand = FDCEMU_CMD_READSECTORS;
 	FDCEmulationRunning = FDCEMU_RUN_READSECTORS_READDATA;
@@ -852,6 +925,9 @@ static void FDC_TypeII_ReadSector(void)
 /*-----------------------------------------------------------------------*/
 static void FDC_TypeII_ReadMultipleSectors(void)
 {
+	HATARI_TRACE ( HATARI_TRACE_FDC , "fdc type II read multi sectors %d count %d VBL=%d video_cyc=%d pc=%x\n" ,
+		FDCSectorRegister , FDCSectorCountRegister , nVBLs , Cycles_GetCounter(CYCLES_COUNTER_VIDEO) , M68000_GetPC() );
+
 	/* Set emulation to read sectors */
 	FDCEmulationCommand = FDCEMU_CMD_READMULTIPLESECTORS;
 	FDCEmulationRunning = FDCEMU_RUN_READSECTORS_READDATA;
@@ -865,6 +941,9 @@ static void FDC_TypeII_ReadMultipleSectors(void)
 /*-----------------------------------------------------------------------*/
 static void FDC_TypeII_WriteSector(void)
 {
+	HATARI_TRACE ( HATARI_TRACE_FDC , "fdc type II write sector %d VBL=%d video_cyc=%d pc=%x\n" ,
+		FDCSectorRegister , nVBLs , Cycles_GetCounter(CYCLES_COUNTER_VIDEO) , M68000_GetPC() );
+
 	/* Set emulation to write a single sector */
 	FDCEmulationCommand = FDCEMU_CMD_WRITESECTORS;
 	FDCEmulationRunning = FDCEMU_RUN_WRITESECTORS_WRITEDATA;
@@ -878,6 +957,9 @@ static void FDC_TypeII_WriteSector(void)
 /*-----------------------------------------------------------------------*/
 static void FDC_TypeII_WriteMultipleSectors(void)
 {
+	HATARI_TRACE ( HATARI_TRACE_FDC , "fdc type II write multi sectors %d count %d VBL=%d video_cyc=%d pc=%x\n" ,
+		FDCSectorRegister , FDCSectorCountRegister , nVBLs , Cycles_GetCounter(CYCLES_COUNTER_VIDEO) , M68000_GetPC() );
+
 	/* Set emulation to write sectors */
 	FDCEmulationCommand = FDCEMU_CMD_WRITEMULTIPLESECTORS;
 	FDCEmulationRunning = FDCEMU_RUN_WRITESECTORS_WRITEDATA;
@@ -899,14 +981,26 @@ static void FDC_TypeII_WriteMultipleSectors(void)
 /*-----------------------------------------------------------------------*/
 static void FDC_TypeIII_ReadAddress(void)
 {
-	Log_Printf(LOG_DEBUG, "FDC type III command 'read address' is not implemented yet!\n");
+	HATARI_TRACE ( HATARI_TRACE_FDC , "fdc type III read address unimplemented VBL=%d video_cyc=%d pc=%x\n" ,
+		nVBLs , Cycles_GetCounter(CYCLES_COUNTER_VIDEO) , M68000_GetPC() );
+
+	Log_Printf(LOG_TODO, "FDC type III command 'read address' is not implemented yet!\n");
+
+	/* Set emulation to seek to track zero */
+	FDCEmulationCommand = FDCEMU_CMD_READADDRESS;
+	FDCEmulationRunning = FDCEMU_RUN_READADDRESS;
+
+	FDC_SetDiskControllerStatus();
 }
 
 
 /*-----------------------------------------------------------------------*/
 static void FDC_TypeIII_ReadTrack(void)
 {
-	Log_Printf(LOG_DEBUG, "FDC type III command 'read track' does not work yet!\n");
+	HATARI_TRACE ( HATARI_TRACE_FDC , "fdc type III read track 0x%x VBL=%d video_cyc=%d pc=%x\n" ,
+		FDCTrackRegister , nVBLs , Cycles_GetCounter(CYCLES_COUNTER_VIDEO) , M68000_GetPC() );
+
+	Log_Printf(LOG_TODO, "FDC type III command 'read track' does not work yet!\n");
 
 	/* FIXME: "Read track" should read more than only the sectors! (also sector headers, gaps, etc.) */
 
@@ -923,7 +1017,10 @@ static void FDC_TypeIII_ReadTrack(void)
 /*-----------------------------------------------------------------------*/
 static void FDC_TypeIII_WriteTrack(void)
 {
-	Log_Printf(LOG_DEBUG, "FDC type III command 'write track' does not work yet!\n");
+	HATARI_TRACE ( HATARI_TRACE_FDC , "fdc type III write track 0x%x VBL=%d video_cyc=%d pc=%x\n" ,
+		FDCTrackRegister , nVBLs , Cycles_GetCounter(CYCLES_COUNTER_VIDEO) , M68000_GetPC() );
+
+	Log_Printf(LOG_TODO, "FDC type III command 'write track' does not work yet!\n");
 
 	/* FIXME: "Write track" not only writes the sectors! (also sector headers, gaps, etc.) */
 
@@ -946,8 +1043,11 @@ static void FDC_TypeIII_WriteTrack(void)
 
 
 /*-----------------------------------------------------------------------*/
-static void FDC_TypeIV_ForceInterrupt(BOOL bCauseCPUInterrupt)
+static void FDC_TypeIV_ForceInterrupt(bool bCauseCPUInterrupt)
 {
+	HATARI_TRACE ( HATARI_TRACE_FDC , "fdc type IV force int VBL=%d video_cyc=%d pc=%x\n" ,
+		nVBLs , Cycles_GetCounter(CYCLES_COUNTER_VIDEO) , M68000_GetPC() );
+
 	/* Acknowledge interrupt, move along there's nothing more to see */
 	if (bCauseCPUInterrupt)
 		FDC_AcknowledgeInterrupt();
@@ -1094,8 +1194,8 @@ static void FDC_ExecuteCommand(void)
  */
 static void FDC_WriteSectorCountRegister(void)
 {
-	HATARI_TRACE ( HATARI_TRACE_FDC , "fdc write 8604 sector count=0x%x video_cyc=%d\n" ,
-		DiskControllerWord_ff8604wr , Cycles_GetCounter(CYCLES_COUNTER_VIDEO) );
+	HATARI_TRACE ( HATARI_TRACE_FDC , "fdc write 8604 sector count=0x%x VBL=%d video_cyc=%d pc=%x\n" ,
+		DiskControllerWord_ff8604wr , nVBLs , Cycles_GetCounter(CYCLES_COUNTER_VIDEO) , M68000_GetPC() );
 
 	FDCSectorCountRegister = DiskControllerWord_ff8604wr;
 }
@@ -1107,8 +1207,8 @@ static void FDC_WriteSectorCountRegister(void)
  */
 static void FDC_WriteCommandRegister(void)
 {
-	HATARI_TRACE ( HATARI_TRACE_FDC , "fdc write 8604 command=0x%x video_cyc=%d\n" ,
-		DiskControllerWord_ff8604wr , Cycles_GetCounter(CYCLES_COUNTER_VIDEO) );
+	HATARI_TRACE ( HATARI_TRACE_FDC , "fdc write 8604 command=0x%x VBL=%d video_cyc=%d pc=%x\n" ,
+		DiskControllerWord_ff8604wr , nVBLs , Cycles_GetCounter(CYCLES_COUNTER_VIDEO) , M68000_GetPC() );
 
 	FDCCommandRegister = DiskControllerWord_ff8604wr;
 	/* And execute */
@@ -1122,10 +1222,10 @@ static void FDC_WriteCommandRegister(void)
  */
 static void FDC_WriteTrackRegister(void)
 {
-	HATARI_TRACE ( HATARI_TRACE_FDC , "fdc write 8604 track=0x%x video_cyc=%d\n" ,
-		DiskControllerWord_ff8604wr , Cycles_GetCounter(CYCLES_COUNTER_VIDEO) );
+	HATARI_TRACE ( HATARI_TRACE_FDC , "fdc write 8604 track=0x%x VBL=%d video_cyc=%d pc=%x\n" ,
+		DiskControllerWord_ff8604wr , nVBLs , Cycles_GetCounter(CYCLES_COUNTER_VIDEO) , M68000_GetPC() );
 
-	FDCTrackRegister = DiskControllerWord_ff8604wr;    /* 0...79 */
+	FDCTrackRegister = DiskControllerWord_ff8604wr & 0xff;	/* 0...79 */
 }
 
 
@@ -1135,10 +1235,10 @@ static void FDC_WriteTrackRegister(void)
  */
 static void FDC_WriteSectorRegister(void)
 {
-	HATARI_TRACE ( HATARI_TRACE_FDC , "fdc write 8604 sector=0x%x video_cyc=%d\n" ,
-		DiskControllerWord_ff8604wr , Cycles_GetCounter(CYCLES_COUNTER_VIDEO) );
+	HATARI_TRACE ( HATARI_TRACE_FDC , "fdc write 8604 sector=0x%x VBL=%d video_cyc=%d pc=%x\n" ,
+		DiskControllerWord_ff8604wr , nVBLs , Cycles_GetCounter(CYCLES_COUNTER_VIDEO) , M68000_GetPC() );
 
-	FDCSectorRegister = DiskControllerWord_ff8604wr;  /* 1,2,3..... */
+	FDCSectorRegister = DiskControllerWord_ff8604wr & 0xff;	/* 1,2,3..... */
 }
 
 
@@ -1148,10 +1248,10 @@ static void FDC_WriteSectorRegister(void)
  */
 static void FDC_WriteDataRegister(void)
 {
-	HATARI_TRACE ( HATARI_TRACE_FDC , "fdc write 8604 data=0x%x video_cyc=%d\n" ,
-		DiskControllerWord_ff8604wr , Cycles_GetCounter(CYCLES_COUNTER_VIDEO) );
+	HATARI_TRACE ( HATARI_TRACE_FDC , "fdc write 8604 data=0x%x VBL=%d video_cyc=%d pc=%x\n" ,
+		DiskControllerWord_ff8604wr , nVBLs , Cycles_GetCounter(CYCLES_COUNTER_VIDEO) , M68000_GetPC() );
 
-	FDCDataRegister = DiskControllerWord_ff8604wr;
+	FDCDataRegister = DiskControllerWord_ff8604wr & 0xff;
 }
 
 
@@ -1172,14 +1272,14 @@ void FDC_DiskController_WriteWord(void)
 
 	DiskControllerWord_ff8604wr = IoMem_ReadWord(0xff8604);
 
-	HATARI_TRACE ( HATARI_TRACE_FDC , "fdc write 8604 data=0x%x video_cyc=%d\n" ,
-		DiskControllerWord_ff8604wr , Cycles_GetCounter(CYCLES_COUNTER_VIDEO) );
+	HATARI_TRACE ( HATARI_TRACE_FDC , "fdc write 8604 data=0x%x VBL=%d video_cyc=%d pc=%x\n" ,
+		DiskControllerWord_ff8604wr , nVBLs , Cycles_GetCounter(CYCLES_COUNTER_VIDEO) , M68000_GetPC() );
 
 	HDC_WriteCommandPacket();                 /*  Handle HDC functions */
 
 	/* filter hdc commands */
 	if ((DMAModeControl_ff8606wr & 0x0018) == 8)
-		return;	
+		return;
 
 	/* Are we trying to set the SectorCount? */
 	if (DMAModeControl_ff8606wr&0x10)         /* Bit 4 */
@@ -1274,8 +1374,8 @@ void FDC_DiskControllerStatus_ReadWord(void)
 
 	IoMem_WriteWord(0xff8604, DiskControllerByte);
 
-	HATARI_TRACE ( HATARI_TRACE_FDC , "fdc read 8604 ctrl status=0x%x video_cyc=%d\n" ,
-		DiskControllerByte , Cycles_GetCounter(CYCLES_COUNTER_VIDEO) );
+	HATARI_TRACE ( HATARI_TRACE_FDC , "fdc read 8604 ctrl status=0x%x VBL=%d video_cyc=%d pc=%x\n" ,
+		DiskControllerByte , nVBLs , Cycles_GetCounter(CYCLES_COUNTER_VIDEO) , M68000_GetPC() );
 }
 
 
@@ -1300,7 +1400,8 @@ Uint32 FDC_ReadDMAAddress(void)
  */
 void FDC_WriteDMAAddress(Uint32 Address)
 {
-	HATARI_TRACE ( HATARI_TRACE_FDC , "fdc write 0x%x to dma address video_cyc=%d\n" , Address , Cycles_GetCounter(CYCLES_COUNTER_VIDEO) );
+	HATARI_TRACE ( HATARI_TRACE_FDC , "fdc write 0x%x to dma address VBL=%d video_cyc=%d pc=%x\n" ,
+		Address , nVBLs , Cycles_GetCounter(CYCLES_COUNTER_VIDEO) , M68000_GetPC() );
 
 	/* Store as 24-bit address */
 	STMemory_WriteByte(0xff8609, Address>>16);
@@ -1314,10 +1415,10 @@ void FDC_WriteDMAAddress(Uint32 Address)
  * Read sector from floppy drive into workspace
  * We copy the bytes in chunks to simulate reading of the floppy using DMA
  */
-BOOL FDC_ReadSectorFromFloppy(void)
+bool FDC_ReadSectorFromFloppy(void)
 {
-	HATARI_TRACE ( HATARI_TRACE_FDC , "fdc read sector dev=%d sect=%d track=%d side=%d video_cyc=%d\n" ,
-		nReadWriteDev, nReadWriteSector, nReadWriteTrack, nReadWriteSide, Cycles_GetCounter(CYCLES_COUNTER_VIDEO) );
+	HATARI_TRACE ( HATARI_TRACE_FDC , "fdc read sector addr=0x%x dev=%d sect=%d track=%d side=%d VBL=%d video_cyc=%d pc=%x\n" ,
+		FDC_ReadDMAAddress(), nReadWriteDev, nReadWriteSector, nReadWriteTrack, nReadWriteSide, nVBLs , Cycles_GetCounter(CYCLES_COUNTER_VIDEO) , M68000_GetPC() );
 
 	/* Copy in 1 sector to our workspace */
 	if (Floppy_ReadSectors(nReadWriteDev, DMASectorWorkSpace, nReadWriteSector, nReadWriteTrack, nReadWriteSide, 1, NULL))
@@ -1343,15 +1444,15 @@ BOOL FDC_ReadSectorFromFloppy(void)
  * Write sector from workspace to floppy drive
  * We copy the bytes in chunks to simulate writing of the floppy using DMA
  */
-BOOL FDC_WriteSectorFromFloppy(void)
+bool FDC_WriteSectorFromFloppy(void)
 {
 	Uint32 Address;
 
 	/* Get DMA address */
 	Address = FDC_ReadDMAAddress();
 
-	HATARI_TRACE ( HATARI_TRACE_FDC , "fdc write sector addr=%x dev=%d sect=%d track=%d side=%d video_cyc=%d\n" ,
-		Address, nReadWriteDev, nReadWriteSector, nReadWriteTrack, nReadWriteSide, Cycles_GetCounter(CYCLES_COUNTER_VIDEO) );
+	HATARI_TRACE ( HATARI_TRACE_FDC , "fdc write sector addr=0x%x dev=%d sect=%d track=%d side=%d VBL=%d video_cyc=%d pc=%x\n" ,
+		Address, nReadWriteDev, nReadWriteSector, nReadWriteTrack, nReadWriteSide, nVBLs , Cycles_GetCounter(CYCLES_COUNTER_VIDEO) , M68000_GetPC() );
 
 	/* Write out 1 sector from our workspace */
 	if (Floppy_WriteSectors(nReadWriteDev, &STRam[Address], nReadWriteSector, nReadWriteTrack, nReadWriteSide, 1, NULL))
@@ -1389,7 +1490,7 @@ void FDC_DMADataFromFloppy(void)
 /*-----------------------------------------------------------------------*/
 /**
  * Write word to 0xff8606 (DMA Mode Control)
- * 
+ *
  * Eg.
  * $80 - Selects command/status register
  * $82 - Selects track register
@@ -1412,7 +1513,8 @@ void FDC_DmaModeControl_WriteWord(void)
 	DMAModeControl_ff8606wr_prev = DMAModeControl_ff8606wr;  /* Store previous to check for _read/_write toggle (DMA reset) */
 	DMAModeControl_ff8606wr = IoMem_ReadWord(0xff8606);      /* Store to DMA Mode control */
 
-	HATARI_TRACE ( HATARI_TRACE_FDC , "fdc write 8606 ctrl=0x%x video_cyc=%d\n" , DMAModeControl_ff8606wr , Cycles_GetCounter(CYCLES_COUNTER_VIDEO) );
+	HATARI_TRACE ( HATARI_TRACE_FDC , "fdc write 8606 ctrl=0x%x VBL=%d video_cyc=%d pc=%x\n" ,
+		DMAModeControl_ff8606wr , nVBLs , Cycles_GetCounter(CYCLES_COUNTER_VIDEO) , M68000_GetPC() );
 
 	/* When write to 0xff8606, check bit '8' toggle. This causes DMA status reset */
 	if ((DMAModeControl_ff8606wr_prev ^ DMAModeControl_ff8606wr) & 0x0100)
