@@ -6,7 +6,7 @@
 
   Common file access functions.
 */
-const char File_rcsid[] = "Hatari $Id: file.c,v 1.51 2008/03/18 00:56:32 thothy Exp $";
+const char File_fileid[] = "Hatari file.c : " __DATE__ " " __TIME__;
 
 #include <config.h>
 
@@ -23,6 +23,10 @@ const char File_rcsid[] = "Hatari $Id: file.c,v 1.51 2008/03/18 00:56:32 thothy
 #include <strings.h>
 #endif
 
+#ifdef WIN32
+#include <winsock2.h>
+#endif
+
 #include "main.h"
 #include "dialog.h"
 #include "file.h"
@@ -72,7 +76,7 @@ void File_AddSlashToEndFileName(char *pszFileName)
 /**
  * Does filename extension match? If so, return TRUE
  */
-BOOL File_DoesFileExtensionMatch(const char *pszFileName, const char *pszExtension)
+bool File_DoesFileExtensionMatch(const char *pszFileName, const char *pszExtension)
 {
 	if (strlen(pszFileName) < strlen(pszExtension))
 		return FALSE;
@@ -91,7 +95,7 @@ BOOL File_DoesFileExtensionMatch(const char *pszFileName, const char *pszExtensi
  * 
  * Return TRUE if filename is '/', else give FALSE
  */
-static BOOL File_IsRootFileName(const char *pszFileName)
+static bool File_IsRootFileName(const char *pszFileName)
 {
 	if (pszFileName[0] == '\0')     /* If NULL string return! */
 		return FALSE;
@@ -100,7 +104,6 @@ static BOOL File_IsRootFileName(const char *pszFileName)
 		return TRUE;
 
 #ifdef WIN32
-
 	if (pszFileName[1] == ':')
 		return TRUE;
 #endif
@@ -122,13 +125,26 @@ const char *File_RemoveFileNameDrive(const char *pszFileName)
 }
 
 
+/*-----------------------------------------------------------------------*/
+/**
+ * Check if given filename is an existing directory
+ * 
+ * Return TRUE if directory, else give FALSE
+ */
+bool File_DirExists(const char *path)
+{
+	struct stat filestat;
+	return (stat(path, &filestat) == 0 && S_ISDIR(filestat.st_mode));
+}
+	
+
 /*-----------------------------------------------------------------------*/
 /**
  * Check if filename end with a '/'
  * 
  * Return TRUE if filename ends with '/'
  */
-BOOL File_DoesFileNameEndWithSlash(char *pszFileName)
+bool File_DoesFileNameEndWithSlash(char *pszFileName)
 {
 	if (pszFileName[0] == '\0')    /* If NULL string return! */
 		return FALSE;
@@ -225,9 +241,9 @@ Uint8 *File_Read(const char *pszFileName, long *pFileSize, const char * const pp
 /**
  * Save file to disk, return FALSE if errors
  */
-BOOL File_Save(const char *pszFileName, const Uint8 *pAddress, size_t Size, BOOL bQueryOverwrite)
+bool File_Save(const char *pszFileName, const Uint8 *pAddress, size_t Size, bool bQueryOverwrite)
 {
-	BOOL bRet = FALSE;
+	bool bRet = FALSE;
 
 	/* Check if need to ask user if to overwrite */
 	if (bQueryOverwrite)
@@ -299,7 +315,7 @@ int File_Length(const char *pszFileName)
  * Return TRUE if file exists, is readable or writable at least and is not
  * a directory. 
  */
-BOOL File_Exists(const char *filename)
+bool File_Exists(const char *filename)
 {
 	struct stat buf;
 	if (stat(filename, &buf) == 0 &&
@@ -316,7 +332,7 @@ BOOL File_Exists(const char *filename)
 /**
  * Return TRUE if directory exists. 
  */
-BOOL File_DirectoryExists(const char *psDirName)
+bool File_DirectoryExists(const char *psDirName)
 {
 	struct stat buf;
 	if (stat(psDirName, &buf) == 0 && (buf.st_mode & S_IFDIR))
@@ -331,11 +347,11 @@ BOOL File_DirectoryExists(const char *psDirName)
 /**
  * Find if file exists, and if so ask user if OK to overwrite
  */
-BOOL File_QueryOverwrite(const char *pszFileName)
+bool File_QueryOverwrite(const char *pszFileName)
 {
 	const char *fmt;
 	char *szString;
-	BOOL ret = TRUE;
+	bool ret = TRUE;
 
 	/* Try and find if file exists */
 	if (File_Exists(pszFileName))
@@ -518,6 +534,10 @@ FILE *File_Open(const char *path, const char *mode)
 	int wr = 0, rd = 0;
 	FILE *fp;
 
+	/* empty name signifies file that shouldn't be opened/enabled */
+	if (!*path)
+		return NULL;
+	
 	/* special "stdout" and "stderr" files can be used
 	 * for files which are written or appended
 	 */
@@ -564,16 +584,49 @@ FILE *File_Close(FILE *fp)
 }
 
 
+/*-----------------------------------------------------------------------*/
+/**
+ * Check if input is available at the specified file descriptor.
+ */
+bool File_InputAvailable(FILE *fp)
+{
+	fd_set rfds;
+	struct timeval tv;
+	int fh;
+	int ret;
+
+	if (!fp || (fh = fileno(fp)) == -1)
+		return false;
+
+	/* Add the file handle to the file descriptor set */
+	FD_ZERO(&rfds);
+	FD_SET(fh, &rfds);
+
+	/* Return immediately */
+	tv.tv_sec = 0;
+	tv.tv_usec = 0;
+
+	/* Check if file descriptor is ready for a read */
+	ret = select(fh+1, &rfds, NULL, NULL, &tv);
+
+	if (ret > 0)
+		return true;    /* Data available */
+
+	return false;
+}
+
+
 /*-----------------------------------------------------------------------*/
 /**
  * Wrapper for File_MakeAbsoluteName() which special-cases stdin/out/err
- * named files.  The given buffer should be opened with File_Open()
- * and closed with File_Close() if this function is used!
+ * named files and empty file name.  The given buffer should be opened
+ * with File_Open() and closed with File_Close() if this function is used!
  * (On Linux one can use /dev/stdout etc, this is intended for other OSes)
  */
 void File_MakeAbsoluteSpecialName(char *path)
 {
-	if (strcmp(path, "stdin")  != 0 &&
+	if (path[0] &&
+	    strcmp(path, "stdin")  != 0 &&
 	    strcmp(path, "stdout") != 0 &&
 	    strcmp(path, "stderr") != 0)
 		File_MakeAbsoluteName(path);
