@@ -4,10 +4,10 @@
   This file is distributed under the GNU Public License, version 2 or at
   your option any later version. Read the file gpl.txt for details.
 
-  This where we read/write sectors to/from the disk image buffers. NOTE these
-  buffers are in memory so we only need to write routines for the .ST format.
-  When the buffer is to be saved (ie eject disk) we save it back to the original
-  file in the correct format (.ST or .MSA).
+  This is where we read/write sectors to/from the disk image buffers.
+  NOTE: these buffers are in memory so we only need to write routines for
+  the .ST format. When the buffer is to be saved (ie eject disk) we save
+  it back to the original file in the correct format (.ST or .MSA).
 
   There are some important notes about image accessing - as we use TOS and the
   FDC to access the disk the boot-sector MUST be valid. Sometimes this is NOT
@@ -21,10 +21,10 @@
   (PaCifiST will, however, read/write to these images as it does not perform
   FDC access as on a real ST)
 */
-const char Floppy_rcsid[] = "Hatari $Id: floppy.c,v 1.35 2008/01/07 21:54:22 eerot Exp $";
+const char Floppy_fileid[] = "Hatari floppy.c : " __DATE__ " " __TIME__;
 
 #include <sys/stat.h>
-
+#include <assert.h>
 #include <SDL_endian.h>
 
 #include "main.h"
@@ -41,8 +41,10 @@ const char Floppy_rcsid[] = "Hatari $Id: floppy.c,v 1.35 2008/01/07 21:54:22 eer
 #include "zip.h"
 
 
-EMULATION_DRIVE EmulationDrives[NUM_EMULATION_DRIVES];  /* Emulation drive details, eg FileName, Inserted, Changed etc... */
-int nBootDrive=0;               /* Drive A, default */
+/* Emulation drive details, eg FileName, Inserted, Changed etc... */
+EMULATION_DRIVE EmulationDrives[MAX_FLOPPYDRIVES];
+/* Drive A is the default */
+int nBootDrive = 0;
 
 /* Possible disk image file extensions to scan for */
 static const char * const pszDiskImageNameExts[] =
@@ -54,6 +56,10 @@ static const char * const pszDiskImageNameExts[] =
 };
 
 
+/* local functions */
+static bool Floppy_EjectBothDrives(void);
+
+
 /*-----------------------------------------------------------------------*/
 /**
  * Initialize emulation floppy drives
@@ -63,10 +69,12 @@ void Floppy_Init(void)
 	int i;
 
 	/* Clear drive structures */
-	for (i = 0; i < NUM_EMULATION_DRIVES; i++)
+	for (i = 0; i < MAX_FLOPPYDRIVES; i++)
 	{
-		/* Clear */
+		/* Clear structs and if floppies available, insert them */
 		memset(&EmulationDrives[i], 0, sizeof(EMULATION_DRIVE));
+		if (strlen(ConfigureParams.DiskImage.szDiskFileName[i]) > 0)
+			Floppy_InsertDiskIntoDrive(i);
 	}
 }
 
@@ -85,7 +93,7 @@ void Floppy_UnInit(void)
 /**
  * Save/Restore snapshot of local variables('MemorySnapShot_Store' handles type)
  */
-void Floppy_MemorySnapShot_Capture(BOOL bSave)
+void Floppy_MemorySnapShot_Capture(bool bSave)
 {
 	int i;
 
@@ -94,7 +102,7 @@ void Floppy_MemorySnapShot_Capture(BOOL bSave)
 		Floppy_EjectBothDrives();
 
 	/* Save/Restore details */
-	for (i = 0; i < NUM_EMULATION_DRIVES; i++)
+	for (i = 0; i < MAX_FLOPPYDRIVES; i++)
 	{
 		MemorySnapShot_Store(&EmulationDrives[i].bDiskInserted, sizeof(EmulationDrives[i].bDiskInserted));
 		MemorySnapShot_Store(&EmulationDrives[i].nImageBytes, sizeof(EmulationDrives[i].nImageBytes));
@@ -106,7 +114,7 @@ void Floppy_MemorySnapShot_Capture(BOOL bSave)
 		}
 		if (EmulationDrives[i].pBuffer)
 			MemorySnapShot_Store(EmulationDrives[i].pBuffer, EmulationDrives[i].nImageBytes);
-		MemorySnapShot_Store(EmulationDrives[i].szFileName, sizeof(EmulationDrives[i].szFileName));
+		MemorySnapShot_Store(EmulationDrives[i].sFileName, sizeof(EmulationDrives[i].sFileName));
 		MemorySnapShot_Store(&EmulationDrives[i].bMediaChanged,sizeof(EmulationDrives[i].bMediaChanged));
 		MemorySnapShot_Store(&EmulationDrives[i].bContentsChanged,sizeof(EmulationDrives[i].bContentsChanged));
 		MemorySnapShot_Store(&EmulationDrives[i].bOKToSave,sizeof(EmulationDrives[i].bOKToSave));
@@ -134,7 +142,7 @@ void Floppy_GetBootDrive(void)
  * in the GUI. When set to "automatic", we check the file permissions of the
  * floppy disk image to decide.
  */
-BOOL Floppy_IsWriteProtected(int Drive)
+bool Floppy_IsWriteProtected(int Drive)
 {
 	if (ConfigureParams.DiskImage.nWriteProtection == WRITEPROT_OFF)
 	{
@@ -148,7 +156,8 @@ BOOL Floppy_IsWriteProtected(int Drive)
 	{
 		struct stat FloppyStat;
 		/* Check whether disk is writable */
-		if (stat(EmulationDrives[Drive].szFileName, &FloppyStat) == 0 && (FloppyStat.st_mode & S_IWUSR))
+		if (stat(EmulationDrives[Drive].sFileName, &FloppyStat) == 0
+		    && (FloppyStat.st_mode & S_IWUSR))
 			return FALSE;
 		else
 			return TRUE;
@@ -165,7 +174,7 @@ BOOL Floppy_IsWriteProtected(int Drive)
  * To try and prevent data loss, we check for this error and flag the drive so
  * the image will not be saved back to the file.
  */
-static BOOL Floppy_IsBootSectorOK(int Drive)
+static bool Floppy_IsBootSectorOK(int Drive)
 {
 	Uint8 *pDiskBuffer;
 
@@ -195,11 +204,13 @@ static BOOL Floppy_IsBootSectorOK(int Drive)
 /**
  * Try to create disk B filename, eg 'auto_100a' becomes 'auto_100b'
  * Return new filename if think we should try, otherwise NULL
+ *
+ * TODO: doesn't work with images in ZIP archives
  */
 static char* Floppy_CreateDiskBFileName(const char *pSrcFileName)
 {
 	char *szDir, *szName, *szExt;
-	
+
 	/* Allocate temporary memory for strings: */
 	szDir = malloc(3 * FILENAME_MAX);
 	if (!szDir)
@@ -216,12 +227,13 @@ static char* Floppy_CreateDiskBFileName(const char *pSrcFileName)
 	/* All OK? */
 	if (strlen(szName) > 0)
 	{
-		/* Now, did filename end with an 'A' or 'a'? */
-		if ((szName[strlen(szName)-1]=='A') || (szName[strlen(szName)-1]=='a'))
+		char *last = &(szName[strlen(szName)-1]);
+		/* Now, did filename end with an 'A' or 'a' */
+		if (*last == 'A' || *last == 'a')
 		{
 			char *szFull;
 			/* Change 'A' to a 'B' */
-			szName[strlen(szName)-1] += 1;
+			*last += 1;
 			/* And re-build name into destination */
 			szFull = File_MakePath(szDir, szName, szExt);
 			if (szFull)
@@ -243,44 +255,93 @@ static char* Floppy_CreateDiskBFileName(const char *pSrcFileName)
 
 /*-----------------------------------------------------------------------*/
 /**
- * Insert disk into floppy drive. Sets the disk image name
- * back to given string with corrected extension. Maxlen is
- * maximum string lenght.
- * Returns TRUE of success and FALSE for failure.
- * The WHOLE image is copied into our drive buffers,
- * and uncompressed if necessary
+ * Set floppy image to be ejected
  */
-BOOL Floppy_InsertDiskIntoDrive(int Drive, char *pszFileName, int maxlen)
+const char* Floppy_SetDiskFileNameNone(int Drive)
 {
-	char *path;
-	path = Floppy_ZipInsertDiskIntoDrive(Drive, pszFileName, NULL);
-	if (path)
-	{
-		strncpy(pszFileName, path, maxlen);
-		pszFileName[maxlen-1] = '\0';
-		free(path);
-		return TRUE;
-	}
-	return FALSE;
+	assert(Drive >= 0 && Drive < MAX_FLOPPYDRIVES);
+	ConfigureParams.DiskImage.szDiskFileName[Drive][0] = '\0';
+	return ConfigureParams.DiskImage.szDiskFileName[Drive];
 }
 
-/* Read image into buffers, handles different image extensions,
- * return corrected file name on success and NULL on failure.
+/*-----------------------------------------------------------------------*/
+/**
+ * Set given floppy drive image file name and handle
+ * different image extensions.
+ * Return corrected file name on success and NULL on failure.
  */
-char* Floppy_ZipInsertDiskIntoDrive(int Drive, const char *pszFileName, const char *pszZipPath)
+const char* Floppy_SetDiskFileName(int Drive, const char *pszFileName, const char *pszZipPath)
+{
+	char *filename;
+
+	/* setting to empty or "none" ejects */
+	if (!*pszFileName || strcasecmp(pszFileName, "none") == 0)
+	{
+		return Floppy_SetDiskFileNameNone(Drive);
+	}
+	/* See if file exists, and if not, get/add correct extension */
+	if (!File_Exists(pszFileName))
+		filename = File_FindPossibleExtFileName(pszFileName, pszDiskImageNameExts);
+	else
+		filename = strdup(pszFileName);
+	if (!filename)
+	{
+		Log_AlertDlg(LOG_INFO, "Image '%s' not found", pszFileName);
+		return NULL;
+	}
+
+	/* If we insert a disk into Drive A, should we try to put disk 2 into drive B? */
+	if (Drive == 0 && ConfigureParams.DiskImage.bAutoInsertDiskB)
+	{
+		/* Attempt to make up second filename, eg was 'auto_100a' to 'auto_100b' */
+		char *szDiskBFileName = Floppy_CreateDiskBFileName(filename);
+		if (szDiskBFileName)
+		{
+			/* recurse with Drive B */
+			Floppy_SetDiskFileName(1, szDiskBFileName, pszZipPath);
+			free(szDiskBFileName);
+		}
+	}
+
+	/* do the changes */
+	assert(Drive >= 0 && Drive < MAX_FLOPPYDRIVES);
+	if (pszZipPath)
+		strcpy(ConfigureParams.DiskImage.szDiskZipPath[Drive], pszZipPath);
+	else
+		ConfigureParams.DiskImage.szDiskZipPath[Drive][0] = '\0';
+	strcpy(ConfigureParams.DiskImage.szDiskFileName[Drive], filename);
+	free(filename);
+	//File_MakeAbsoluteName(ConfigureParams.DiskImage.szDiskFileName[Drive]);
+	return ConfigureParams.DiskImage.szDiskFileName[Drive];
+}
+
+/*-----------------------------------------------------------------------*/
+/**
+ * Insert previously set disk file image into floppy drive.
+ * The WHOLE image is copied into Hatari drive buffers, and
+ * uncompressed if necessary.
+ * Return TRUE on success, false otherwise.
+ */
+bool Floppy_InsertDiskIntoDrive(int Drive)
 {
 	long nImageBytes = 0;
 	char *filename;
 
 	/* Eject disk, if one is inserted (doesn't inform user) */
-	Floppy_EjectDiskFromDrive(Drive,FALSE);
+	assert(Drive >= 0 && Drive < MAX_FLOPPYDRIVES);
+	Floppy_EjectDiskFromDrive(Drive);
+
+	filename = ConfigureParams.DiskImage.szDiskFileName[Drive];
+	if (!filename[0])
+	{
+		return TRUE; /* only do eject */
+	}
+	if (!File_Exists(filename))
+	{
+		Log_AlertDlg(LOG_INFO, "Image '%s' not found", filename);
+		return FALSE;
+	}
 
-	/* See if file exists, and if not, get/add correct extension */
-	if ( !File_Exists(pszFileName) )
-		filename = File_FindPossibleExtFileName(pszFileName, pszDiskImageNameExts);
-	else
-		filename = strdup(pszFileName);
-	
 	/* Check disk image type and read the file: */
 	if (MSA_FileNameIsMSA(filename, TRUE))
 		EmulationDrives[Drive].pBuffer = MSA_ReadDisk(filename, &nImageBytes);
@@ -289,38 +350,28 @@ char* Floppy_ZipInsertDiskIntoDrive(int Drive, const char *pszFileName, const ch
 	else if (DIM_FileNameIsDIM(filename, TRUE))
 		EmulationDrives[Drive].pBuffer = DIM_ReadDisk(filename, &nImageBytes);
 	else if (ZIP_FileNameIsZIP(filename))
-		EmulationDrives[Drive].pBuffer = ZIP_ReadDisk(filename, pszZipPath, &nImageBytes);
+	{
+		const char *zippath = ConfigureParams.DiskImage.szDiskZipPath[Drive];
+		EmulationDrives[Drive].pBuffer = ZIP_ReadDisk(filename, zippath, &nImageBytes);
+	}
 
 	if (EmulationDrives[Drive].pBuffer == NULL)
 	{
-		free(filename);
-		return NULL;
+		return FALSE;
 	}
-	/* Store filename and size */
-	strcpy(EmulationDrives[Drive].szFileName, filename);
+
+	/* Store image filename (required for ejecting the disk later!) */
+	strcpy(EmulationDrives[Drive].sFileName, filename);
+
+	/* Store size and set drive states */
 	EmulationDrives[Drive].nImageBytes = nImageBytes;
-	/* And set drive states */
 	EmulationDrives[Drive].bDiskInserted = TRUE;
 	EmulationDrives[Drive].bContentsChanged = FALSE;
 	EmulationDrives[Drive].bMediaChanged = TRUE;
 	EmulationDrives[Drive].bOKToSave = Floppy_IsBootSectorOK(Drive);
-
-	/* If we insert a disk into Drive A, should we try to put disk 2 into drive B? */
-	if (Drive == 0 && ConfigureParams.DiskImage.bAutoInsertDiskB)
-	{
-		char *szTmp;
-		/* Attempt to make up second filename, eg was 'auto_100a' to 'auto_100b' */
-		char *szDiskBFileName =  Floppy_CreateDiskBFileName(filename);
-		if (szDiskBFileName)
-		{
-			/* recurse with Drive B */
-			szTmp = Floppy_ZipInsertDiskIntoDrive(1, szDiskBFileName, NULL);
-			if (szTmp)
-				free(szTmp);
-			free(szDiskBFileName);
-		}
-	}
-	return filename;
+	Log_Printf(LOG_INFO, "Inserted disk '%s' to drive %c:.",
+		   filename, 'A'+Drive);
+	return TRUE;
 }
 
 
@@ -328,35 +379,41 @@ char* Floppy_ZipInsertDiskIntoDrive(int Drive, const char *pszFileName, const ch
 /**
  * Eject disk from floppy drive, save contents back to PCs hard-drive if
  * they have been changed.
+ * Return TRUE if there was something to eject.
  */
-void Floppy_EjectDiskFromDrive(int Drive, BOOL bInformUser)
+bool Floppy_EjectDiskFromDrive(int Drive)
 {
+	bool bEjected = FALSE;
+
 	/* Does our drive have a disk in? */
 	if (EmulationDrives[Drive].bDiskInserted)
 	{
+		char *psFileName = EmulationDrives[Drive].sFileName;
+
 		/* OK, has contents changed? If so, need to save */
 		if (EmulationDrives[Drive].bContentsChanged)
 		{
 			/* Is OK to save image (if boot-sector is bad, don't allow a save) */
 			if (EmulationDrives[Drive].bOKToSave && !Floppy_IsWriteProtected(Drive))
 			{
+				Log_Printf(LOG_INFO, "Flush contents to floppy image '%s'.", psFileName);
 				/* Save as .MSA or .ST image? */
-				if (MSA_FileNameIsMSA(EmulationDrives[Drive].szFileName, TRUE))
-					MSA_WriteDisk(EmulationDrives[Drive].szFileName, EmulationDrives[Drive].pBuffer, EmulationDrives[Drive].nImageBytes);
-				else if (ST_FileNameIsST(EmulationDrives[Drive].szFileName, TRUE))
-					ST_WriteDisk(EmulationDrives[Drive].szFileName, EmulationDrives[Drive].pBuffer, EmulationDrives[Drive].nImageBytes);
-				else if (DIM_FileNameIsDIM(EmulationDrives[Drive].szFileName, TRUE))
-					DIM_WriteDisk(EmulationDrives[Drive].szFileName, EmulationDrives[Drive].pBuffer, EmulationDrives[Drive].nImageBytes);
-				else if (ZIP_FileNameIsZIP(EmulationDrives[Drive].szFileName))
-					ZIP_WriteDisk(EmulationDrives[Drive].szFileName, EmulationDrives[Drive].pBuffer, EmulationDrives[Drive].nImageBytes);
+				if (MSA_FileNameIsMSA(psFileName, TRUE))
+					MSA_WriteDisk(psFileName, EmulationDrives[Drive].pBuffer, EmulationDrives[Drive].nImageBytes);
+				else if (ST_FileNameIsST(psFileName, TRUE))
+					ST_WriteDisk(psFileName, EmulationDrives[Drive].pBuffer, EmulationDrives[Drive].nImageBytes);
+				else if (DIM_FileNameIsDIM(psFileName, TRUE))
+					DIM_WriteDisk(psFileName, EmulationDrives[Drive].pBuffer, EmulationDrives[Drive].nImageBytes);
+				else if (ZIP_FileNameIsZIP(psFileName))
+					ZIP_WriteDisk(psFileName, EmulationDrives[Drive].pBuffer, EmulationDrives[Drive].nImageBytes);
 			}
 		}
 
 		/* Inform user that disk has been ejected! */
-		if (bInformUser)
-		{
-			Log_AlertDlg(LOG_INFO, "Disk has been removed from drive '%c:'.", 'A'+Drive);
-		}
+		Log_Printf(LOG_INFO, "Floppy %c: has been removed from drive.",
+			   'A'+Drive);
+
+		bEjected = TRUE;
 	}
 
 	/* Drive is now empty */
@@ -365,23 +422,31 @@ void Floppy_EjectDiskFromDrive(int Drive, BOOL bInformUser)
 		free(EmulationDrives[Drive].pBuffer);
 		EmulationDrives[Drive].pBuffer = NULL;
 	}
-	strcpy(EmulationDrives[Drive].szFileName,"");
+
+	EmulationDrives[Drive].sFileName[0] = '\0';
 	EmulationDrives[Drive].nImageBytes = 0;
 	EmulationDrives[Drive].bDiskInserted = FALSE;
 	EmulationDrives[Drive].bContentsChanged = FALSE;
 	EmulationDrives[Drive].bOKToSave = FALSE;
+
+	return bEjected;
 }
 
 
 /*-----------------------------------------------------------------------*/
 /**
  * Eject all disk image from floppy drives - call when quit.
+ * Return TRUE if there was something to eject.
  */
-void Floppy_EjectBothDrives(void)
+static bool Floppy_EjectBothDrives(void)
 {
+	bool bEjectedA, bEjectedB;
+
 	/* Eject disk images from drives 'A' and 'B' */
-	Floppy_EjectDiskFromDrive(0, FALSE);
-	Floppy_EjectDiskFromDrive(1, FALSE);
+	bEjectedA = Floppy_EjectDiskFromDrive(0);
+	bEjectedB = Floppy_EjectDiskFromDrive(1);
+
+	return bEjectedA || bEjectedB;
 }
 
 
@@ -433,7 +498,7 @@ static void Floppy_DoubleCheckFormat(long nDiskSize, Uint16 *pnSides, Uint16 *pn
  * boot sector and this attempts to find the correct values.
  */
 void Floppy_FindDiskDetails(const Uint8 *pBuffer, int nImageBytes,
-                            unsigned short *pnSectorsPerTrack, unsigned short *pnSides)
+                            Uint16 *pnSectorsPerTrack, Uint16 *pnSides)
 {
 	Uint16 nSectorsPerTrack, nSides, nSectors;
 
@@ -461,12 +526,12 @@ void Floppy_FindDiskDetails(const Uint8 *pBuffer, int nImageBytes,
  * Read sectors from floppy disk image, return TRUE if all OK
  * NOTE Pass -ve as Count to read whole track
  */
-BOOL Floppy_ReadSectors(int Drive, Uint8 *pBuffer, unsigned short Sector,
-                        unsigned short Track, unsigned short Side, short Count,
+bool Floppy_ReadSectors(int Drive, Uint8 *pBuffer, Uint16 Sector,
+                        Uint16 Track, Uint16 Side, short Count,
                         int *pnSectorsPerTrack)
 {
 	Uint8 *pDiskBuffer;
-	unsigned short int nSectorsPerTrack,nSides,nBytesPerTrack;
+	Uint16 nSectorsPerTrack, nSides, nBytesPerTrack;
 	long Offset;
 	int nImageTracks;
 
@@ -540,12 +605,12 @@ BOOL Floppy_ReadSectors(int Drive, Uint8 *pBuffer, unsigned short Sector,
  * Write sectors from floppy disk image, return TRUE if all OK
  * NOTE Pass -ve as Count to write whole track
  */
-BOOL Floppy_WriteSectors(int Drive, Uint8 *pBuffer, unsigned short Sector,
-                         unsigned short Track, unsigned short Side, short Count,
+bool Floppy_WriteSectors(int Drive, Uint8 *pBuffer, Uint16 Sector,
+                         Uint16 Track, Uint16 Side, short Count,
                          int *pnSectorsPerTrack)
 {
 	Uint8 *pDiskBuffer;
-	unsigned short int nSectorsPerTrack,nSides,nBytesPerTrack;
+	Uint16 nSectorsPerTrack, nSides, nBytesPerTrack;
 	long Offset;
 	int nImageTracks;
 
