@@ -18,21 +18,16 @@
   * rmdir routine, can't remove dir with files in it. (another tos/unix difference)
   * Fix bugs, there are probably a few lurking around in here..
 */
-const char Gemdos_rcsid[] = "Hatari $Id: gemdos.c,v 1.69 2008/01/09 23:34:29 thothy Exp $";
+const char Gemdos_fileid[] = "Hatari gemdos.c : " __DATE__ " " __TIME__;
 
 #include <config.h>
 
-#include <string.h>
 #include <sys/stat.h>
 #include <time.h>
 #include <ctype.h>
 #include <unistd.h>
 #include <errno.h>
 
-#if HAVE_STRINGS_H
-#include <strings.h>
-#endif
-
 #if HAVE_GLOB_H
 #include <glob.h>
 #endif
@@ -49,19 +44,15 @@ const char Gemdos_rcsid[] = "Hatari $Id: gemdos.c,v 1.69 2008/01/09 23:34:29 tho
 #include "log.h"
 #include "m68000.h"
 #include "memorySnapShot.h"
-#include "misc.h"
 #include "printer.h"
 #include "rs232.h"
+#include "statusbar.h"
 #include "scandir.h"
 #include "stMemory.h"
+#include "str.h"
 #include "hatari-glue.h"
 #include "maccess.h"
 
-
-//#define GEMDOS_VERBOSE
-// uncomment the following line to debug filename lookups on hd
-// #define GEMDOS_FILE_DEBUG 1
-
 #define ENABLE_SAVING             /* Turn on saving stuff */
 
 #define INVALID_HANDLE_VALUE -1
@@ -79,7 +70,7 @@ const char Gemdos_rcsid[] = "Hatari $Id: gemdos.c,v 1.69 2008/01/09 23:34:29 tho
 
 
 /* Have we re-directed GemDOS vector to our own routines yet? */
-BOOL bInitGemDOS;
+bool bInitGemDOS;
 
 /* structure with all the drive-specific data for our emulated drives,
  * used by GEMDOS_EMU_ON macro
@@ -123,14 +114,14 @@ typedef struct {
 
 typedef struct
 {
-	BOOL bUsed;
+	bool bUsed;
 	FILE *FileHandle;
 	char szActualName[MAX_GEMDOS_PATH];        /* used by F_DATIME (0x57) */
 } FILE_HANDLE;
 
 typedef struct
 {
-	BOOL bUsed;
+	bool bUsed;
 	int  nentries;                      /* number of entries in fs directory */
 	int  centry;                        /* current entry # */
 	struct dirent **found;              /* legal files */
@@ -143,9 +134,10 @@ static int DTAIndex;        /* Circular index into above */
 static DTA *pDTA;           /* Our GEMDOS hard drive Disk Transfer Address structure */
 static Uint16 CurrentDrive; /* Current drive (0=A,1=B,2=C etc...) */
 static Uint32 act_pd;       /* Used to get a pointer to the current basepage */
+static Uint16 nAttrSFirst;  /* File attribute for SFirst/Snext */
 
 
-#ifdef GEMDOS_VERBOSE
+#ifdef HATARI_TRACE_ACTIVATED
 /* List of GEMDos functions... */
 static const char *pszGemDOSNames[] =
 {
@@ -318,7 +310,7 @@ static Uint16 GemDOS_Date2dos(time_t t)
 /**
  * Populate a DATETIME structure with file info
  */
-static BOOL GemDOS_GetFileInformation(char *name, DATETIME *DateTime)
+static bool GemDOS_GetFileInformation(char *name, DATETIME *DateTime)
 {
 	struct stat filestat;
 	int n;
@@ -371,32 +363,40 @@ static unsigned char GemDOS_ConvertAttribute(mode_t mode)
 
 /*-----------------------------------------------------------------------*/
 /**
- * Populate the DTA buffer with file info
+ * Populate the DTA buffer with file info.
+ * @return   0 if entry is ok, 1 if entry should be skipped, < 0 for errors.
  */
-static BOOL PopulateDTA(char *path, struct dirent *file)
+static int PopulateDTA(char *path, struct dirent *file)
 {
 	char tempstr[MAX_GEMDOS_PATH];
 	struct stat filestat;
 	int n;
+	int nFileAttr;
 
 	snprintf(tempstr, sizeof(tempstr), "%s%c%s", path, PATHSEP, file->d_name);
 	n = stat(tempstr, &filestat);
 	if (n != 0)
 	{
 		perror(tempstr);
-		return FALSE;   /* return on error */
+		return -1;   /* return on error */
 	}
 
 	if (!pDTA)
-		return FALSE;   /* no DTA pointer set */
-	Misc_strupr(file->d_name);    /* convert to atari-style uppercase */
+		return -2;   /* no DTA pointer set */
+
+	/* Check file attributes (check is done according to the Profibuch */
+	nFileAttr = GemDOS_ConvertAttribute(filestat.st_mode);
+	if (nFileAttr != 0 && !((nAttrSFirst|0x21) & nFileAttr))
+		return 1;
+
+	Str_ToUpper(file->d_name);    /* convert to atari-style uppercase */
 	strncpy(pDTA->dta_name,file->d_name,TOS_NAMELEN); /* FIXME: better handling of long file names */
 	do_put_mem_long(pDTA->dta_size, filestat.st_size);
 	do_put_mem_word(pDTA->dta_time, GemDOS_Time2dos(filestat.st_mtime));
 	do_put_mem_word(pDTA->dta_date, GemDOS_Date2dos(filestat.st_mtime));
-	pDTA->dta_attrib = GemDOS_ConvertAttribute(filestat.st_mode);
+	pDTA->dta_attrib = nFileAttr;
 
-	return TRUE;
+	return 0;
 }
 
 
@@ -634,14 +634,39 @@ void GemDOS_UnInitDrives(void)
 /**
  * Save/Restore snapshot of local variables('MemorySnapShot_Store' handles type)
  */
-void GemDOS_MemorySnapShot_Capture(BOOL bSave)
+void GemDOS_MemorySnapShot_Capture(bool bSave)
 {
 	unsigned int Addr;
 	int i;
+	bool bEmudrivesAvailable;
+
+	/* Save/Restore the emudrives structure */
+	bEmudrivesAvailable = (emudrives != NULL);
+	MemorySnapShot_Store(&bEmudrivesAvailable, sizeof(bEmudrivesAvailable));
+	if (bEmudrivesAvailable)
+	{
+		if (!bSave && !emudrives)
+		{
+			/* We're loading a memory snapshot, but the emudrives
+			 * structure has not been malloc yet... let's do it now! */
+			GemDOS_InitDrives();
+		}
+
+		for(i=0; i<MAX_HARDDRIVES; i++)
+		{
+			MemorySnapShot_Store(emudrives[i]->hd_emulation_dir,
+			                     sizeof(emudrives[i]->hd_emulation_dir));
+			MemorySnapShot_Store(emudrives[i]->fs_currpath,
+			                     sizeof(emudrives[i]->fs_currpath));
+			MemorySnapShot_Store(&emudrives[i]->hd_letter,
+			                     sizeof(emudrives[i]->hd_letter));
+		}
+	}
 
 	/* Save/Restore details */
 	MemorySnapShot_Store(&DTAIndex,sizeof(DTAIndex));
 	MemorySnapShot_Store(&bInitGemDOS,sizeof(bInitGemDOS));
+	MemorySnapShot_Store(&act_pd, sizeof(act_pd));
 	if (bSave)
 	{
 		Addr = ((Uint8 *)pDTA - STRam);
@@ -690,9 +715,9 @@ static int GemDOS_FindFreeFileHandle(void)
 /**
  * Check ST handle is within our table range, return TRUE if not
  */
-static BOOL GemDOS_IsInvalidFileHandle(int Handle)
+static bool GemDOS_IsInvalidFileHandle(int Handle)
 {
-	BOOL bInvalidHandle=FALSE;
+	bool bInvalidHandle=FALSE;
 
 	/* Check handle was valid with our handle table */
 	if ((Handle < 0) || (Handle >= MAX_FILE_HANDLES))
@@ -840,6 +865,19 @@ void GemDOS_CreateHardDriveFileName(int Drive, const char *pszFileName,
 					j = globbuf.gl_pathc;
 					found = 1;
 				}
+				/* Here comes a work-around for a bug in the file selector
+				 * of TOS 1.02: When a folder name has exactly 8 characters,
+				 * it appends a '.' at the end of the name... */
+				else if (base_len == 8 && pszDestName[len-1] == '.' &&
+						 !strncasecmp(globbuf.gl_pathv[j],pszDestName,len-1))
+				{
+					/* we found a matching name... */
+					snprintf(pszDestName, nDestNameLen, "%s%c%s",
+					         globbuf.gl_pathv[j], PATHSEP, s+1);
+					s -= 1;
+					j = globbuf.gl_pathc;  /* break */
+					found = 1;
+				}
 			}
 			globfree(&globbuf);
 			if (!found)
@@ -910,21 +948,18 @@ void GemDOS_CreateHardDriveFileName(int Drive, const char *pszFileName,
 					found = 1;
 				}
 			}
-#if GEMDOS_FILE_DEBUG
+#ifdef HATARI_TRACE_ACTIVATED
 			if (!found)
 			{
 				/* It's often normal, the gem uses this to test for existence */
 				/* of desktop.inf or newdesk.inf for example. */
-				fprintf(stderr,"didn't find filename %s\n",pszDestName);
+				HATARI_TRACE ( HATARI_TRACE_OS_GEMDOS, "didn't find filename %s\n", pszDestName );
 			}
 #endif
 			globfree(&globbuf);
 		}
 	}
-
-#if GEMDOS_FILE_DEBUG
-	fprintf(stderr,"conv %s -> %s\n",pszFileName,pszDestName);
-#endif
+	HATARI_TRACE ( HATARI_TRACE_OS_GEMDOS, "conv %s -> %s\n", pszFileName, pszDestName );
 }
 
 
@@ -934,7 +969,7 @@ void GemDOS_CreateHardDriveFileName(int Drive, const char *pszFileName,
  * Call 0x3
  */
 #if 0
-static BOOL GemDOS_Cauxin(Uint32 Params)
+static bool GemDOS_Cauxin(Uint32 Params)
 {
 	unsigned char c;
 
@@ -956,7 +991,7 @@ static BOOL GemDOS_Cauxin(Uint32 Params)
  * Call 0x4
  */
 #if 0
-static BOOL GemDOS_Cauxout(Uint32 Params)
+static bool GemDOS_Cauxout(Uint32 Params)
 {
 	unsigned char c;
 
@@ -973,7 +1008,8 @@ static BOOL GemDOS_Cauxout(Uint32 Params)
  * GEMDOS Cprnout
  * Call 0x5
  */
-static BOOL GemDOS_Cprnout(Uint32 Params)
+#if 0
+static bool GemDOS_Cprnout(Uint32 Params)
 {
 	unsigned char c;
 
@@ -984,13 +1020,14 @@ static BOOL GemDOS_Cprnout(Uint32 Params)
 
 	return TRUE;
 }
+#endif
 
 /*-----------------------------------------------------------------------*/
 /**
  * GEMDOS Set drive (0=A,1=B,2=C etc...)
  * Call 0xE
  */
-static BOOL GemDOS_SetDrv(Uint32 Params)
+static bool GemDOS_SetDrv(Uint32 Params)
 {
 	/* Read details from stack for our own use */
 	CurrentDrive = STMemory_ReadWord(Params+SIZE_WORD);
@@ -1004,9 +1041,10 @@ static BOOL GemDOS_SetDrv(Uint32 Params)
  * GEMDOS Cprnos
  * Call 0x11
  */
-static BOOL GemDOS_Cprnos(Uint32 Params)
+#if 0
+static bool GemDOS_Cprnos(Uint32 Params)
 {
-	/* pritner status depends if printing is enabled or not... */
+	/* printer status depends if printing is enabled or not... */
 	if (ConfigureParams.Printer.bEnablePrinting)
 		Regs[REG_D0] = -1;              /* Printer OK */
 	else
@@ -1014,6 +1052,7 @@ static BOOL GemDOS_Cprnos(Uint32 Params)
 
 	return TRUE;
 }
+#endif
 
 /*-----------------------------------------------------------------------*/
 /**
@@ -1021,7 +1060,7 @@ static BOOL GemDOS_Cprnos(Uint32 Params)
  * Call 0x12
  */
 #if 0
-static BOOL GemDOS_Cauxis(Uint32 Params)
+static bool GemDOS_Cauxis(Uint32 Params)
 {
 	/* Read our RS232 state */
 	if (RS232_GetStatus())
@@ -1039,7 +1078,7 @@ static BOOL GemDOS_Cauxis(Uint32 Params)
  * Call 0x13
  */
 #if 0
-static BOOL GemDOS_Cauxos(Uint32 Params)
+static bool GemDOS_Cauxos(Uint32 Params)
 {
 	Regs[REG_D0] = -1;                /* Device ready */
 
@@ -1052,7 +1091,7 @@ static BOOL GemDOS_Cauxos(Uint32 Params)
  * GEMDOS Set Disk Transfer Address (DTA)
  * Call 0x1A
  */
-static BOOL GemDOS_SetDTA(Uint32 Params)
+static bool GemDOS_SetDTA(Uint32 Params)
 {
 	/* Look up on stack to find where DTA is! Store as PC pointer */
 	pDTA = (DTA *)STRAM_ADDR(STMemory_ReadLong(Params+SIZE_WORD));
@@ -1065,7 +1104,7 @@ static BOOL GemDOS_SetDTA(Uint32 Params)
  * GEMDOS Dfree Free disk space.
  * Call 0x39
  */
-static BOOL GemDOS_DFree(Uint32 Params)
+static bool GemDOS_DFree(Uint32 Params)
 {
 	int Drive;
 	Uint32 Address;
@@ -1093,7 +1132,7 @@ static BOOL GemDOS_DFree(Uint32 Params)
  * GEMDOS MkDir
  * Call 0x39
  */
-static BOOL GemDOS_MkDir(Uint32 Params)
+static bool GemDOS_MkDir(Uint32 Params)
 {
 	char *pDirName;
 	int Drive;
@@ -1134,7 +1173,7 @@ static BOOL GemDOS_MkDir(Uint32 Params)
  * GEMDOS RmDir
  * Call 0x3A
  */
-static BOOL GemDOS_RmDir(Uint32 Params)
+static bool GemDOS_RmDir(Uint32 Params)
 {
 	char *pDirName;
 	int Drive;
@@ -1174,7 +1213,7 @@ static BOOL GemDOS_RmDir(Uint32 Params)
  * GEMDOS ChDir
  * Call 0x3B
  */
-static BOOL GemDOS_ChDir(Uint32 Params)
+static bool GemDOS_ChDir(Uint32 Params)
 {
 	char *pDirName;
 	int Drive;
@@ -1182,9 +1221,7 @@ static BOOL GemDOS_ChDir(Uint32 Params)
 	/* Find new directory */
 	pDirName = (char *)STRAM_ADDR(STMemory_ReadLong(Params+SIZE_WORD));
 
-#if GEMDOS_FILE_DEBUG
-	Log_Printf(LOG_DEBUG, "Dsetpath(\"%s\")\n", pDirName);
-#endif
+	HATARI_TRACE ( HATARI_TRACE_OS_GEMDOS, "Dsetpath(\"%s\")\n", pDirName );
 
 	Drive = GemDOS_IsFileNameAHardDrive(pDirName);
 
@@ -1230,7 +1267,7 @@ static BOOL GemDOS_ChDir(Uint32 Params)
  * GEMDOS Create file
  * Call 0x3C
  */
-static BOOL GemDOS_Create(Uint32 Params)
+static bool GemDOS_Create(Uint32 Params)
 {
 	char szActualFileName[MAX_GEMDOS_PATH];
 	char *pszFileName, *ptr;
@@ -1312,7 +1349,7 @@ static BOOL GemDOS_Create(Uint32 Params)
  * GEMDOS Open file
  * Call 0x3D
  */
-static BOOL GemDOS_Open(Uint32 Params)
+static bool GemDOS_Open(Uint32 Params)
 {
 	char szActualFileName[MAX_GEMDOS_PATH];
 	char *pszFileName;
@@ -1366,7 +1403,7 @@ static BOOL GemDOS_Open(Uint32 Params)
 	}
 	
 	if (Mode != 1 && errno == EACCES)
-		Log_Printf(LOG_DEBUG, "Missing permission to read file '%s'\n", szActualFileName);
+		HATARI_TRACE ( HATARI_TRACE_OS_GEMDOS, "Missing permission to read file '%s'\n", szActualFileName );
 	
 	Regs[REG_D0] = GEMDOS_EFILNF;     /* File not found/ error opening */
 	return TRUE;
@@ -1377,7 +1414,7 @@ static BOOL GemDOS_Open(Uint32 Params)
  * GEMDOS Close file
  * Call 0x3E
  */
-static BOOL GemDOS_Close(Uint32 Params)
+static bool GemDOS_Close(Uint32 Params)
 {
 	int Handle;
 
@@ -1406,7 +1443,7 @@ static BOOL GemDOS_Close(Uint32 Params)
  * GEMDOS Read file
  * Call 0x3F
  */
-static BOOL GemDOS_Read(Uint32 Params)
+static bool GemDOS_Read(Uint32 Params)
 {
 	char *pBuffer;
 	unsigned long nBytesRead,Size,CurrentPos,FileSize;
@@ -1463,7 +1500,7 @@ static BOOL GemDOS_Read(Uint32 Params)
  * GEMDOS Write file
  * Call 0x40
  */
-static BOOL GemDOS_Write(Uint32 Params)
+static bool GemDOS_Write(Uint32 Params)
 {
 	char *pBuffer;
 	unsigned long Size,nBytesWritten;
@@ -1500,7 +1537,7 @@ static BOOL GemDOS_Write(Uint32 Params)
  * GEMDOS Delete file
  * Call 0x41
  */
-static BOOL GemDOS_FDelete(Uint32 Params)
+static bool GemDOS_FDelete(Uint32 Params)
 {
 #ifdef ENABLE_SAVING
 	char *pszFileName;
@@ -1542,7 +1579,7 @@ static BOOL GemDOS_FDelete(Uint32 Params)
  * GEMDOS File seek
  * Call 0x42
  */
-static BOOL GemDOS_LSeek(Uint32 Params)
+static bool GemDOS_LSeek(Uint32 Params)
 {
 	long Offset;
 	int Handle,Mode;
@@ -1573,7 +1610,7 @@ static BOOL GemDOS_LSeek(Uint32 Params)
  * GEMDOS Fattrib() - get or set file attributes
  * Call 0x43
  */
-static BOOL GemDOS_Fattrib(Uint32 Params)
+static bool GemDOS_Fattrib(Uint32 Params)
 {
 	char sActualFileName[MAX_GEMDOS_PATH];
 	char *psFileName;
@@ -1588,9 +1625,8 @@ static BOOL GemDOS_Fattrib(Uint32 Params)
 	nRwFlag = STMemory_ReadWord(Params+SIZE_WORD+SIZE_LONG);
 	nAttrib = STMemory_ReadWord(Params+SIZE_WORD+SIZE_LONG+SIZE_WORD);
 
-#ifdef GEMDOS_VERBOSE
-	Log_Printf(LOG_DEBUG, "Fattrib('%s', %d, 0x%x)\n", psFileName, nRwFlag, nAttrib);
-#endif
+	HATARI_TRACE ( HATARI_TRACE_OS_GEMDOS, "Fattrib('%s', %d, 0x%x)\n",
+		       psFileName, nRwFlag, nAttrib );
 
 	if (!ISHARDDRIVE(nDrive))
 	{
@@ -1661,9 +1697,7 @@ static int GemDOS_GetDir(Uint32 Params)
 			c = path[i];
 			STMemory_WriteByte(Address+i, (c==PATHSEP ? '\\' : c) );
 		}
-#if GEMDOS_FILE_DEBUG
-		Log_Printf(LOG_DEBUG, "GemDOS_GetDir (%d) = %s\n", Drive, path);
-#endif
+		HATARI_TRACE ( HATARI_TRACE_OS_GEMDOS, "GemDOS_GetDir (%d) = %s\n", Drive, path );
 
 		return TRUE;
 	}
@@ -1750,10 +1784,10 @@ static int GemDOS_Pexec(Uint32 Params)
  * GEMDOS Search Next
  * Call 0x4F
  */
-static BOOL GemDOS_SNext(void)
+static bool GemDOS_SNext(void)
 {
-	struct dirent **temp;
 	int Index;
+	int ret;
 
 	/* Refresh pDTA pointer (from the current basepage) */
 	pDTA = (DTA *)STRAM_ADDR(STMemory_ReadLong(STMemory_ReadLong(act_pd)+32));
@@ -1761,20 +1795,34 @@ static BOOL GemDOS_SNext(void)
 	/* Was DTA ours or TOS? */
 	if (do_get_mem_long(pDTA->magic) == DTA_MAGIC_NUMBER)
 	{
+		struct dirent **temp;
 
 		/* Find index into our list of structures */
 		Index = do_get_mem_word(pDTA->index) & (MAX_DTAS_FILES-1);
 
-		if (InternalDTAs[Index].centry >= InternalDTAs[Index].nentries)
+		if (nAttrSFirst == 8)
 		{
 			Regs[REG_D0] = GEMDOS_ENMFIL;    /* No more files */
 			return TRUE;
 		}
 
 		temp = InternalDTAs[Index].found;
-		if (PopulateDTA(InternalDTAs[Index].path, temp[InternalDTAs[Index].centry++]) == FALSE)
+		do
 		{
-			Log_Printf(LOG_ERROR, "GemDOS_SNext: Error setting DTA.\n");
+			if (InternalDTAs[Index].centry >= InternalDTAs[Index].nentries)
+			{
+				Regs[REG_D0] = GEMDOS_ENMFIL;    /* No more files */
+				return TRUE;
+			}
+
+			ret = PopulateDTA(InternalDTAs[Index].path,
+					  temp[InternalDTAs[Index].centry++]);
+		} while (ret == 1);
+
+		if (ret < 0)
+		{
+			Log_Printf(LOG_WARN, "GemDOS_SNext: Error setting DTA.\n");
+			Regs[REG_D0] = GEMDOS_EINTRN;
 			return TRUE;
 		}
 
@@ -1791,20 +1839,19 @@ static BOOL GemDOS_SNext(void)
  * GEMDOS Find first file
  * Call 0x4E
  */
-static BOOL GemDOS_SFirst(Uint32 Params)
+static bool GemDOS_SFirst(Uint32 Params)
 {
 	char szActualFileName[MAX_GEMDOS_PATH];
 	char tempstr[MAX_GEMDOS_PATH];
 	char *pszFileName;
 	struct dirent **files;
-	Uint16 Attr;
 	int Drive;
 	DIR *fsdir;
 	int i,j,count;
 
 	/* Find filename to search for */
 	pszFileName = (char *)STRAM_ADDR(STMemory_ReadLong(Params+SIZE_WORD));
-	Attr = STMemory_ReadWord(Params+SIZE_WORD+SIZE_LONG);
+	nAttrSFirst = STMemory_ReadWord(Params+SIZE_WORD+SIZE_LONG);
 
 	/* Refresh pDTA pointer (from the current basepage) */
 	pDTA = (DTA *)STRAM_ADDR(STMemory_ReadLong(STMemory_ReadLong(act_pd)+32));
@@ -1827,7 +1874,7 @@ static BOOL GemDOS_SFirst(Uint32 Params)
 		InternalDTAs[DTAIndex].bUsed = TRUE;
 
 		/* Were we looking for the volume label? */
-		if (Attr == GEMDOS_FILE_ATTRIB_VOLUME_LABEL)
+		if (nAttrSFirst == GEMDOS_FILE_ATTRIB_VOLUME_LABEL)
 		{
 			/* Volume name */
 			strcpy(pDTA->dta_name,"EMULATED.001");
@@ -1862,6 +1909,7 @@ static BOOL GemDOS_SFirst(Uint32 Params)
 		/* count & copy the entries that match our mask and discard the rest */
 		j = 0;
 		for (i=0; i < count; i++)
+		{
 			if (match(tempstr, files[i]->d_name))
 			{
 				InternalDTAs[DTAIndex].found[j] = files[i];
@@ -1870,7 +1918,9 @@ static BOOL GemDOS_SFirst(Uint32 Params)
 			else
 			{
 				free(files[i]);
+				files[i] = NULL;
 			}
+		}
 		InternalDTAs[DTAIndex].nentries = j; /* set number of legal entries */
 
 		/* No files of that match, return error code */
@@ -1899,7 +1949,7 @@ static BOOL GemDOS_SFirst(Uint32 Params)
  * GEMDOS Rename
  * Call 0x56
  */
-static BOOL GemDOS_Rename(Uint32 Params)
+static bool GemDOS_Rename(Uint32 Params)
 {
 	char *pszNewFileName,*pszOldFileName;
 	char szNewActualFileName[MAX_GEMDOS_PATH],szOldActualFileName[MAX_GEMDOS_PATH];
@@ -1935,7 +1985,7 @@ static BOOL GemDOS_Rename(Uint32 Params)
  * GEMDOS GSDToF
  * Call 0x57
  */
-static BOOL GemDOS_GSDToF(Uint32 Params)
+static bool GemDOS_GSDToF(Uint32 Params)
 {
 	DATETIME DateTime;
 	Uint32 pBuffer;
@@ -2010,11 +2060,17 @@ void GemDOS_OpCode(void)
 	/* Find pointer to call parameters */
 	GemDOSCall = STMemory_ReadWord(Params);
 
-#ifdef GEMDOS_VERBOSE
+#ifdef HATARI_TRACE_ACTIVATED
 	if (GemDOSCall < (sizeof(pszGemDOSNames)/sizeof(pszGemDOSNames[0])))
-		fprintf(stderr, "GemDOS 0x%X (%s)\n",GemDOSCall,pszGemDOSNames[GemDOSCall]);
+	{
+		HATARI_TRACE ( HATARI_TRACE_OS_GEMDOS, "GemDOS 0x%X (%s)\n",
+			       GemDOSCall, pszGemDOSNames[GemDOSCall] );
+	}
 	else
-		fprintf(stderr, "GemDOS 0x%X\n",GemDOSCall);
+	{
+		HATARI_TRACE ( HATARI_TRACE_OS_GEMDOS, "GemDOS 0x%X\n",
+			       GemDOSCall );
+	}
 #endif
 
 	/* Intercept call */
@@ -2032,18 +2088,24 @@ void GemDOS_OpCode(void)
 			RunOld = FALSE;
 		break;
 	 */
-	 case 0x5:          /* direct printing via GEMDOS */
+	 /* direct printing via GEMDOS */
+	 /*
+	 case 0x5:
 		if (GemDOS_Cprnout(Params))
 			RunOld = FALSE;
 		break;
+	 */
 	 case 0xe:
 		if (GemDOS_SetDrv(Params))
 			RunOld = FALSE;
 		break;
-	 case 0x11:         /* Printer status  */
+	 /* Printer status  */
+	 /*
+	 case 0x11:
 		if (GemDOS_Cprnos(Params))
 			RunOld = FALSE;
 		break;
+	 */
 	 /*
 	 case 0x12:
 		if (GemDOS_Cauxis(Params))
@@ -2133,13 +2195,36 @@ void GemDOS_OpCode(void)
 			RunOld = FALSE;
 		break;
 	}
-
+	
 	switch(RunOld)
 	{
-	 case FALSE:        /* skip over branch to pexec to RTE */
+	 case FALSE:
+		/* skip over branch to pexec to RTE */
 		SR |= SR_ZERO;
+		/* visualize GemDOS emu HD access? */
+		switch (GemDOSCall)
+		{
+		 case 0x36:
+		 case 0x39:
+		 case 0x3a:
+		 case 0x3b:
+		 case 0x3c:
+		 case 0x3d:
+		 case 0x3e:
+		 case 0x3f:
+		 case 0x40:
+		 case 0x41:
+		 case 0x42:
+		 case 0x43:
+		 case 0x47:
+		 case 0x4e:
+		 case 0x4f:
+		 case 0x56:
+			Statusbar_EnableHDLed();
+		}
 		break;
-	 case CALL_PEXEC_ROUTINE:   /* branch to pexec, then redirect to old gemdos. */
+	 case CALL_PEXEC_ROUTINE:
+		/* branch to pexec, then redirect to old gemdos. */
 		SR |= SR_OVERFLOW;
 		break;
 	}
@@ -2158,9 +2243,7 @@ void GemDOS_Boot(void)
 {
 	bInitGemDOS = TRUE;
 
-#ifdef GEMDOS_VERBOSE
-	fprintf(stderr, "Gemdos_Boot()\n");
-#endif
+	HATARI_TRACE ( HATARI_TRACE_OS_GEMDOS, "Gemdos_Boot()\n" );
 
 	/* install our gemdos handler, if -e or --harddrive option used */
 	if (GEMDOS_EMU_ON)
