@@ -97,8 +97,9 @@ static BOOL   gFinderLaunch;
 		size_t cbPath = strlen(constSzPath) + 1;
 		char szPath[cbPath];
 		strncpy(szPath, constSzPath, cbPath);	
-	
-		Floppy_InsertDiskIntoDrive(0, szPath, cbPath);
+
+		Floppy_SetDiskFileName(0, szPath, NULL);
+		Floppy_InsertDiskIntoDrive(0);
 	}
 }
 
@@ -121,7 +122,8 @@ static BOOL   gFinderLaunch;
 		char szPath[cbPath];
 		strncpy(szPath, constSzPath, cbPath);	
 	
-		Floppy_InsertDiskIntoDrive(1, szPath, cbPath);
+		Floppy_SetDiskFileName(1, szPath, NULL);
+		Floppy_InsertDiskIntoDrive(1);
 	}
 }
 
@@ -161,7 +163,7 @@ static BOOL   gFinderLaunch;
 - (IBAction)captureAnimation:(id)sender
 {
 	GuiOsx_PauseAndSwitchToCocoaUI();
-	ScreenSnapShot_BeginRecording(DialogParams.Screen.bCaptureChange, 25);
+	ScreenSnapShot_BeginRecording(ConfigureParams.Screen.bCaptureChange);
 	GuiOsx_ResumeFromCocoaUI();
 }
 
@@ -325,12 +327,7 @@ static BOOL   gFinderLaunch;
 
 - (IBAction)help:(id)sender
 {
-	[[NSWorkspace sharedWorkspace] openURL:[NSURL URLWithString:@"http://hatari.sourceforge.net/"]];
-	/*
-	NSRunAlertPanel (@"Help", 
-			@"Sorry, there is no built-in help available.\n\nPlease visit http://hatari.sourceforge.net/ for more information.\n\n",
-			@"OK", nil, nil);
-	*/
+	[[NSWorkspace sharedWorkspace] openURL:[NSURL URLWithString:@"http://hatari.berlios.de/"]];
 }
 
 
