@@ -6,7 +6,7 @@
 
   Show information about the program and its license.
 */
-const char DlgAbout_rcsid[] = "Hatari $Id: dlgAbout.c,v 1.6 2006/02/08 22:46:10 eerot Exp $";
+const char DlgAbout_fileid[] = "Hatari dlgAbout.c : " __DATE__ " " __TIME__;
 
 #include "main.h"
 #include "dialog.h"
@@ -17,10 +17,10 @@ const char DlgAbout_rcsid[] = "Hatari $Id: dlgAbout.c,v 1.6 2006/02/08 22:46:10
 static SGOBJ aboutdlg[] =
 {
 	{ SGBOX, 0, 0, 0,0, 40,25, NULL },
-	{ SGTEXT, 0, 0, 14,1, 12,1, PROG_NAME },
-	{ SGTEXT, 0, 0, 14,2, 12,1, "============" },
-	{ SGTEXT, 0, 0, 1,4, 38,1, "Hatari has been written by T. Huth," },
-	{ SGTEXT, 0, 0, 1,5, 38,1, "S. Marothy and many others..." },
+	{ SGTEXT, 0, 0, 13,1, 12,1, PROG_NAME },
+	{ SGTEXT, 0, 0, 13,2, 12,1, "=============" },
+	{ SGTEXT, 0, 0, 1,4, 38,1, "Written by Thomas Huth and many other" },
+	{ SGTEXT, 0, 0, 1,5, 38,1, "people around the world." },
 	{ SGTEXT, 0, 0, 2,7, 34,1, "Please see the docs for more info!" },
 	{ SGTEXT, 0, 0, 1,9, 38,1, "This program is free software; you can" },
 	{ SGTEXT, 0, 0, 1,10, 38,1, "redistribute it and/or modify it under" },
@@ -33,7 +33,7 @@ static SGOBJ aboutdlg[] =
 	{ SGTEXT, 0, 0, 1,18, 38,1, "hope that it will be useful, but" },
 	{ SGTEXT, 0, 0, 1,19, 38,1, "WITHOUT ANY WARRANTY. See the GNU Ge-" },
 	{ SGTEXT, 0, 0, 1,20, 38,1, "neral Public License for more details." },
-	{ SGBUTTON, 0, 0, 16,23, 8,1, "Okay" },
+	{ SGBUTTON, SG_DEFAULT, 0, 16,23, 8,1, "Okay" },
 	{ -1, 0, 0, 0,0, 0,0, NULL }
 };
 
