@@ -6,7 +6,7 @@
 
   Low-level hard drive emulation
 */
-const char HDC_rcsid[] = "Hatari $Id: hdc.c,v 1.17 2007/01/16 18:42:59 thothy Exp $";
+const char HDC_fileid[] = "Hatari hdc.c : " __DATE__ " " __TIME__;
 
 #include "main.h"
 #include "configuration.h"
@@ -19,6 +19,8 @@ const char HDC_rcsid[] = "Hatari $Id: hdc.c,v 1.17 2007/01/16 18:42:59 thothy Ex
 #include "mfp.h"
 #include "stMemory.h"
 #include "tos.h"
+#include "statusbar.h"
+
 
 /*
   ACSI emulation: 
@@ -48,10 +50,10 @@ HDCOMMAND HDCCommand;
 FILE *hd_image_file = NULL;
 int nPartitions = 0;
 short int HDCSectorCount;
-BOOL bAcsiEmuOn = FALSE;
+bool bAcsiEmuOn = FALSE;
 
 static Uint32 nLastBlockAddr;
-static BOOL bSetLastBlockAddr;
+static bool bSetLastBlockAddr;
 static Uint8 nLastError;
 
 /*
@@ -164,7 +166,7 @@ static void HDC_Cmd_RequestSense(void)
 
 	if ((nRetLen < 4 && nRetLen != 0) || nRetLen > 22)
 	{
-		Log_Printf(LOG_ERROR, "HDC: *** Strange REQUEST SENSE ***!\n");
+		Log_Printf(LOG_WARN, "HDC: *** Strange REQUEST SENSE ***!\n");
 	}
 
 	/* Limit to sane length */
@@ -280,7 +282,7 @@ static void HDC_Cmd_ModeSense(void)
 	}
 	else
 	{
-		Log_Printf(LOG_ERROR, "HDC: Unsupported MODE SENSE command\n");
+		Log_Printf(LOG_TODO, "HDC: Unsupported MODE SENSE command\n");
 		HDCCommand.returnCode = HD_STATUS_ERROR;
 		nLastError = HD_REQSENS_INVARG;
 	}
@@ -337,6 +339,8 @@ static void HDC_Cmd_WriteSector(void)
 		fwrite(&STRam[FDC_ReadDMAAddress()], 512, HD_SECTORCOUNT(HDCCommand),
 		       hd_image_file);
 #endif
+		/* Update DMA counter */
+		FDC_WriteDMAAddress(FDC_ReadDMAAddress() + 512*HD_SECTORCOUNT(HDCCommand));
 	}
 
 	FDC_SetDMAStatus(FALSE);              /* no DMA error */
@@ -371,6 +375,9 @@ static void HDC_Cmd_ReadSector(void)
 		      hd_image_file);
 		HDCCommand.returnCode = HD_STATUS_OK;
 		nLastError = HD_REQSENS_OK;
+
+		/* Update DMA counter */
+		FDC_WriteDMAAddress(FDC_ReadDMAAddress() + 512*HD_SECTORCOUNT(HDCCommand));
 	}
 
 	FDC_SetDMAStatus(FALSE);              /* no DMA error */
@@ -415,7 +422,7 @@ void HDC_EmulateCommandPacket()
 		break;
 
 	 case HD_MODESELECT:
-		Log_Printf(LOG_DEBUG, "HDC: MODE SELECT call not implemented yet.\n");
+		Log_Printf(LOG_TODO, "HDC: MODE SELECT call not implemented yet.\n");
 		HDCCommand.returnCode = HD_STATUS_OK;
 		nLastError = HD_REQSENS_OK;
 		bSetLastBlockAddr = FALSE;
@@ -443,6 +450,9 @@ void HDC_EmulateCommandPacket()
 		FDC_AcknowledgeInterrupt();
 		break;
 	}
+
+	/* Update the led each time a command is processed */
+	Statusbar_EnableHDLed();
 }
 
 
@@ -568,7 +578,7 @@ static void HDC_GetInfo(void)
 /**
  * Open the disk image file, set partitions.
  */
-BOOL HDC_Init(char *filename)
+bool HDC_Init(char *filename)
 {
 	bAcsiEmuOn = FALSE;
 
