@@ -10,10 +10,8 @@
 
 #include "configuration.h"
 
-extern CNF_PARAMS  DialogParams;
-
 /* prototypes for gui-sdl/dlg*.c functions: */
-extern int Dialog_MainDlg(BOOL *bReset);
+extern int Dialog_MainDlg(bool *bReset, bool *bLoadedSnapshot);
 extern void Dialog_AboutDlg(void);
 extern int DlgAlert_Notice(const char *text);
 extern int DlgAlert_Query(const char *text);
@@ -21,17 +19,13 @@ extern void Dialog_DeviceDlg(void);
 extern void Dialog_DiskDlg(void);
 extern void Dialog_JoyDlg(void);
 extern void Dialog_KeyboardDlg(void);
-extern void Dialog_MemDlg(void);
+extern bool Dialog_MemDlg(void);
 extern void DlgNewDisk_Main(void);
 extern void Dialog_ScreenDlg(void);
 extern void Dialog_SoundDlg(void);
 extern void Dialog_SystemDlg(void);
 extern void DlgRom_Main(void);
 /* and dialog.c */
-extern BOOL Dialog_DoNeedReset(void);
-extern BOOL Dialog_DoProperty(void);
-extern void Dialog_CopyDialogParamsToConfiguration(BOOL bForceReset);
-extern void Dialog_LoadParams(void);
-extern void Dialog_SaveParams(void);
+extern bool Dialog_DoProperty(void);
 
 #endif
