@@ -21,8 +21,13 @@ typedef enum
   INTERRUPT_MFP_TIMERD,
   INTERRUPT_IKBD_RESETTIMER,
   INTERRUPT_IKBD_ACIA,
+  INTERRUPT_IKBD_MFP,
+  INTERRUPT_IKBD_AUTOSEND,
   INTERRUPT_DMASOUND,
+  INTERRUPT_DMASOUND_MICROWIRE,
   INTERRUPT_FDC,
+  INTERRUPT_BLITTER,
+  INTERRUPT_MIDI,
 
   MAX_INTERRUPTS
 } interrupt_id;
@@ -53,15 +58,15 @@ extern void (*PendingInterruptFunction)(void);
 extern int PendingInterruptCount;
 
 extern void Int_Reset(void);
-extern void Int_MemorySnapShot_Capture(BOOL bSave);
+extern void Int_MemorySnapShot_Capture(bool bSave);
 extern void Int_AcknowledgeInterrupt(void);
 extern void Int_AddAbsoluteInterrupt(int CycleTime, int CycleType, interrupt_id Handler);
-extern void Int_AddRelativeInterrupt(int CycleTime, int CycleType, interrupt_id Handler, int AddInternalCycle);
+extern void Int_AddRelativeInterrupt(int CycleTime, int CycleType, interrupt_id Handler);
 extern void Int_AddRelativeInterruptNoOffset(int CycleTime, int CycleType, interrupt_id Handler);
 extern void Int_AddRelativeInterruptWithOffset(int CycleTime, int CycleType, interrupt_id Handler, int CycleOffset);
 extern void Int_RemovePendingInterrupt(interrupt_id Handler);
 extern void Int_ResumeStoppedInterrupt(interrupt_id Handler);
-extern BOOL Int_InterruptActive(interrupt_id Handler);
+extern bool Int_InterruptActive(interrupt_id Handler);
 extern int Int_FindCyclesPassed(interrupt_id Handler, int CycleType);
 
 #endif /* ifndef HATARI_INT_H */
