@@ -8,18 +8,22 @@
 #ifndef HATARI_IOMEM_H
 #define HATARI_IOMEM_H
 
-#if 0
-#include "sysdeps.h"
-#include "maccess.h"
-#include "main.h"
+#include "config.h"
+
+#if ENABLE_SMALL_MEM
+# include "sysdeps.h"
+# include "maccess.h"
+# include "main.h"
+extern uae_u8 *IOmemory;
+# define IoMem (IOmemory-0xff0000)
 #else
-/* TODO: IoMem will later become independent from STRam... */
-#include "stMemory.h"
-#define IoMem STRam
-#endif
+# include "stMemory.h"
+# define IoMem STRam
+#endif  /* ENABLE_SMALL_MEM */
 
 
 extern Uint32 IoAccessBaseAddress;
+extern Uint32 IoAccessCurrentAddress;
 extern int nIoMemAccessSize;
 
 
