@@ -39,10 +39,12 @@ enum
 extern int JoystickSpaceBar;
 
 extern void Joy_Init(void);
+extern void Joy_UnInit(void);
 extern Uint8 Joy_GetStickData(int nStJoyId);
+extern bool Joy_SetCursorEmulation(int port);
 extern void Joy_ToggleCursorEmulation(void);
-extern BOOL Joy_KeyDown(int symkey, int modkey);
-extern BOOL Joy_KeyUp(int symkey, int modkey);
+extern bool Joy_KeyDown(int symkey, int modkey);
+extern bool Joy_KeyUp(int symkey, int modkey);
 extern void Joy_StePadButtons_ReadWord(void);
 extern void Joy_StePadMulti_ReadWord(void);
 extern void Joy_StePadMulti_WriteWord(void);
