@@ -11,17 +11,17 @@
 
 #define  MAX_RS232INPUT_BUFFER    2048  /* Must be ^2 */
 
-extern BOOL bConnectedRS232;
+extern bool bConnectedRS232;
 
 extern void RS232_Init(void);
 extern void RS232_UnInit(void);
 extern void RS232_HandleUCR(short int ucr);
-extern BOOL RS232_SetBaudRate(int nBaud);
+extern bool RS232_SetBaudRate(int nBaud);
 extern void RS232_SetBaudRateFromTimerD(void);
 extern void RS232_SetFlowControl(int ctrl);
-extern BOOL RS232_TransferBytesTo(unsigned char *pBytes, int nBytes);
-extern BOOL RS232_ReadBytes(unsigned char *pBytes, int nBytes);
-extern BOOL RS232_GetStatus(void);
+extern bool RS232_TransferBytesTo(unsigned char *pBytes, int nBytes);
+extern bool RS232_ReadBytes(unsigned char *pBytes, int nBytes);
+extern bool RS232_GetStatus(void);
 extern void RS232_SCR_ReadByte(void);
 extern void RS232_SCR_WriteByte(void);
 extern void RS232_UCR_ReadByte(void);
