@@ -15,7 +15,7 @@
 #define dirent direct
 #endif
 
-#if defined(__BEOS__) || (defined(__sun) && defined(__SVR4)) || defined(WIN32)
+#if defined(__BEOS__) || (defined(__sun) && defined(__SVR4)) || defined(WIN32) || defined(__CEGCC__)
 extern int alphasort(const void *d1, const void *d2);
 extern int scandir(const char *dirname, struct dirent ***namelist, int (*sdfilter)(struct dirent *), int (*dcomp)(const void *, const void *));
 #endif
