@@ -59,22 +59,26 @@
 /*			where it was stopped, without updating 'cycles'.			*/
 /* 2007/12/27	[NP]	Parameter 'AddInternalCycle' in Int_AddRelativeInterrupt, used in mfp.c	*/
 /*			to precisely start a timer after the current inst.			*/
+/* 2008/03/30	[NP]	ActiveInterrupt was not saved in the memory snapshot, which could cause	*/
+/*			errors when restoring it. We call Int_SetNewInterrupt() after restoring	*/
+/*			the snapshot to set ActiveInterrupt to the correct value.		*/
 
 
+const char Int_fileid[] = "Hatari int.c : " __DATE__ " " __TIME__;
 
-const char Int_rcsid[] = "Hatari $Id: int.c,v 1.19 2008/02/29 21:11:12 thothy Exp $";
-
+#include <stdint.h>
 #include "main.h"
+#include "blitter.h"
 #include "dmaSnd.h"
 #include "fdc.h"
 #include "ikbd.h"
 #include "int.h"
 #include "m68000.h"
-#include "memorySnapShot.h"
 #include "mfp.h"
+#include "midi.h"
+#include "memorySnapShot.h"
 #include "sound.h"
 #include "video.h"
-#include "trace.h"
 
 
 void (*PendingInterruptFunction)(void);
@@ -96,22 +100,28 @@ static void (* const pIntHandlerFunctions[MAX_INTERRUPTS])(void) =
 	MFP_InterruptHandler_TimerD,
 	IKBD_InterruptHandler_ResetTimer,
 	IKBD_InterruptHandler_ACIA,
+	IKBD_InterruptHandler_MFP,
+	IKBD_InterruptHandler_AutoSend,
 	DmaSnd_InterruptHandler,
-	FDC_InterruptHandler_Update
+	DmaSnd_InterruptHandler_Microwire,
+	FDC_InterruptHandler_Update,
+	Blitter_InterruptHandler,
+	Midi_InterruptHandler_Update
 };
 
 /* Event timer structure - keeps next timer to occur in structure so don't need
  * to check all entries */
 typedef struct
 {
-	BOOL bUsed;                   /* Is interrupt active? */
-	int Cycles;
+	bool bUsed;                   /* Is interrupt active? */
+	Sint64 Cycles;
 	void (*pFunction)(void);
 } INTERRUPTHANDLER;
 
 static INTERRUPTHANDLER InterruptHandlers[MAX_INTERRUPTS];
 static int ActiveInterrupt=0;
 
+static void Int_SetNewInterrupt(void);
 
 /*-----------------------------------------------------------------------*/
 /**
@@ -130,6 +140,7 @@ void Int_Reset(void)
 	for (i=0; i<MAX_INTERRUPTS; i++)
 	{
 		InterruptHandlers[i].bUsed = FALSE;
+		InterruptHandlers[i].Cycles = INT_MAX;
 		InterruptHandlers[i].pFunction = pIntHandlerFunctions[i];
 	}
 }
@@ -172,7 +183,7 @@ static void *Int_IDToHandlerFunction(int ID)
 /**
  * Save/Restore snapshot of local variables('MemorySnapShot_Store' handles type)
  */
-void Int_MemorySnapShot_Capture(BOOL bSave)
+void Int_MemorySnapShot_Capture(bool bSave)
 {
 	int i,ID;
 
@@ -208,18 +219,29 @@ void Int_MemorySnapShot_Capture(BOOL bSave)
 		MemorySnapShot_Store(&ID, sizeof(int));
 		PendingInterruptFunction = Int_IDToHandlerFunction(ID);
 	}
+
+
+	if (!bSave)
+		Int_SetNewInterrupt();			/* when restoring snapshot, compute current state after */
 }
 
 
 /*-----------------------------------------------------------------------*/
 /**
- * Find next interrupt to occur, and store to global variables for decrement in instruction decode loop
+ * Find next interrupt to occur, and store to global variables for decrement
+ * in instruction decode loop.
+ * Note: Although InterruptHandlers.Cycles and LowestCycleCount are 64 bit
+ * variables to get all the cycle counters right (e.g. the DMA sound counter
+ * can get very high), PendingInterruptCount is still a 32 bit variable for
+ * performance reasons (it's decremented after each CPU instruction).
+ * So we have to initialize LowestCycleCount with INT_MAX, not with INT64_MAX!
+ * Since there is always a VBL or HBL counter pending which fits fine into the
+ * 32 bit variable, we can be sure that we don't run into problems here.
  */
 static void Int_SetNewInterrupt(void)
 {
-	int LowestCycleCount=0x7fffffff, LowestInterrupt=0;
-	int i;
-
+	Sint64 LowestCycleCount = INT_MAX;
+	interrupt_id LowestInterrupt = 0, i;
 
 	HATARI_TRACE ( HATARI_TRACE_INT , "int set new in video_cyc=%d active_int=%d pending_count=%d\n",
 	               Cycles_GetCounter(CYCLES_COUNTER_VIDEO), ActiveInterrupt, PendingInterruptCount );
@@ -254,13 +276,13 @@ static void Int_SetNewInterrupt(void)
  */
 static void Int_UpdateInterrupt(void)
 {
-	int CycleSubtract;
+	Sint64 CycleSubtract;
 	int i;
 
 	/* Find out how many cycles we went over (<=0) */
 	nCyclesOver = PendingInterruptCount;
 	/* Calculate how many cycles have passed, included time we went over */
-	CycleSubtract = InterruptHandlers[ActiveInterrupt].Cycles-nCyclesOver;
+	CycleSubtract = InterruptHandlers[ActiveInterrupt].Cycles - nCyclesOver;
 
 	/* Adjust table */
 	for (i = 0; i < MAX_INTERRUPTS; i++)
@@ -269,7 +291,7 @@ static void Int_UpdateInterrupt(void)
 			InterruptHandlers[i].Cycles -= CycleSubtract;
 	}
 
-	HATARI_TRACE ( HATARI_TRACE_INT , "int upd video_cyc=%d cycle_over=%d cycle_sub=%d\n",
+	HATARI_TRACE ( HATARI_TRACE_INT , "int upd video_cyc=%d cycle_over=%d cycle_sub=%lld\n",
 	               Cycles_GetCounter(CYCLES_COUNTER_VIDEO), nCyclesOver, CycleSubtract );
 }
 
@@ -291,7 +313,7 @@ void Int_AcknowledgeInterrupt(void)
 	Int_SetNewInterrupt();
 
 	HATARI_TRACE ( HATARI_TRACE_INT , "int ack video_cyc=%d active_int=%d active_cyc=%d pending_count=%d\n",
-	               Cycles_GetCounter(CYCLES_COUNTER_VIDEO), ActiveInterrupt, InterruptHandlers[ActiveInterrupt].Cycles, PendingInterruptCount );
+	               Cycles_GetCounter(CYCLES_COUNTER_VIDEO), ActiveInterrupt, (int)InterruptHandlers[ActiveInterrupt].Cycles, PendingInterruptCount );
 }
 
 
@@ -308,40 +330,23 @@ void Int_AddAbsoluteInterrupt(int CycleTime, int CycleType, interrupt_id Handler
 //    Int_UpdateInterrupt();
 
 	InterruptHandlers[Handler].bUsed = TRUE;
-	InterruptHandlers[Handler].Cycles = INT_CONVERT_TO_INTERNAL ( CycleTime , CycleType ) + nCyclesOver;
+	InterruptHandlers[Handler].Cycles = INT_CONVERT_TO_INTERNAL((Sint64)CycleTime , CycleType) + nCyclesOver;
 
 	/* Set new */
 	Int_SetNewInterrupt();
 
-	HATARI_TRACE ( HATARI_TRACE_INT , "int add abs video_cyc=%d handler=%d handler_cyc=%d pending_count=%d\n",
+	HATARI_TRACE ( HATARI_TRACE_INT , "int add abs video_cyc=%d handler=%d handler_cyc=%lld pending_count=%d\n",
 	               Cycles_GetCounter(CYCLES_COUNTER_VIDEO), Handler, InterruptHandlers[Handler].Cycles, PendingInterruptCount );
 }
 
 
 /*-----------------------------------------------------------------------*/
 /**
- * Add interrupt to occur from now
- * 'AddInternalCycle' can be used to add another delay to the resulting
- * number of internal cycles (should be 0 most of the time, except in
- * the MFP emulation to start timers precisely based on the number of
- * cycles of the current instruction).
+ * Add interrupt to occur from now.
  */
-void Int_AddRelativeInterrupt(int CycleTime, int CycleType, interrupt_id Handler, int AddInternalCycle)
+void Int_AddRelativeInterrupt(int CycleTime, int CycleType, interrupt_id Handler)
 {
-	/* Update list cycle counts before adding a new one, */
-	/* since Int_SetNewInterrupt can change the active int / PendingInterruptCount */
-	if ( ( ActiveInterrupt > 0 ) && ( PendingInterruptCount > 0 ) )
-		Int_UpdateInterrupt();
-
-	InterruptHandlers[Handler].bUsed = TRUE;
-	InterruptHandlers[Handler].Cycles = INT_CONVERT_TO_INTERNAL ( CycleTime , CycleType );
-	InterruptHandlers[Handler].Cycles += AddInternalCycle;
-
-	/* Set new */
-	Int_SetNewInterrupt();
-
-	HATARI_TRACE ( HATARI_TRACE_INT , "int add rel video_cyc=%d handler=%d handler_cyc=%d pending_count=%d\n",
-	               Cycles_GetCounter(CYCLES_COUNTER_VIDEO), Handler, InterruptHandlers[Handler].Cycles, PendingInterruptCount );
+	Int_AddRelativeInterruptWithOffset(CycleTime, CycleType, Handler, 0);
 }
 
 
@@ -358,20 +363,24 @@ void Int_AddRelativeInterruptNoOffset(int CycleTime, int CycleType, interrupt_id
 
 //  nCyclesOver = 0;
 	InterruptHandlers[Handler].bUsed = TRUE;
-	InterruptHandlers[Handler].Cycles = INT_CONVERT_TO_INTERNAL ( CycleTime , CycleType ) + PendingInterruptCount;
+	InterruptHandlers[Handler].Cycles = INT_CONVERT_TO_INTERNAL((Sint64)CycleTime , CycleType) + PendingInterruptCount;
 
 	/* Set new */
 	Int_SetNewInterrupt();
 
-	HATARI_TRACE ( HATARI_TRACE_INT , "int add rel no_off video_cyc=%d handler=%d handler_cyc=%d pending_count=%d\n",
+	HATARI_TRACE ( HATARI_TRACE_INT , "int add rel no_off video_cyc=%d handler=%d handler_cyc=%lld pending_count=%d\n",
 	               Cycles_GetCounter(CYCLES_COUNTER_VIDEO), Handler, InterruptHandlers[Handler].Cycles, PendingInterruptCount );
 }
 
 
 /*-----------------------------------------------------------------------*/
 /**
- * Add interrupt to occur after CycleTime/CycleType + CycleOffset
- * This allows to restart an MFP timer just after it expired
+ * Add interrupt to occur after CycleTime/CycleType + CycleOffset.
+ * CycleOffset can be used to add another delay to the resulting
+ * number of internal cycles (should be 0 most of the time, except in
+ * the MFP emulation to start timers precisely based on the number of
+ * cycles of the current instruction).
+ * This allows to restart an MFP timer just after it expired.
  */
 void Int_AddRelativeInterruptWithOffset(int CycleTime, int CycleType, interrupt_id Handler, int CycleOffset)
 {
@@ -381,12 +390,12 @@ void Int_AddRelativeInterruptWithOffset(int CycleTime, int CycleType, interrupt_
 		Int_UpdateInterrupt();
 
 	InterruptHandlers[Handler].bUsed = TRUE;
-	InterruptHandlers[Handler].Cycles = INT_CONVERT_TO_INTERNAL ( CycleTime , CycleType ) + CycleOffset;
+	InterruptHandlers[Handler].Cycles = INT_CONVERT_TO_INTERNAL((Sint64)CycleTime , CycleType) + CycleOffset;
 
 	/* Set new */
 	Int_SetNewInterrupt();
 
-	HATARI_TRACE ( HATARI_TRACE_INT , "int add rel offset video_cyc=%d handler=%d handler_cyc=%di offset_cyc=%d pending_count=%d\n",
+	HATARI_TRACE ( HATARI_TRACE_INT , "int add rel offset video_cyc=%d handler=%d handler_cyc=%lld offset_cyc=%d pending_count=%d\n",
 	               Cycles_GetCounter(CYCLES_COUNTER_VIDEO), Handler, InterruptHandlers[Handler].Cycles, CycleOffset, PendingInterruptCount );
 }
 
@@ -407,7 +416,7 @@ void Int_RemovePendingInterrupt(interrupt_id Handler)
 	/* Set new */
 	Int_SetNewInterrupt();
 
-	HATARI_TRACE ( HATARI_TRACE_INT , "int remove pending video_cyc=%d handler=%d handler_cyc=%d pending_count=%d\n",
+	HATARI_TRACE ( HATARI_TRACE_INT , "int remove pending video_cyc=%d handler=%d handler_cyc=%lld pending_count=%d\n",
 	               Cycles_GetCounter(CYCLES_COUNTER_VIDEO), Handler, InterruptHandlers[Handler].Cycles, PendingInterruptCount );
 }
 
@@ -426,7 +435,7 @@ void Int_ResumeStoppedInterrupt(interrupt_id Handler)
 	/* Set new */
 	Int_SetNewInterrupt();
 
-	HATARI_TRACE ( HATARI_TRACE_INT , "int resume stopped video_cyc=%d handler=%d handler_cyc=%d pending_count=%d\n",
+	HATARI_TRACE ( HATARI_TRACE_INT , "int resume stopped video_cyc=%d handler=%d handler_cyc=%lld pending_count=%d\n",
 	               Cycles_GetCounter(CYCLES_COUNTER_VIDEO), Handler, InterruptHandlers[Handler].Cycles, PendingInterruptCount );
 }
 
@@ -435,7 +444,7 @@ void Int_ResumeStoppedInterrupt(interrupt_id Handler)
 /**
  * Return TRUE if interrupt is active in list
  */
-BOOL Int_InterruptActive(interrupt_id Handler)
+bool Int_InterruptActive(interrupt_id Handler)
 {
 	/* Is timer active? */
 	if (InterruptHandlers[Handler].bUsed)
@@ -451,12 +460,12 @@ BOOL Int_InterruptActive(interrupt_id Handler)
  */
 int Int_FindCyclesPassed(interrupt_id Handler, int CycleType)
 {
-	int CyclesPassed, CyclesFromLastInterrupt;
+	Sint64 CyclesPassed, CyclesFromLastInterrupt;
 
 	CyclesFromLastInterrupt = InterruptHandlers[ActiveInterrupt].Cycles - PendingInterruptCount;
 	CyclesPassed = InterruptHandlers[Handler].Cycles - CyclesFromLastInterrupt;
 
-	HATARI_TRACE ( HATARI_TRACE_INT , "int find passed cyc video_cyc=%d handler=%d last_cyc=%d passed_cyc=%d\n",
+	HATARI_TRACE ( HATARI_TRACE_INT , "int find passed cyc video_cyc=%d handler=%d last_cyc=%lld passed_cyc=%lld\n",
 	               Cycles_GetCounter(CYCLES_COUNTER_VIDEO), Handler, CyclesFromLastInterrupt, CyclesPassed );
 
 	return INT_CONVERT_FROM_INTERNAL ( CyclesPassed , CycleType ) ;
