@@ -6,16 +6,17 @@
 
   Here we process a key press and the remapping of the scancodes.
 */
-const char Keymap_rcsid[] = "Hatari $Id: keymap.c,v 1.32 2008/01/12 02:04:59 thothy Exp $";
+const char Keymap_fileid[] = "Hatari keymap.c : " __DATE__ " " __TIME__;
 
+#include <ctype.h>
 #include "main.h"
 #include "keymap.h"
-#include "misc.h"
 #include "configuration.h"
 #include "file.h"
 #include "ikbd.h"
 #include "joy.h"
 #include "shortcut.h"
+#include "str.h"
 #include "screen.h"
 #include "debugui.h"
 #include "log.h"
@@ -231,7 +232,7 @@ static const char SymbolicKeyToSTScanCode[SDLK_LAST] =
 	/* End of ASCII mapped keysyms */
 	-1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, /* 128-143*/
 	-1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, /* 144-159*/
-	-1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, /* 160-175*/
+	0x0d, 0x0c, 0x1a, 0x28, 0x27, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, /* 160-175*/
 	-1, -1, -1, -1, 0x0D, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, /* 176-191*/
 	-1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, /* 192-207*/
 	-1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, 0x0C, /* 208-223*/
@@ -254,7 +255,7 @@ static const char SymbolicKeyToSTScanCode[SDLK_LAST] =
 	0x4A,    /* SDLK_KP_MINUS = 269 */
 	0x4E,    /* SDLK_KP_PLUS = 270 */
 	0x72,    /* SDLK_KP_ENTER = 271 */
-	-1,      /* SDLK_KP_EQUALS = 272 */
+	0x61,    /* SDLK_KP_EQUALS = 272 */
 	/* Arrows + Home/End pad */
 	0x48,    /* SDLK_UP = 273 */
 	0x50,    /* SDLK_DOWN = 274 */
@@ -278,7 +279,7 @@ static const char SymbolicKeyToSTScanCode[SDLK_LAST] =
 	0x44,    /* SDLK_F10 = 291 */
 	-1,      /* SDLK_F11 = 292 */
 	-1,      /* SDLK_F12 = 293 */
-	-1,      /* SDLK_F13 = 294 */
+	0x62,    /* SDLK_F13 = 294 */
 	-1,      /* SDLK_F14 = 295 */
 	-1,      /* SDLK_F15 = 296 */
 	-1,      /* 297 */
@@ -598,41 +599,45 @@ void Keymap_LoadRemapFile(char *pszFileName)
 	/* Initialize table with default values */
 	memcpy(LoadedKeyToSTScanCode, SymbolicKeyToSTScanCode, sizeof(LoadedKeyToSTScanCode));
 
+	if (!*pszFileName)
+		return;
+	
 	/* Attempt to load file */
-	if (strlen(pszFileName) > 0 && !File_DoesFileNameEndWithSlash(pszFileName))
+	if (!File_Exists(pszFileName))
 	{
-		/* Open file */
-		in = fopen(pszFileName, "r");
-		if (!in)
-		{
-			Log_Printf(LOG_DEBUG, "Keymap_LoadRemapFile: failed to "
-				      " open keymap file '%s'\n", pszFileName);
-			return;
-		}
-
-		while (!feof(in))
-		{
-			/* Read line from file */
-			if (fgets(szString, sizeof(szString), in) == NULL)
-				break;
-			/* Remove white-space from start of line */
-			Misc_RemoveWhiteSpace(szString, sizeof(szString));
-			if (strlen(szString)>0)
-			{
-				/* Is a comment? */
-				if ( (szString[0]==';') || (szString[0]=='#') )
-					continue;
-				/* Read values */
-				sscanf(szString, "%d,%d", &PCKeyCode, &STScanCode);
-				/* Store into remap table, check both value within range */
-				if ( (PCKeyCode>=0) && (PCKeyCode<SDLK_LAST) &&
-				        (STScanCode>=0) && (STScanCode<256) )
-					LoadedKeyToSTScanCode[PCKeyCode] = STScanCode;
-			}
-		}
-
-		fclose(in);
+		Log_Printf(LOG_DEBUG, "Keymap_LoadRemapFile: '%s' not a file\n", pszFileName);
+		return;
 	}
+	in = fopen(pszFileName, "r");
+	if (!in)
+	{
+		Log_Printf(LOG_DEBUG, "Keymap_LoadRemapFile: failed to "
+			   " open keymap file '%s'\n", pszFileName);
+		return;
+	}
+	
+	while (!feof(in))
+	{
+		/* Read line from file */
+		if (fgets(szString, sizeof(szString), in) == NULL)
+			break;
+		/* Remove white-space from start of line */
+		Str_Trim(szString);
+		if (strlen(szString)>0)
+		{
+			/* Is a comment? */
+			if ( (szString[0]==';') || (szString[0]=='#') )
+				continue;
+			/* Read values */
+			sscanf(szString, "%d,%d", &PCKeyCode, &STScanCode);
+			/* Store into remap table, check both value within range */
+			if ( (PCKeyCode>=0) && (PCKeyCode<SDLK_LAST) &&
+			     (STScanCode>=0) && (STScanCode<256) )
+				LoadedKeyToSTScanCode[PCKeyCode] = STScanCode;
+		}
+	}
+	
+	fclose(in);
 }
 
 
@@ -641,12 +646,12 @@ void Keymap_LoadRemapFile(char *pszFileName)
  * Scan list of keys to NOT de-bounce when running in maximum speed, eg ALT,SHIFT,CTRL etc...
  * Return TRUE if key requires de-bouncing
  */
-static BOOL Keymap_DebounceSTKey(char STScanCode)
+static bool Keymap_DebounceSTKey(char STScanCode)
 {
 	int i=0;
 
-	/* Are we in maximum speed, and have disabled key repeat? */
-	if ((ConfigureParams.System.nMinMaxSpeed!=MINMAXSPEED_MIN)
+	/* Are we in fast forward, and have disabled key repeat? */
+	if ((ConfigureParams.System.bFastForward == TRUE)
 	    && (ConfigureParams.Keyboard.bDisableKeyRepeat))
 	{
 		/* We should de-bounce all non extended keys,
@@ -679,8 +684,8 @@ void Keymap_DebounceAllKeys(void)
 	char STScanCode;
 	SDL_keysym tmpKeySym;
 
-	/* Return if we aren't in maximum speed or have not disabled key repeat */
-	if ((ConfigureParams.System.nMinMaxSpeed == MINMAXSPEED_MIN)
+	/* Return if we aren't in fast forward or have not disabled key repeat */
+	if ((ConfigureParams.System.bFastForward == FALSE)
 	        || (!ConfigureParams.Keyboard.bDisableKeyRepeat))
 	{
 		return;
@@ -717,7 +722,7 @@ void Keymap_DebounceAllKeys(void)
  */
 void Keymap_KeyDown(SDL_keysym *sdlkey)
 {
-	BOOL bPreviousKeyState;
+	bool bPreviousKeyState;
 	char STScanCode;
 	int symkey = sdlkey->sym;
 	int modkey = sdlkey->mod;
@@ -739,13 +744,6 @@ void Keymap_KeyDown(SDL_keysym *sdlkey)
 		/* Ignore modifier keys that aren't passed to the ST */
 		return;
 	}
-	else if (symkey == SDLK_PAUSE && bEnableDebug)
-	{
-		/* Call the debugger */
-		if (bInFullScreen)  Screen_ReturnFromFullScreen();
-		DebugUI();
-		return;
-	}
 
 	/* Set down */
 	bPreviousKeyState = Keyboard.KeyStates[symkey];
@@ -808,3 +806,33 @@ void Keymap_KeyUp(SDL_keysym *sdlkey)
 	Keyboard.KeyStates[symkey] = FALSE;
 }
 
+/*-----------------------------------------------------------------------*/
+/**
+ * Simulate press or release of a key corresponding to given character
+ */
+void Keymap_SimulateCharacter(char asckey, bool press)
+{
+	SDL_keysym sdlkey;
+
+	sdlkey.mod = 0;
+	sdlkey.scancode = 0;
+	if (isupper(asckey)) {
+		if (press) {
+			sdlkey.sym = SDLK_LSHIFT;
+			Keymap_KeyDown(&sdlkey);
+		}
+		sdlkey.sym = tolower(asckey);
+		sdlkey.mod = SDLK_LSHIFT;
+	} else {
+		sdlkey.sym = asckey;
+	}
+	if (press) {
+		Keymap_KeyDown(&sdlkey);
+	} else {
+		Keymap_KeyUp(&sdlkey);
+		if (isupper(asckey)) {
+			sdlkey.sym = SDLK_LSHIFT;
+			Keymap_KeyUp(&sdlkey);
+		}
+	}
+}
