@@ -28,27 +28,32 @@
 /*			After Demo).									*/
 /* 2008/02/11	[NP]	Add pairing for MULS/MOVEA (Delirious Demo IV Loader).				*/
 /* 2008/01/25	[NP]	Add pairing for LSR/MOVEA (and all other bit shifting instr) (Decade Demo Reset)*/
-/* 2008/02/16	[NP]	Add pairing for MULS/DIVS, but not 100% sure. This fixes e605 demo part 3, but	*/
-/*			this could be due to another missing pairing. FIXME : This needs to be checked	*/
-/*			on a real ST.									*/
+/* 2008/02/16	[NP]	Add pairing for MULS/DIVS (fixes e605 demo part 3).				*/
 /* 2008/03/08	[NP]	In M68000_Exception, we need to know if the exception was triggered by an MFP	*/
 /*			interrupt or by a video interrupt. In the case MFP vector base was changed in	*/
 /*			fffa17 to an other value than the default $40, testing exceptionNr is not enough*/
 /*			to correctly process the exception. For example, if vector base is set to $10	*/
 /*			then MFP Timer A will call vector stored at address $74, which would be wrongly	*/
 /*			interpreted as a level 5 int (which doesn't exist on Atari and will cause an	*/
-/*			assert to fail in intlevel()). We use InterruptType to correctly recognize the	*/
+/*			assert to fail in intlevel()). We use InterruptSource to correctly recognize the*/
 /*			MFP interrupts (fix 'Toki' end part fullscreen which sets vector base to $10).	*/
+/* 2008/04/14	[NP]	Add pairing for BTST/Bcc (eg btst #7,d0 + bne.s label  (with branch taken)).	*/
+/* 2008/04/15	[NP]	As tested on a real STF :							*/
+/*				- MUL/DIV can pair (but DIV/MUL can't)					*/
+/*					(eg mulu d0,d0 + divs d1,d1 with d0=0 and d1=1)			*/
+/*				- MUL/MOVE can pair, but not DIV/MOVE					*/
+/*				- EXG/MOVE can pair (eg exg d3,d4 + move.l 0(a3,d1.w),a4)		*/
+/*				- MOVE/DBcc can't pair							*/
+/* 2008/04/16	[NP]	Functions 'M68000_InitPairing_BitShift' to ease code maintenance.		*/
+/*			Tested on STF : add pairing between bit shift instr and ADD/SUB/OR/AND/EOR/NOT	*/
+/*			CLR/NEG (certainly some more possible, haven't tested everything)		*/
+/*			(fixes lsr.w #4,d4 + add.b $f0(a4,d4),d7 used in Zoolook part of ULM New Year).	*/
+/* 2008/07/08	[NP]	Add pairing between bit shift instr and ADDX/SUBX/ABCD/SBCD (fixes lsl.l #1,d0	*/
+/*			+ abcd d1,d1 used in Dragonnels - Rainbow Wall).				*/
+/* 2008/10/05	[NP]	Pass the 'ExceptionSource' parameter to Exception() in uae-cpu/newcpu.c		*/
 
 
-/* [NP] possible pairing to check :             */
-/*      exg / move.b (a0),d0                    */
-/*	div / move				*/
-/*	btst / bxx				*/
-/*	mul/div div/mul				*/
-
-
-const char M68000_rcsid[] = "Hatari $Id: m68000.c,v 1.56 2008/03/09 12:53:28 npomarede Exp $";
+const char M68000_fileid[] = "Hatari m68000.c : " __DATE__ " " __TIME__;
 
 #include "main.h"
 #include "configuration.h"
@@ -66,9 +71,10 @@ const char M68000_rcsid[] = "Hatari $Id: m68000.c,v 1.56 2008/03/09 12:53:28 npo
 
 Uint32 BusErrorAddress;         /* Stores the offending address for bus-/address errors */
 Uint32 BusErrorPC;              /* Value of the PC when bus error occurs */
-BOOL bBusErrorReadWrite;        /* 0 for write error, 1 for read error */
+bool bBusErrorReadWrite;        /* 0 for write error, 1 for read error */
 int nCpuFreqShift;              /* Used to emulate higher CPU frequencies: 0=8MHz, 1=16MHz, 2=32Mhz */
 int nWaitStateCycles;           /* Used to emulate the wait state cycles of certain IO registers */
+int BusMode = BUS_MODE_CPU;	/* Used to tell which part is owning the bus (cpu, blitter, ...) */
 
 int LastOpcodeFamily = -1;      /* see the enum in readcpu.h i_XXX */
 int LastInstrCycles = -1;       /* number of cycles for previous instr. (not rounded to 4) */
@@ -104,6 +110,24 @@ const char *OpcodeName[] = { "ILLG",
 };
 
 
+/*-----------------------------------------------------------------------*/
+/**
+ * Add pairing between all the bit shifting instructions and a given Opcode
+ */
+
+static void M68000_InitPairing_BitShift ( int OpCode )
+{
+	PairingArray[  i_ASR ][ OpCode ] = 1; 
+	PairingArray[  i_ASL ][ OpCode ] = 1; 
+	PairingArray[  i_LSR ][ OpCode ] = 1; 
+	PairingArray[  i_LSL ][ OpCode ] = 1; 
+	PairingArray[  i_ROL ][ OpCode ] = 1; 
+	PairingArray[  i_ROR ][ OpCode ] = 1; 
+	PairingArray[ i_ROXR ][ OpCode ] = 1; 
+	PairingArray[ i_ROXL ][ OpCode ] = 1; 
+}
+
+
 /*-----------------------------------------------------------------------*/
 /**
  * Init the pairing matrix
@@ -116,45 +140,41 @@ void M68000_InitPairing(void)
 
 	/* Set all valid pairing combinations to 1 */
 	PairingArray[  i_EXG ][ i_DBcc ] = 1;
+	PairingArray[  i_EXG ][ i_MOVE ] = 1;
+	PairingArray[  i_EXG ][ i_MOVEA] = 1;
+
 	PairingArray[ i_CMPA ][  i_Bcc ] = 1;
-	PairingArray[  i_ASR ][ i_DBcc ] = 1;
-	PairingArray[  i_ASL ][ i_DBcc ] = 1;
-	PairingArray[  i_LSR ][ i_DBcc ] = 1;
-	PairingArray[  i_LSL ][ i_DBcc ] = 1;
-	PairingArray[  i_ROL ][ i_DBcc ] = 1;
-	PairingArray[  i_ROR ][ i_DBcc ] = 1;
-	PairingArray[ i_ROXR ][ i_DBcc ] = 1;
-	PairingArray[ i_ROXL ][ i_DBcc ] = 1;
-	PairingArray[  i_ASR ][ i_MOVE ] = 1; 
-	PairingArray[  i_ASL ][ i_MOVE ] = 1; 
-	PairingArray[  i_LSR ][ i_MOVE ] = 1; 
-	PairingArray[  i_LSL ][ i_MOVE ] = 1; 
-	PairingArray[  i_ROL ][ i_MOVE ] = 1; 
-	PairingArray[  i_ROR ][ i_MOVE ] = 1; 
-	PairingArray[ i_ROXR ][ i_MOVE ] = 1; 
-	PairingArray[ i_ROXL ][ i_MOVE ] = 1; 
-	PairingArray[  i_ASR ][ i_MOVEA] = 1; 
-	PairingArray[  i_ASL ][ i_MOVEA] = 1; 
-	PairingArray[  i_LSR ][ i_MOVEA] = 1; 
-	PairingArray[  i_LSL ][ i_MOVEA] = 1; 
-	PairingArray[  i_ROL ][ i_MOVEA] = 1; 
-	PairingArray[  i_ROR ][ i_MOVEA] = 1; 
-	PairingArray[ i_ROXR ][ i_MOVEA] = 1; 
-	PairingArray[ i_ROXL ][ i_MOVEA] = 1; 
 	PairingArray[  i_CMP ][  i_Bcc ] = 1;
-	PairingArray[  i_ASR ][  i_LEA ] = 1; 
-	PairingArray[  i_ASL ][  i_LEA ] = 1; 
-	PairingArray[  i_LSR ][  i_LEA ] = 1; 
-	PairingArray[  i_LSL ][  i_LEA ] = 1; 
-	PairingArray[  i_ROL ][  i_LEA ] = 1; 
-	PairingArray[  i_ROR ][  i_LEA ] = 1; 
-	PairingArray[ i_ROXR ][  i_LEA ] = 1; 
-	PairingArray[ i_ROXL ][  i_LEA ] = 1; 
+
+	M68000_InitPairing_BitShift ( i_DBcc );
+	M68000_InitPairing_BitShift ( i_MOVE );
+	M68000_InitPairing_BitShift ( i_MOVEA );
+	M68000_InitPairing_BitShift ( i_LEA );
+
 	PairingArray[ i_MULU ][ i_MOVEA] = 1; 
 	PairingArray[ i_MULS ][ i_MOVEA] = 1; 
 	PairingArray[ i_MULU ][ i_MOVE ] = 1; 
 	PairingArray[ i_MULS ][ i_MOVE ] = 1; 
-	PairingArray[ i_MULS ][ i_DIVS ] = 1; 		/* not 100% sure of this one */
+
+	PairingArray[ i_MULU ][ i_DIVU ] = 1;
+	PairingArray[ i_MULU ][ i_DIVS ] = 1;
+	PairingArray[ i_MULS ][ i_DIVU ] = 1;
+	PairingArray[ i_MULS ][ i_DIVS ] = 1;
+
+	PairingArray[ i_BTST ][  i_Bcc ] = 1;
+
+	M68000_InitPairing_BitShift ( i_ADD );
+	M68000_InitPairing_BitShift ( i_SUB );
+	M68000_InitPairing_BitShift ( i_OR );
+	M68000_InitPairing_BitShift ( i_AND );
+	M68000_InitPairing_BitShift ( i_EOR );
+	M68000_InitPairing_BitShift ( i_NOT );
+	M68000_InitPairing_BitShift ( i_CLR );
+	M68000_InitPairing_BitShift ( i_NEG );
+	M68000_InitPairing_BitShift ( i_ADDX );
+	M68000_InitPairing_BitShift ( i_SUBX );
+	M68000_InitPairing_BitShift ( i_ABCD );
+	M68000_InitPairing_BitShift ( i_SBCD );
 }
 
 
@@ -162,7 +182,7 @@ void M68000_InitPairing(void)
 /**
  * Reset CPU 68000 variables
  */
-void M68000_Reset(BOOL bCold)
+void M68000_Reset(bool bCold)
 {
 	int i;
 
@@ -178,6 +198,8 @@ void M68000_Reset(BOOL bCold)
 
 	/* Init the pairing matrix */
 	M68000_InitPairing();
+
+	BusMode = BUS_MODE_CPU;
 }
 
 
@@ -223,7 +245,7 @@ void M68000_CheckCpuLevel(void)
 /**
  * Save/Restore snapshot of CPU variables ('MemorySnapShot_Store' handles type)
  */
-void M68000_MemorySnapShot_Capture(BOOL bSave)
+void M68000_MemorySnapShot_Capture(bool bSave)
 {
 	Uint32 savepc;
 
@@ -324,7 +346,7 @@ void M68000_MemorySnapShot_Capture(BOOL bSave)
  * BUSERROR - Access outside valid memory range.
  * Use bReadWrite = 0 for write errors and bReadWrite = 1 for read errors!
  */
-void M68000_BusError(Uint32 addr, BOOL bReadWrite)
+void M68000_BusError(Uint32 addr, bool bReadWrite)
 {
 	/* FIXME: In prefetch mode, m68k_getpc() seems already to point to the next instruction */
 	// BusErrorPC = M68000_GetPC();		/* [NP] We set BusErrorPC in m68k_run_1 */
@@ -348,11 +370,11 @@ void M68000_BusError(Uint32 addr, BOOL bReadWrite)
 /**
  * Exception handler
  */
-void M68000_Exception(Uint32 ExceptionVector , int InterruptType)
+void M68000_Exception(Uint32 ExceptionVector , int ExceptionSource)
 {
 	int exceptionNr = ExceptionVector/4;
 
-	if ( ( InterruptType == M68000_INT_VIDEO )
+	if ( ( ExceptionSource == M68000_EXCEPTION_SRC_INT_VIDEO )
 		&& (exceptionNr>24 && exceptionNr<32) )	/* 68k autovector interrupt? */
 	{
 		/* Handle autovector interrupts the UAE's way
@@ -362,7 +384,8 @@ void M68000_Exception(Uint32 ExceptionVector , int InterruptType)
 		pendingInterrupts |= (1 << intnr);
 		M68000_SetSpecial(SPCFLAG_INT);
 	}
-	else
+
+	else							/* MFP or direct CPU exceptions */
 	{
 		Uint16 SR;
 
@@ -375,7 +398,7 @@ void M68000_Exception(Uint32 ExceptionVector , int InterruptType)
 
 		/* 68k exceptions are handled by Exception() of the UAE CPU core */
 #ifdef UAE_NEWCPU_H
-		Exception(exceptionNr, m68k_getpc());
+		Exception(exceptionNr, m68k_getpc(), ExceptionSource);
 #else
 		Exception(exceptionNr, &regs, m68k_getpc(&regs));
 #endif
