@@ -6,9 +6,6 @@
 
   Main initialization and event handling routines.
 */
-const char Opt_rcsid[] = "Hatari $Id: main.c,v 1.121 2008/03/28 22:33:11 eerot Exp $";
-
-#include "config.h"
 
 #include <time.h>
 #include <SDL.h>
@@ -16,21 +13,21 @@ const char Opt_rcsid[] = "Hatari $Id: main.c,v 1.121 2008/03/28 22:33:11 eerot E
 #include "config.h"
 #include "main.h"
 #include "configuration.h"
+#include "control.h"
 #include "options.h"
 #include "dialog.h"
 #include "audio.h"
 #include "joy.h"
-#include "file.h"
 #include "floppy.h"
 #include "gemdos.h"
 #include "hdc.h"
+#include "ide.h"
 #include "ikbd.h"
 #include "ioMem.h"
 #include "keymap.h"
 #include "log.h"
 #include "m68000.h"
 #include "memorySnapShot.h"
-#include "misc.h"
 #include "midi.h"
 #include "nvram.h"
 #include "paths.h"
@@ -41,12 +38,11 @@ const char Opt_rcsid[] = "Hatari $Id: main.c,v 1.121 2008/03/28 22:33:11 eerot E
 #include "sdlgui.h"
 #include "shortcut.h"
 #include "sound.h"
+#include "statusbar.h"
 #include "stMemory.h"
+#include "str.h"
 #include "tos.h"
-#include "vdi.h"
 #include "video.h"
-#include "ymFormat.h"
-#include "debugui.h"
 
 #include "hatari-glue.h"
 
@@ -56,21 +52,19 @@ const char Opt_rcsid[] = "Hatari $Id: main.c,v 1.121 2008/03/28 22:33:11 eerot E
 #endif
 
 
-BOOL bQuitProgram = FALSE;                /* Flag to quit program cleanly */
-BOOL bEnableDebug = FALSE;                /* Enable debug UI? */
-static BOOL bEmulationActive = TRUE;      /* Run emulation when started */
-static BOOL bAccurateDelays;              /* Host system has an accurate SDL_Delay()? */
-static BOOL bIgnoreNextMouseMotion = FALSE;  /* Next mouse motion will be ignored (needed after SDL_WarpMouse) */
-static char szBootDiskImage[FILENAME_MAX];   /* boot disk path or empty */
+bool bQuitProgram = FALSE;                /* Flag to quit program cleanly */
+bool bEnableDebug = FALSE;                /* Enable debug UI? */
 
-extern char sWorkingDir[FILENAME_MAX];
+static bool bEmulationActive = TRUE;      /* Run emulation when started */
+static bool bAccurateDelays;              /* Host system has an accurate SDL_Delay()? */
+static bool bIgnoreNextMouseMotion = FALSE;  /* Next mouse motion will be ignored (needed after SDL_WarpMouse) */
 
 
 /*-----------------------------------------------------------------------*/
 /**
  * Save/Restore snapshot of local variables('MemorySnapShot_Store' handles type)
  */
-void Main_MemorySnapShot_Capture(BOOL bSave)
+void Main_MemorySnapShot_Capture(bool bSave)
 {
 	int nBytes;
 
@@ -88,39 +82,51 @@ void Main_MemorySnapShot_Capture(BOOL bSave)
 		MemorySnapShot_Store(STRam, nBytes);
 	}
 	/* And Cart/TOS/Hardware area */
-	MemorySnapShot_Store(&STRam[0xE00000], 0x200000);
-	MemorySnapShot_Store(szBootDiskImage, sizeof(szBootDiskImage));
-	MemorySnapShot_Store(sWorkingDir, sizeof(sWorkingDir));  /* TODO: Remove this line? */
+	MemorySnapShot_Store(&RomMem[0xE00000], 0x200000);
 }
 
 
 /*-----------------------------------------------------------------------*/
 /**
- * Pause emulation, stop sound
+ * Pause emulation, stop sound.  'visualize' should be set TRUE,
+ * unless unpause will be called immediately afterwards.
+ * 
+ * Return TRUE if paused now, FALSE if was already paused
  */
-void Main_PauseEmulation(void)
-{
-	if ( bEmulationActive )
-	{
-		Audio_EnableAudio(FALSE);
-		bEmulationActive = FALSE;
-	}
-}
-
-/*-----------------------------------------------------------------------*/
-/**
- * Start emulation
- */
-void Main_UnPauseEmulation(void)
+bool Main_PauseEmulation(bool visualize)
 {
 	if ( !bEmulationActive )
-	{
-		Sound_ResetBufferIndex();
-		Audio_EnableAudio(ConfigureParams.Sound.bEnableSound);
-		Screen_SetFullUpdate();       /* Cause full screen update (to clear all) */
+		return FALSE;
 
-		bEmulationActive = TRUE;
+	Audio_EnableAudio(FALSE);
+	bEmulationActive = FALSE;
+	if (visualize)
+	{
+		Statusbar_AddMessage("Emulation paused", 100);
+		/* make sure msg gets shown */
+		Statusbar_Update(sdlscrn);
 	}
+	return TRUE;
+}
+
+/*-----------------------------------------------------------------------*/
+/**
+ * Start/continue emulation
+ * 
+ * Return TRUE if continued, FALSE if was already running
+ */
+bool Main_UnPauseEmulation(void)
+{
+	if ( bEmulationActive )
+		return FALSE;
+
+	Sound_ResetBufferIndex();
+	Audio_EnableAudio(ConfigureParams.Sound.bEnableSound);
+	bEmulationActive = TRUE;
+
+	/* Cause full screen update (to clear all) */
+	Screen_SetFullUpdate();
+	return TRUE;
 }
 
 /*-----------------------------------------------------------------------*/
@@ -172,14 +178,29 @@ void Main_WaitOnVbl(void)
 	nFrameDuration = 1000/nScreenRefreshRate;
 	nDelay = nDestMilliTicks - nCurrentMilliTicks;
 
-	/* Do not wait if we are in max speed mode or if we are totally out of sync */
-	if (ConfigureParams.System.nMinMaxSpeed == MINMAXSPEED_MAX
+	/* Do not wait if we are in fast forward mode or if we are totally out of sync */
+	if (ConfigureParams.System.bFastForward == TRUE
 	        || nDelay < -4*nFrameDuration)
 	{
+		if (nFrameSkips < ConfigureParams.Screen.nFrameSkips)
+		{
+			nFrameSkips += 1;
+			Log_Printf(LOG_DEBUG, "Increased frameskip to %d\n", nFrameSkips);
+		}
 		/* Only update nDestMilliTicks for next VBL */
 		nDestMilliTicks = nCurrentMilliTicks + nFrameDuration;
 		return;
 	}
+	/* If automatic frameskip is enabled and delay's more than twice
+	 * the effect of single frameskip, decrease frameskip
+	 */
+	if (nFrameSkips > 0
+	    && ConfigureParams.Screen.nFrameSkips >= AUTO_FRAMESKIP_LIMIT
+	    && 2*nDelay > nFrameDuration/nFrameSkips)
+	{
+		nFrameSkips -= 1;
+		Log_Printf(LOG_DEBUG, "Decreased frameskip to %d\n", nFrameSkips);
+	}
 
 	if (bAccurateDelays)
 	{
@@ -228,7 +249,7 @@ static void Main_CheckForAccurateDelays(void)
 	if (bAccurateDelays)
 		Log_Printf(LOG_DEBUG, "Host system has accurate delays. (%d)\n", nEndTicks - nStartTicks);
 	else
-		Log_Printf(LOG_DEBUG, "Host system does not have accurate delays. (%d)\n", nEndTicks - nStartTicks);
+		Log_Printf(LOG_WARN, "Host system does not have accurate delays. (%d)\n", nEndTicks - nStartTicks);
 }
 
 
@@ -295,9 +316,29 @@ static void Main_HandleMouseMotion(SDL_Event *pEvent)
 void Main_EventHandler(void)
 {
 	SDL_Event event;
-
-	if (SDL_PollEvent(&event))
+	int ok;
+	
+	do
 	{
+		/* check remote process control */
+		int remotepause = Control_CheckUpdates();
+
+		if ( bEmulationActive || remotepause )
+		{
+			ok = SDL_PollEvent(&event);
+		}
+		else
+		{
+			ShortCut_ActKey();
+			/* last (shortcut) event activated emulation? */
+			if ( bEmulationActive )
+				break;
+			ok = SDL_WaitEvent(&event);
+		}
+		if (!ok)
+		{
+			continue;
+		}
 		switch (event.type)
 		{
 
@@ -365,60 +406,7 @@ void Main_EventHandler(void)
 			Keymap_KeyUp(&event.key.keysym);
 			break;
 		}
-	}
-}
-
-
-/*-----------------------------------------------------------------------*/
-/**
- * Reparent Hatari window if so requested.  Needs to be done inside
- * Hatari because if SDL itself is requested to reparent itself,
- * SDL window stops accepting any input (specifically done like
- * this in SDL backends for some reason).
- * 
- * Currently only works on X11.
- * 
- * SDL_syswm.h automatically includes everything else needed.
- */
-
-#if HAVE_X11
-#include <SDL_syswm.h>
-#endif
-
-static void Main_Reparent_Window(void)
-{
-#if HAVE_X11
-	Window parent_win;
-	const char *parent_win_id;
-	SDL_SysWMinfo info;
-
-	parent_win_id = getenv("PARENT_WIN_ID");
-	if (!parent_win_id) {
-		return;
-	}
-	parent_win = strtol(parent_win_id, NULL, 0);
-	if (!parent_win) {
-		Log_Printf(LOG_DEBUG, "Invalid PARENT_WIN_ID value '%s'\n", parent_win_id);
-		return;
-	}
-
-	SDL_VERSION(&info.version);
-	if (!SDL_GetWMInfo(&info)) {
-		Log_Printf(LOG_DEBUG, "Failed to get SDL_GetWMInfo()\n");
-		return;
-	}
-
-	/* hide WM window for Hatari */
-	XUnmapWindow(info.info.x11.display,
-			info.info.x11.wmwindow);
-	/* reparent main Hatari window to given parent */
-	XReparentWindow(info.info.x11.display,
-			info.info.x11.window,
-			parent_win, 0, 0);
-#else
-	/* TODO: implement the Windows part.  SDL sources offer example */
-	//Log_Printf(LOG_DEBUG, "Support for Hatari window reparenting not built in\n");
-#endif /* HAVE_X11 */
+	} while (!(bEmulationActive || bQuitProgram));
 }
 
 
@@ -429,7 +417,11 @@ static void Main_Reparent_Window(void)
 static void Main_Init(void)
 {
 	/* Open debug log file */
-	Log_Init();
+	if (!Log_Init())
+	{
+		fprintf(stderr, "Logging/tracing initialization failed\n");
+		exit(-1);
+	}
 	Log_Printf(LOG_INFO, PROG_NAME ", compiled on:  " __DATE__ ", " __TIME__ "\n");
 
 	/* Init SDL's video subsystem. Note: Audio and joystick subsystems
@@ -445,7 +437,6 @@ static void Main_Init(void)
 	RS232_Init();
 	Midi_Init();
 	Screen_Init();
-	Main_Reparent_Window();
 	HostScreen_Init();
 #if ENABLE_DSP_EMU
 	if (ConfigureParams.System.nDSPType == DSP_TYPE_EMU)
@@ -467,6 +458,7 @@ static void Main_Init(void)
 		else
 			printf("Couldn't open HD file: %s, or no partitions\n", szHardDiskImage);
 	}
+	Ide_Init();
 	GemDOS_Init();
 	if (ConfigureParams.HardDisk.bUseHardDiskDirectories)
 	{
@@ -490,12 +482,6 @@ static void Main_Init(void)
 	NvRam_Init();
 	Joy_Init();
 	Sound_Init();
-
-	/* Check passed disk image parameter, boot directly into emulator */
-	if (strlen(szBootDiskImage) > 0)
-	{
-		Floppy_InsertDiskIntoDrive(0, szBootDiskImage, sizeof(szBootDiskImage));
-	}
 }
 
 
@@ -514,6 +500,7 @@ static void Main_UnInit(void)
 	IoMem_UnInit();
 	NvRam_UnInit();
 	GemDOS_UnInitDrives();
+	Ide_UnInit();
 	Joy_UnInit();
 	if (Sound_AreWeRecording())
 		Sound_EndRecording();
@@ -524,8 +511,8 @@ static void Main_UnInit(void)
 	{
 		DSP_UnInit();
 	}
-	HostScreen_UnInit();
 #endif
+	HostScreen_UnInit();
 	Screen_UnInit();
 	Exit680x0();
 
@@ -537,9 +524,64 @@ static void Main_UnInit(void)
 }
 
 
+/*-----------------------------------------------------------------------*/
+/**
+ * Load initial configuration file(s)
+ */
+static void Main_LoadInitialConfig(void)
+{
+	char *psGlobalConfig;
+
+	psGlobalConfig = malloc(FILENAME_MAX);
+	if (psGlobalConfig)
+	{
+#if defined(__AMIGAOS4__)
+		strncpy(psGlobalConfig, CONFDIR"hatari.cfg", FILENAME_MAX);
+#else
+		snprintf(psGlobalConfig, FILENAME_MAX, CONFDIR"%chatari.cfg", PATHSEP);
+#endif
+		/* Try to load the global configuration file */
+		Configuration_Load(psGlobalConfig);
+
+		free(psGlobalConfig);
+	}
+
+	/* Now try the users configuration file */
+	Configuration_Load(NULL);
+}
+
+/*-----------------------------------------------------------------------*/
+/**
+ * Set TOS etc information and initial help message
+ */
+static void Main_StatusbarSetup(void)
+{
+	const char *name = NULL;
+	SDLKey key;
+
+	key = ConfigureParams.Shortcut.withoutModifier[SHORTCUT_OPTIONS];
+	if (!key)
+		key = ConfigureParams.Shortcut.withModifier[SHORTCUT_OPTIONS];
+	if (key)
+		name = SDL_GetKeyName(key);
+	if (name)
+	{
+		char message[24], *keyname;
+		keyname = Str_ToUpper(strdup(name));
+		snprintf(message, sizeof(message), "Press %s for Options", keyname);
+		free(keyname);
+
+		Statusbar_AddMessage(message, 6000);
+	}
+	/* update information loaded by Main_Init() */
+	Statusbar_UpdateInfo();
+}
+
 /*-----------------------------------------------------------------------*/
 /**
  * Main
+ * 
+ * Note: 'argv' cannot be declared const, MinGW would then fail to link.
  */
 int main(int argc, char *argv[])
 {
@@ -549,18 +591,17 @@ int main(int argc, char *argv[])
 	/* Initialize directory strings */
 	Paths_Init(argv[0]);
 
-	/* no boot disk image */
-	szBootDiskImage[0] = 0;
-
 	/* Set default configuration values: */
 	Configuration_SetDefault();
 
 	/* Now load the values from the configuration file */
-	Configuration_Load(CONFDIR"/hatari.cfg");     /* Try the global configuration file first */
-	Configuration_Load(NULL);                     /* Now try the users configuration file */
+	Main_LoadInitialConfig();
 
-	/* Check for any passed parameters, get boot disk */
-	Opt_ParseParameters(argc, argv, szBootDiskImage, sizeof(szBootDiskImage));
+	/* Check for any passed parameters */
+	if (!Opt_ParseParameters(argc, (const char**)argv))
+	{
+		return 1;
+	}
 	/* monitor type option might require "reset" -> TRUE */
 	Configuration_Apply(TRUE);
 
@@ -578,6 +619,9 @@ int main(int argc, char *argv[])
 	/* Init emulator system */
 	Main_Init();
 
+	/* Set initial Statusbar information */
+	Main_StatusbarSetup();
+	
 	/* Check if SDL_Delay is accurate */
 	Main_CheckForAccurateDelays();
 
