@@ -16,7 +16,7 @@
   reduce redundancy and the function 'MemorySnapShot_Store' decides if it
   should save or restore the data.
 */
-const char MemorySnapShot_rcsid[] = "Hatari $Id: memorySnapShot.c,v 1.39 2008/03/29 23:01:56 thothy Exp $";
+const char MemorySnapShot_fileid[] = "Hatari memorySnapShot.c : " __DATE__ " " __TIME__;
 
 #include <config.h>
 
@@ -50,7 +50,7 @@ const char MemorySnapShot_rcsid[] = "Hatari $Id: memorySnapShot.c,v 1.39 2008/03
 #include "video.h"
 
 
-#define VERSION_STRING      "1.0.0"   /* Version number of compatible memory snapshots - Always 6 bytes (inc' NULL) */
+#define VERSION_STRING      "1.2.0"   /* Version number of compatible memory snapshots - Always 6 bytes (inc' NULL) */
 #define VERSION_STRING_SIZE    6      /* Size of above (inc' NULL) */
 
 
@@ -69,7 +69,7 @@ typedef FILE* MSS_File;
 
 
 static MSS_File CaptureFile;
-static BOOL bCaptureSave, bCaptureError;
+static bool bCaptureSave, bCaptureError;
 
 
 /*-----------------------------------------------------------------------*/
@@ -133,7 +133,7 @@ static int MemorySnapShot_fwrite(MSS_File fhndl, const char *buf, int len)
  * Open/Create snapshot file, and set flag so 'MemorySnapShot_Store' knows
  * how to handle data.
  */
-static BOOL MemorySnapShot_OpenFile(const char *pszFileName, BOOL bSave)
+static bool MemorySnapShot_OpenFile(const char *pszFileName, bool bSave)
 {
 	char VersionString[VERSION_STRING_SIZE];
 
@@ -175,7 +175,7 @@ static BOOL MemorySnapShot_OpenFile(const char *pszFileName, BOOL bSave)
 		if (strcasecmp(VersionString, VERSION_STRING))
 		{
 			/* No, inform user and error */
-			Log_AlertDlg(LOG_WARN, "Unable to Restore Memory State.\nFile is "
+			Log_AlertDlg(LOG_ERROR, "Unable to Restore Memory State.\nFile is "
 			                       "only compatible with Hatari v%s", VersionString);
 			bCaptureError = TRUE;
 			return(FALSE);
@@ -225,7 +225,7 @@ void MemorySnapShot_Store(void *pData, int Size)
 /**
  * Save 'snapshot' of memory/chips/emulation variables
  */
-void MemorySnapShot_Capture(const char *pszFileName, BOOL bConfirm)
+void MemorySnapShot_Capture(const char *pszFileName, bool bConfirm)
 {
 	/* Set to 'saving' */
 	if (MemorySnapShot_OpenFile(pszFileName,TRUE))
@@ -263,7 +263,7 @@ void MemorySnapShot_Capture(const char *pszFileName, BOOL bConfirm)
 /**
  * Restore 'snapshot' of memory/chips/emulation variables
  */
-void MemorySnapShot_Restore(const char *pszFileName, BOOL bConfirm)
+void MemorySnapShot_Restore(const char *pszFileName, bool bConfirm)
 {
 	/* Set to 'restore' */
 	if (MemorySnapShot_OpenFile(pszFileName,FALSE))
