@@ -23,7 +23,7 @@
 /*			PendingCyclesOver is the value of  PendingInterruptCount when	*/
 /*			the timer expired.						*/
 /*			- MFP_ReadTimer_AB/CD was wrong (returned the elapsed counter	*/
-/*			changes since start, instead of the remainging counter value).	*/
+/*			changes since start, instead of the remaining counter value).	*/
 /*			(ULM DSOTS Demos and Overscan Demos).				*/
 /* 2007/09/25	[NP]	Replace printf by calls to HATARI_TRACE.			*/
 /* 2007/10/21	[NP]	Use 'Int_AddRelativeInterruptWithOffset' when an MFP timer is	*/
@@ -57,8 +57,42 @@
 /*			processed).							*/
 /* 2008/03/08	[NP]	Add traces when writing to vector register fffa17.		*/
 /*			Use M68000_INT_MFP when calling M68000_Exception().		*/
+/* 2008/04/17	[NP]	Handle the case where Timer B is in event count mode and the	*/
+/*			content of $fffa21 is updated by the end of line signal while a	*/
+/*			read instruction at addr $fffa21 occurs at the same time (before*/
+/*			calling MFP_TimerB_EventCount_Interrupt).			*/
+/*			In that case, we need to return MFP_TB_MAINCOUNTER - 1.		*/
+/*			(fix B.I.G. Demo Screen 1).					*/
+/*			FIXME : this should be handled by Cycles_GetCounterOnReadAccess	*/
+/*			but it's not correctly implemented at the moment.		*/
+/* 2008/04/20	[NP]	In the TRACE call in 'MFP_Exception', replace 'get_long' by	*/
+/*			'STMemory_ReadLong' because 'get_long' produced a bus error	*/
+/*			if we were not already in supervisor mode when the mfp exception*/
+/*			occured. This could cause bus error when restoring snapshot	*/
+/*			of a gemdos program for example if trace mode was activated.	*/
+/* 2008/07/12	[NP]	When stopping an active timer just when the internal data	*/
+/*			counter is going from 1 to 0, the internal data counter will be	*/
+/*			set to 0 (=256) instead of being reloaded with the original	*/
+/*			data value. In case no new value is written to the data reg,	*/
+/*			this means a write > 0 to the control reg will restart the timer*/
+/*			with a counter of 256 ! (fix timer saving routine used by	*/
+/*			ST Cnx in the Punish Your Machine and the Froggies Over The	*/
+/*			Fence (although this routine is in fact buggy)).		*/
+/* 2008/09/13	[NP]	Add some traces when stopping a timer and changing data reg.	*/
+/*			Don't apply timer D patch if timer D ctrl reg is 0.		*/
+/* 2008/10/04	[NP]	In MFP_TimerBData_ReadByte, test for overlap only when nHBL	*/
+/*			is between nStartHBL and nEndHBL (fix Wolfenstein 3D intro).	*/
+/*			In event count mode for timer A and B, set data reg to 255 when	*/
+/*			data reg was 0 (which in fact means 256).			*/
+/* 2008/10/16	[NP]	No need to set data reg to 255 when decrementing a data reg that*/
+/*			was 0, this is already what is implicitly done, because data	*/
+/*			reg for timer A/B is Uint8 (revert 2008/10/04 changes).		*/
+/* 2008/12/11	[NP]	In MFP_CheckPendingInterrupts(), returns TRUE or FALSE instead	*/
+/*			of void, depending on whether at least one MFP interrupt was	*/
+/*			allowed or not.							*/
 
-const char MFP_rcsid[] = "Hatari $Id: mfp.c,v 1.35 2008/03/09 12:53:26 npomarede Exp $";
+
+const char MFP_fileid[] = "Hatari mfp.c : " __DATE__ " " __TIME__;
 
 #include "main.h"
 #include "configuration.h"
@@ -74,9 +108,10 @@ const char MFP_rcsid[] = "Hatari $Id: mfp.c,v 1.35 2008/03/09 12:53:26 npomarede
 #include "psg.h"
 #include "rs232.h"
 #include "sound.h"
+#include "stMemory.h"
 #include "tos.h"
+#include "vdi.h"
 #include "video.h"
-#include "trace.h"
 
 
 /*
@@ -126,12 +161,12 @@ static int TimerDClockCycles=0;
 /* If a timer is stopped then restarted later without writing to the data register, */
 /* we must resume the timer from where we left in the interrupts table, instead of */
 /* computing a new number of clock cycles to restart the interrupt. */
-static BOOL TimerACanResume = FALSE;
-static BOOL TimerBCanResume = FALSE;
-static BOOL TimerCCanResume = FALSE;
-static BOOL TimerDCanResume = FALSE;
+static bool TimerACanResume = FALSE;
+static bool TimerBCanResume = FALSE;
+static bool TimerCCanResume = FALSE;
+static bool TimerDCanResume = FALSE;
 
-BOOL bAppliedTimerDPatch;           /* TRUE if the Timer-D patch has been applied */
+bool bAppliedTimerDPatch;           /* TRUE if the Timer-D patch has been applied */
 static int nTimerDFakeValue;        /* Faked Timer-D data register for the Timer-D patch */
 
 static int PendingCyclesOver = 0;   /* >= 0 value, used to "loop" a timer when data counter reaches 0 */
@@ -189,7 +224,7 @@ void MFP_Reset(void)
 /**
  * Save/Restore snapshot of local variables('MemorySnapShot_Store' handles type)
  */
-void MFP_MemorySnapShot_Capture(BOOL bSave)
+void MFP_MemorySnapShot_Capture(bool bSave)
 {
 	/* Save/Restore details */
 	MemorySnapShot_Store(&MFP_GPIP, sizeof(MFP_GPIP));
@@ -244,13 +279,13 @@ static void MFP_Exception(int Interrupt)
 
 	if ( HATARI_TRACE_LEVEL ( HATARI_TRACE_MFP_EXCEPTION ) )
 	{
-		int nFrameCycles = Cycles_GetCounter(CYCLES_COUNTER_VIDEO);;
+		int nFrameCycles = Cycles_GetCounter(CYCLES_COUNTER_VIDEO);
 		int nLineCycles = nFrameCycles % nCyclesPerLine;
 		HATARI_TRACE_PRINT ( "mfp excep int=%d vec=0x%x new_pc=0x%x video_cyc=%d %d@%d\n" ,
-			Interrupt, Vec, get_long ( Vec ), nFrameCycles, nLineCycles, nHBL );
+			Interrupt, Vec, STMemory_ReadLong ( Vec ), nFrameCycles, nLineCycles, nHBL );
 	}
 
-	M68000_Exception ( Vec , M68000_INT_MFP );
+	M68000_Exception ( Vec , M68000_EXCEPTION_SRC_INT_MFP );
 }
 
 
@@ -278,7 +313,7 @@ static void MFP_UpdateFlags(void)
 /**
  * Test interrupt request to see if can cause exception,return TRUE if pass vector
  */
-static BOOL MFP_InterruptRequest(int nMfpException, Uint8 Bit, Uint8 *pPendingReg, Uint8 MaskRegister,
+static bool MFP_InterruptRequest(int nMfpException, Uint8 Bit, Uint8 *pPendingReg, Uint8 MaskRegister,
                                  Uint8 PriorityMaskLow, Uint8 PriorityMaskHigh, Uint8 *pInServiceReg)
 {
 	/* Are any higher priority interupts in service? */
@@ -314,42 +349,61 @@ static BOOL MFP_InterruptRequest(int nMfpException, Uint8 Bit, Uint8 *pPendingRe
 /**
  * Check 'pending' registers to see if any MFP interrupts need servicing.
  * Request interrupt if necessary.
+ * Return TRUE if at least one MFP interrupt was allowed, else return FALSE.
  */
-void MFP_CheckPendingInterrupts(void)
+bool MFP_CheckPendingInterrupts(void)
 {
-	if ((MFP_IPRA & 0xb5) == 0 && (MFP_IPRB & 0xf0) == 0)
+	int	InterruptPossible;
+
+
+	if ((MFP_IPRA & 0xb5) == 0 && (MFP_IPRB & 0xfb) == 0)
 	{
 		/* Should never get here, but if do just clear flag (see 'MFP_UpdateFlags') */
 		M68000_UnsetSpecial(SPCFLAG_MFP);
-		return;
+		return FALSE;
 	}
 
+
+	InterruptPossible = FALSE;
+
 	if (MFP_IPRA & MFP_TIMER_GPIP7_BIT)   /* Check MFP GPIP7 interrupt (bit 7) */
-		MFP_InterruptRequest(MFP_EXCEPT_GPIP7, MFP_TIMER_GPIP7_BIT, &MFP_IPRA, MFP_IMRA, 0x80, 0x00, &MFP_ISRA);
+		InterruptPossible |= MFP_InterruptRequest(MFP_EXCEPT_GPIP7, MFP_TIMER_GPIP7_BIT, &MFP_IPRA, MFP_IMRA, 0x80, 0x00, &MFP_ISRA);
 
 	if (MFP_IPRA & MFP_TIMER_A_BIT)       /* Check Timer A (bit 5) */
-		MFP_InterruptRequest(MFP_EXCEPT_TIMERA, MFP_TIMER_A_BIT, &MFP_IPRA, MFP_IMRA, 0xe0, 0x00, &MFP_ISRA);
+		InterruptPossible |= MFP_InterruptRequest(MFP_EXCEPT_TIMERA, MFP_TIMER_A_BIT, &MFP_IPRA, MFP_IMRA, 0xe0, 0x00, &MFP_ISRA);
 
 	if (MFP_IPRA & MFP_RCVBUFFULL_BIT)    /* Check Receive buffer full (bit 4) */
-		MFP_InterruptRequest(MFP_EXCEPT_RECBUFFULL, MFP_RCVBUFFULL_BIT, &MFP_IPRA, MFP_IMRA, 0xf0, 0x00, &MFP_ISRA);
+		InterruptPossible |= MFP_InterruptRequest(MFP_EXCEPT_RECBUFFULL, MFP_RCVBUFFULL_BIT, &MFP_IPRA, MFP_IMRA, 0xf0, 0x00, &MFP_ISRA);
 
 	if (MFP_IPRA & MFP_TRNBUFEMPTY_BIT)   /* Check transmit buffer empty (bit 2) */
-		MFP_InterruptRequest(MFP_EXCEPT_TRANSBUFFEMPTY, MFP_TRNBUFEMPTY_BIT, &MFP_IPRA, MFP_IMRA, 0xfb, 0x00, &MFP_ISRA);
+		InterruptPossible |= MFP_InterruptRequest(MFP_EXCEPT_TRANSBUFFEMPTY, MFP_TRNBUFEMPTY_BIT, &MFP_IPRA, MFP_IMRA, 0xfc, 0x00, &MFP_ISRA);
 
 	if (MFP_IPRA & MFP_TIMER_B_BIT)       /* Check Timer B (bit 0) */
-		MFP_InterruptRequest(MFP_EXCEPT_TIMERB, MFP_TIMER_B_BIT, &MFP_IPRA, MFP_IMRA, 0xff, 0x00, &MFP_ISRA);
+		InterruptPossible |= MFP_InterruptRequest(MFP_EXCEPT_TIMERB, MFP_TIMER_B_BIT, &MFP_IPRA, MFP_IMRA, 0xff, 0x00, &MFP_ISRA);
+
 
 	if (MFP_IPRB & MFP_FDCHDC_BIT)        /* Check FDC (bit 7) */
-		MFP_InterruptRequest(MFP_EXCEPT_GPIP5, MFP_FDCHDC_BIT, &MFP_IPRB, MFP_IMRB, 0xff, 0x80, &MFP_ISRB);
+		InterruptPossible |= MFP_InterruptRequest(MFP_EXCEPT_GPIP5, MFP_FDCHDC_BIT, &MFP_IPRB, MFP_IMRB, 0xff, 0x80, &MFP_ISRB);
 
 	if (MFP_IPRB & MFP_ACIA_BIT)          /* Check ACIA (Keyboard or MIDI) (bit 6) */
-		MFP_InterruptRequest(MFP_EXCEPT_ACIA, MFP_ACIA_BIT, &MFP_IPRB, MFP_IMRB, 0xff, 0xc0, &MFP_ISRB);
+		InterruptPossible |= MFP_InterruptRequest(MFP_EXCEPT_ACIA, MFP_ACIA_BIT, &MFP_IPRB, MFP_IMRB, 0xff, 0xc0, &MFP_ISRB);
 
 	if (MFP_IPRB & MFP_TIMER_C_BIT)       /* Check Timer C (bit 5) */
-		MFP_InterruptRequest(MFP_EXCEPT_TIMERC, MFP_TIMER_C_BIT, &MFP_IPRB, MFP_IMRB, 0xff, 0xe0, &MFP_ISRB);
+		InterruptPossible |= MFP_InterruptRequest(MFP_EXCEPT_TIMERC, MFP_TIMER_C_BIT, &MFP_IPRB, MFP_IMRB, 0xff, 0xe0, &MFP_ISRB);
 
 	if (MFP_IPRB & MFP_TIMER_D_BIT)       /* Check Timer D (bit 4) */
-		MFP_InterruptRequest(MFP_EXCEPT_TIMERD, MFP_TIMER_D_BIT, &MFP_IPRB, MFP_IMRB, 0xff, 0xf0, &MFP_ISRB);
+		InterruptPossible |= MFP_InterruptRequest(MFP_EXCEPT_TIMERD, MFP_TIMER_D_BIT, &MFP_IPRB, MFP_IMRB, 0xff, 0xf0, &MFP_ISRB);
+
+	if (MFP_IPRB & MFP_GPU_DONE_BIT)      /* Check GPU done (bit 3) */
+		InterruptPossible |= MFP_InterruptRequest(MFP_EXCEPT_GPIP3, MFP_GPU_DONE_BIT, &MFP_IPRB, MFP_IMRB, 0xff, 0xf8, &MFP_ISRB);
+
+	if (MFP_IPRB & MFP_GPIP_1_BIT)        /* Check (Falcon) Centronics ACK / (ST) RS232 DCD (bit 1) */
+		InterruptPossible |= MFP_InterruptRequest(MFP_EXCEPT_GPIP1, MFP_GPIP_1_BIT, &MFP_IPRB, MFP_IMRB, 0xff, 0xfe, &MFP_ISRB);
+
+	if (MFP_IPRB & MFP_GPIP_0_BIT)        /* Check Centronics BUSY (bit 0) */
+		InterruptPossible |= MFP_InterruptRequest(MFP_EXCEPT_GPIP0, MFP_GPIP_0_BIT, &MFP_IPRB, MFP_IMRB, 0xff, 0xff, &MFP_ISRB);
+
+	return InterruptPossible;
 }
 
 
@@ -374,15 +428,23 @@ void MFP_InputOnChannel(Uint8 Bit, Uint8 EnableBit, Uint8 *pPendingReg)
  */
 void MFP_TimerA_EventCount_Interrupt(void)
 {
-	if (MFP_TA_MAINCOUNTER == 1)            /* Timer expired? If so, generate interrupt */
+	if (MFP_TA_MAINCOUNTER == 1)			/* Timer expired? If so, generate interrupt */
 	{
-		MFP_TA_MAINCOUNTER = MFP_TADR;      /* Reload timer from data register */
+		MFP_TA_MAINCOUNTER = MFP_TADR;		/* Reload timer from data register */
 
 		/* Acknowledge in MFP circuit, pass bit,enable,pending */
 		MFP_InputOnChannel(MFP_TIMER_A_BIT,MFP_IERA,&MFP_IPRA);
 	}
 	else
-		MFP_TA_MAINCOUNTER--;               /* Subtract timer main counter */
+	{
+		MFP_TA_MAINCOUNTER--;			/* Decrement timer main counter */
+		/* As MFP_TA_MAINCOUNTER is Uint8, when we decrement MFP_TA_MAINCOUNTER=0 */
+		/* we go to MFP_TA_MAINCOUNTER=255, which is the wanted behaviour because */
+		/* data reg = 0 means 256 in fact. So, the next 2 lines are redundant. */
+/*		if ( MFP_TA_MAINCOUNTER < 0 )
+			MFP_TA_MAINCOUNTER = 255;
+*/
+	}
 }
 
 
@@ -392,15 +454,23 @@ void MFP_TimerA_EventCount_Interrupt(void)
  */
 void MFP_TimerB_EventCount_Interrupt(void)
 {
-	if (MFP_TB_MAINCOUNTER == 1)            /* Timer expired? If so, generate interrupt */
+	if (MFP_TB_MAINCOUNTER == 1)			/* Timer expired? If so, generate interrupt */
 	{
-		MFP_TB_MAINCOUNTER = MFP_TBDR;      /* Reload timer from data register */
+		MFP_TB_MAINCOUNTER = MFP_TBDR;		/* Reload timer from data register */
 
 		/* Acknowledge in MFP circuit, pass bit,enable,pending */
 		MFP_InputOnChannel(MFP_TIMER_B_BIT,MFP_IERA,&MFP_IPRA);
 	}
 	else
-		MFP_TB_MAINCOUNTER--;               /* Subtract timer main counter */
+	{
+		MFP_TB_MAINCOUNTER--;			/* Decrement timer main counter */
+		/* As MFP_TB_MAINCOUNTER is Uint8, when we decrement MFP_TB_MAINCOUNTER=0 */
+		/* we go to MFP_TB_MAINCOUNTER=255, which is the wanted behaviour because */
+		/* data reg = 0 means 256 in fact. So, the next 2 lines are redundant. */
+/*		if ( MFP_TB_MAINCOUNTER < 0 )
+			MFP_TB_MAINCOUNTER = 255;
+*/
+	}
 }
 
 
@@ -409,7 +479,7 @@ void MFP_TimerB_EventCount_Interrupt(void)
  * Start Timer A or B - EventCount mode is done in HBL handler to time correctly
  */
 static int MFP_StartTimer_AB(Uint8 TimerControl, Uint16 TimerData, int Handler,
-                             BOOL bFirstTimer, BOOL *pTimerCanResume)
+                             bool bFirstTimer, bool *pTimerCanResume)
 {
 	int TimerClockCycles = 0;
 
@@ -449,7 +519,7 @@ static int MFP_StartTimer_AB(Uint8 TimerControl, Uint16 TimerData, int Handler,
 
 				/* Start timer from now? If not continue timer using PendingCycleOver */
 				if (bFirstTimer)
-					Int_AddRelativeInterrupt(TimerClockCycles, INT_MFP_CYCLE, Handler, AddCurCycles);
+					Int_AddRelativeInterruptWithOffset(TimerClockCycles, INT_MFP_CYCLE, Handler, AddCurCycles);
 				else
 				{
 					int	TimerClockCyclesInternal = INT_CONVERT_TO_INTERNAL ( TimerClockCycles , INT_MFP_CYCLE );
@@ -464,13 +534,45 @@ static int MFP_StartTimer_AB(Uint8 TimerControl, Uint16 TimerData, int Handler,
 				*pTimerCanResume = TRUE;		/* timer was set, resume is possible if stop/start it later */
 			}
 		}
+
+		else	/* Ctrl was 0 -> timer is stopped */
+		{
+			/* do nothing, only print some traces */
+			if ( HATARI_TRACE_LEVEL ( HATARI_TRACE_MFP_START ) )
+			{
+				int nFrameCycles = Cycles_GetCounter(CYCLES_COUNTER_VIDEO);
+				int nLineCycles = nFrameCycles % nCyclesPerLine;
+				HATARI_TRACE_PRINT ( "mfp stop AB handler=%d data=%d ctrl=%d timer_cyc=%d pending_cyc=%d video_cyc=%d %d@%d pc=%x instr_cyc=%d first=%s resume=%s\n" ,
+			                     Handler, TimerData, TimerControl, TimerClockCycles, PendingCyclesOver,
+			                     nFrameCycles, nLineCycles, nHBL, M68000_GetPC(), CurrentInstrCycles,
+			                     bFirstTimer?"true":"false" , *pTimerCanResume?"true":"false" );
+			}
+		}
 	}
-	else
+
+	else	/* timer control > 7 */
 	{
 		/* Make sure no outstanding interrupts in list if channel is disabled */
 		Int_RemovePendingInterrupt(Handler);
 	}
 
+	if (TimerControl == 8 )				/* event count mode */
+	{
+		/* do nothing, only print some traces */
+
+		if ( HATARI_TRACE_LEVEL ( HATARI_TRACE_MFP_START ) )
+		{
+			int nFrameCycles = Cycles_GetCounter(CYCLES_COUNTER_VIDEO);
+			int nLineCycles = nFrameCycles % nCyclesPerLine;
+			HATARI_TRACE_PRINT ( "mfp start AB handler=%d data=%d ctrl=%d timer_cyc=%d pending_cyc=%d video_cyc=%d %d@%d pc=%x instr_cyc=%d first=%s resume=%s\n" ,
+			                     Handler, TimerData, TimerControl, TimerClockCycles, PendingCyclesOver,
+			                     nFrameCycles, nLineCycles, nHBL, M68000_GetPC(), CurrentInstrCycles,
+			                     bFirstTimer?"true":"false" , *pTimerCanResume?"true":"false" );
+		}
+
+
+	}
+
 	return TimerClockCycles;
 }
 
@@ -480,7 +582,7 @@ static int MFP_StartTimer_AB(Uint8 TimerControl, Uint16 TimerData, int Handler,
  * Start Timer C or D
  */
 static int MFP_StartTimer_CD(Uint8 TimerControl, Uint16 TimerData, int Handler,
-                             BOOL bFirstTimer, BOOL *pTimerCanResume)
+                             bool bFirstTimer, bool *pTimerCanResume)
 {
 	int TimerClockCycles = 0;
 
@@ -519,7 +621,7 @@ static int MFP_StartTimer_CD(Uint8 TimerControl, Uint16 TimerData, int Handler,
 
 				/* Start timer from now? If not continue timer using PendingCycleOver */
 				if (bFirstTimer)
-					Int_AddRelativeInterrupt(TimerClockCycles, INT_MFP_CYCLE, Handler, AddCurCycles);
+					Int_AddRelativeInterruptWithOffset(TimerClockCycles, INT_MFP_CYCLE, Handler, AddCurCycles);
 				else
 				{
 					int	TimerClockCyclesInternal = INT_CONVERT_TO_INTERNAL ( TimerClockCycles , INT_MFP_CYCLE );
@@ -535,8 +637,19 @@ static int MFP_StartTimer_CD(Uint8 TimerControl, Uint16 TimerData, int Handler,
 			}
 		}
 	}
-	else
+
+	else	/* timer control is 0 */
 	{
+		if ( HATARI_TRACE_LEVEL ( HATARI_TRACE_MFP_START ) )
+		{
+			int nFrameCycles = Cycles_GetCounter(CYCLES_COUNTER_VIDEO);
+			int nLineCycles = nFrameCycles % nCyclesPerLine;
+			HATARI_TRACE_PRINT ( "mfp stop CD handler=%d data=%d ctrl=%d timer_cyc=%d pending_cyc=%d video_cyc=%d %d@%d pc=%x instr_cyc=%d first=%s resume=%s\n" ,
+			                     Handler, TimerData, TimerControl, TimerClockCycles, PendingCyclesOver,
+			                     nFrameCycles, nLineCycles, nHBL, M68000_GetPC(), CurrentInstrCycles,
+			                     bFirstTimer?"true":"false" , *pTimerCanResume?"true":"false" );
+		}
+
 		/* Make sure no outstanding interrupts in list if channel is disabled */
 		Int_RemovePendingInterrupt(Handler);
 	}
@@ -549,20 +662,33 @@ static int MFP_StartTimer_CD(Uint8 TimerControl, Uint16 TimerData, int Handler,
 /**
  * Read Timer A or B - If in EventCount MainCounter already has correct value
  */
-static Uint8 MFP_ReadTimer_AB(Uint8 TimerControl, Uint8 MainCounter, int TimerCycles, int Handler)
+static Uint8 MFP_ReadTimer_AB(Uint8 TimerControl, Uint8 MainCounter, int TimerCycles, int Handler, bool TimerIsStopping)
 {
-	int TimerCyclesPassed;
+//	int TimerCyclesPassed;
 
 	/* Find TimerAB count, if no interrupt or not in delay mode assume
 	 * in Event Count mode so already up-to-date as kept by HBL */
 	if (Int_InterruptActive(Handler) && (TimerControl > 0) && (TimerControl <= 7))
 	{
 		/* Find cycles passed since last interrupt */
-		TimerCyclesPassed = TimerCycles - Int_FindCyclesPassed ( Handler, INT_MFP_CYCLE );
+		//TimerCyclesPassed = TimerCycles - Int_FindCyclesPassed ( Handler, INT_MFP_CYCLE );
 		MainCounter = MFP_CYCLE_TO_REG ( Int_FindCyclesPassed ( Handler, INT_MFP_CYCLE ), TimerControl );
 		//fprintf ( stderr , "mfp read AB passed %d count %d\n" , TimerCyclesPassed, MainCounter );
 	}
 
+	/* If the timer is stopped when the internal mfp data reg is already < 1 */
+	/* then the data reg will be 0 (=256) next time the timer will be restarted */
+	/* if no write is made to the data reg before */
+	if ( TimerIsStopping )
+	{
+		if ( Int_FindCyclesPassed ( Handler, INT_MFP_CYCLE ) < MFP_REG_TO_CYCLES ( 1 , TimerControl ) )
+		{
+			MainCounter = 0;			/* internal mfp counter becomes 0 (=256) */
+			HATARI_TRACE ( HATARI_TRACE_MFP_READ , "mfp read AB handler=%d stopping timer while data reg between 1 and 0 : forcing data to 256\n" ,
+					Handler );
+		}
+	}
+
 	if ( HATARI_TRACE_LEVEL ( HATARI_TRACE_MFP_READ ) )
 	{
 		int nFrameCycles = Cycles_GetCounter(CYCLES_COUNTER_VIDEO);
@@ -580,20 +706,33 @@ static Uint8 MFP_ReadTimer_AB(Uint8 TimerControl, Uint8 MainCounter, int TimerCy
 /**
  * Read Timer C or D
  */
-static Uint8 MFP_ReadTimerCD(Uint8 TimerControl, Uint8 TimerData, Uint8 MainCounter, int TimerCycles, int Handler)
+static Uint8 MFP_ReadTimerCD(Uint8 TimerControl, Uint8 TimerData, Uint8 MainCounter, int TimerCycles, int Handler, bool TimerIsStopping)
 {
-	int TimerCyclesPassed;
+//	int TimerCyclesPassed;
 
 	/* Find TimerCD count. If timer is off, MainCounter already contains
 	 * the latest value */
 	if (Int_InterruptActive(Handler))
 	{
 		/* Find cycles passed since last interrupt */
-		TimerCyclesPassed = TimerCycles - Int_FindCyclesPassed ( Handler, INT_MFP_CYCLE );
+		//TimerCyclesPassed = TimerCycles - Int_FindCyclesPassed ( Handler, INT_MFP_CYCLE );
 		MainCounter = MFP_CYCLE_TO_REG ( Int_FindCyclesPassed ( Handler, INT_MFP_CYCLE ), TimerControl);
 		//fprintf ( stderr , "mfp read CD passed %d count %d\n" , TimerCyclesPassed, MainCounter );
 	}
 
+	/* If the timer is stopped when the internal mfp data reg is already < 1 */
+	/* then the data reg will be 0 (=256) next time the timer will be restarted */
+	/* if no write is made to the data reg before */
+	if ( TimerIsStopping )
+	{
+		if ( Int_FindCyclesPassed ( Handler, INT_MFP_CYCLE ) < MFP_REG_TO_CYCLES ( 1 , TimerControl ) )
+		{
+			MainCounter = 0;			/* internal mfp counter becomes 0 (=256) */
+			HATARI_TRACE ( HATARI_TRACE_MFP_READ , "mfp read CD handler=%d stopping timer while data reg between 1 and 0 : forcing data to 256\n" ,
+					Handler );
+		}
+	}
+
 	if ( HATARI_TRACE_LEVEL ( HATARI_TRACE_MFP_READ ) )
 	{
 		int nFrameCycles = Cycles_GetCounter(CYCLES_COUNTER_VIDEO);
@@ -622,10 +761,10 @@ static void MFP_StartTimerA(void)
 /**
  * Read Timer A
  */
-static void MFP_ReadTimerA(void)
+static void MFP_ReadTimerA(bool TimerIsStopping)
 {
 	MFP_TA_MAINCOUNTER = MFP_ReadTimer_AB(MFP_TACR, MFP_TA_MAINCOUNTER,
-	                                      TimerAClockCycles, INTERRUPT_MFP_TIMERA);
+	                                      TimerAClockCycles, INTERRUPT_MFP_TIMERA, TimerIsStopping);
 }
 
 
@@ -646,10 +785,10 @@ static void MFP_StartTimerB(void)
 /**
  * Read Timer B
  */
-static void MFP_ReadTimerB(void)
+static void MFP_ReadTimerB(bool TimerIsStopping)
 {
 	MFP_TB_MAINCOUNTER = MFP_ReadTimer_AB(MFP_TBCR, MFP_TB_MAINCOUNTER,
-	                                      TimerBClockCycles, INTERRUPT_MFP_TIMERB);
+	                                      TimerBClockCycles, INTERRUPT_MFP_TIMERB, TimerIsStopping);
 }
 
 
@@ -668,10 +807,10 @@ static void MFP_StartTimerC(void)
 /**
  * Read Timer C
  */
-static void MFP_ReadTimerC(void)
+static void MFP_ReadTimerC(bool TimerIsStopping)
 {
 	MFP_TC_MAINCOUNTER = MFP_ReadTimerCD((MFP_TCDCR>>4)&7, MFP_TCDR, MFP_TC_MAINCOUNTER,
-	                                     TimerCClockCycles, INTERRUPT_MFP_TIMERC);
+	                                     TimerCClockCycles, INTERRUPT_MFP_TIMERC, TimerIsStopping);
 }
 
 
@@ -690,10 +829,10 @@ static void MFP_StartTimerD(void)
 /**
  * Read Timer D
  */
-static void MFP_ReadTimerD(void)
+static void MFP_ReadTimerD(bool TimerIsStopping)
 {
 	MFP_TD_MAINCOUNTER = MFP_ReadTimerCD(MFP_TCDCR&7, MFP_TDDR, MFP_TD_MAINCOUNTER,
-	                                     TimerDClockCycles, INTERRUPT_MFP_TIMERD);
+	                                     TimerDClockCycles, INTERRUPT_MFP_TIMERD, TimerIsStopping);
 }
 
 
@@ -834,6 +973,14 @@ void MFP_GPIP_ReadByte(void)
 	FDC_GpipRead();
 
 	IoMem[0xfffa01] = MFP_GPIP;
+
+	if ( HATARI_TRACE_LEVEL ( HATARI_TRACE_MFP_READ ) )
+	{
+		int nFrameCycles = Cycles_GetCounter(CYCLES_COUNTER_VIDEO);
+		int nLineCycles = nFrameCycles % nCyclesPerLine;
+		HATARI_TRACE_PRINT ( "mfp read gpip fa01=0x%x video_cyc=%d %d@%d pc=%x instr_cycle %d\n" ,
+			MFP_GPIP, nFrameCycles, nLineCycles, nHBL, M68000_GetPC(), CurrentInstrCycles );
+	}
 }
 
 /*-----------------------------------------------------------------------*/
@@ -998,8 +1145,8 @@ void MFP_TimerAData_ReadByte(void)
 {
 	M68000_WaitState(4);
 
-	if (MFP_TACR != 8)          /* Is event count? Need to re-calculate counter */
-		MFP_ReadTimerA();       /* Stores result in 'MFP_TA_MAINCOUNTER' */
+	if (MFP_TACR != 8)          		/* Is event count? Need to re-calculate counter */
+		MFP_ReadTimerA(FALSE);		/* Stores result in 'MFP_TA_MAINCOUNTER' */
 
 	IoMem[0xfffa1f] = MFP_TA_MAINCOUNTER;
 }
@@ -1010,10 +1157,64 @@ void MFP_TimerAData_ReadByte(void)
  */
 void MFP_TimerBData_ReadByte(void)
 {
+	Uint8 TB_count;
+
 	M68000_WaitState(4);
 
-	if (MFP_TBCR != 8)          /* Is event count? Need to re-calculate counter */
-		MFP_ReadTimerB();       /* Stores result in 'MFP_TB_MAINCOUNTER' */
+	/* Is it event count mode or not? */
+	if (MFP_TBCR != 8)
+	{
+		/* Not event count mode, so handle as normal timer
+		 * and store result in 'MFP_TB_MAINCOUNTER' */
+		MFP_ReadTimerB(FALSE);
+	}
+	else if (bUseVDIRes)
+	{
+		/* HBLs are disabled in VDI mode, but TOS expects to read a 1. */
+		MFP_TB_MAINCOUNTER = 1;
+	}
+	/* Special case when reading $fffa21, we need to test if the current read instruction */
+	/* overlaps the horizontal video position where $fffa21 is changed */
+	else
+	{
+		int nFrameCycles = Cycles_GetCounter(CYCLES_COUNTER_VIDEO);
+		int pos_start , pos_read;
+
+		/* Cycle position of the start of the current instruction */
+		pos_start = nFrameCycles % nCyclesPerLine;
+		/* Cycle position of the read for the current instruction (approximatively, we consider */
+		/* the read happens after 4 cycles (due to MFP wait states in that case)) */
+		/* This is quite a hack, but hard to do without proper 68000 read cycle emulation */
+		if ( CurrentInstrCycles <= 8 )			/* move.b (a0),d0 / cmp.b (a0),d0 ... */
+			pos_read = pos_start + 4;		/* wait state */
+		else						/* cmp.b $fa21.w,d0 (BIG Demo) ... */
+			pos_read = pos_start + 8;		/* more time needed to compute the effective address */
+
+		TB_count = MFP_TB_MAINCOUNTER;			/* default value */
+
+		/* If Timer B's change happens before the read cycle of the current instruction, we must return */
+		/* the current value - 1 (because MFP_TimerB_EventCount_Interrupt was not called yet) */
+		if ( (nHBL >= nStartHBL ) && ( nHBL < nEndHBL )	/* ensure display is ON and timer B can happen */
+			&& ( LineTimerBCycle > pos_start ) && ( LineTimerBCycle < pos_read ) )
+		{
+			HATARI_TRACE ( HATARI_TRACE_MFP_READ , "mfp read TB overlaps pos_start=%d TB_pos=%d pos_read=%d nHBL=%d \n",
+					pos_start, LineTimerBCycle, pos_read , nHBL );
+
+			TB_count--;
+			if ( TB_count == 0 )			/* going from 1 to 0 : timer restart, reload data reg */
+				TB_count = MFP_TBDR;
+			/* Going from 0 to -1 : data reg is in fact going from 256 to 255. As TB_count is Uint8, */
+			/* this is already what we get when we decrement TB_count=0. So, the next 2 lines are redundant. */
+/*			else if ( TB_count < 0 )
+				TB_count = 255;
+*/
+		}
+
+		HATARI_TRACE ( HATARI_TRACE_MFP_READ , "mfp read TB data=%d video_cyc=%d %d@%d pc=%x instr_cyc=%d\n" ,
+					TB_count, nFrameCycles, pos_start, nHBL, M68000_GetPC(), CurrentInstrCycles );
+		IoMem[0xfffa21] = TB_count;
+		return;
+	}
 
 	IoMem[0xfffa21] = MFP_TB_MAINCOUNTER;
 }
@@ -1026,7 +1227,7 @@ void MFP_TimerCData_ReadByte(void)
 {
 	M68000_WaitState(4);
 
-	MFP_ReadTimerC();        /* Stores result in 'MFP_TC_MAINCOUNTER' */
+	MFP_ReadTimerC(FALSE);		/* Stores result in 'MFP_TC_MAINCOUNTER' */
 
 	IoMem[0xfffa23] = MFP_TC_MAINCOUNTER;
 }
@@ -1048,7 +1249,7 @@ void MFP_TimerDData_ReadByte(void)
 	}
 	else
 	{
-		MFP_ReadTimerD();        /* Stores result in 'MFP_TD_MAINCOUNTER' */
+		MFP_ReadTimerD(FALSE);	/* Stores result in 'MFP_TD_MAINCOUNTER' */
 		IoMem[0xfffa25] = MFP_TD_MAINCOUNTER;
 	}
 }
@@ -1217,9 +1418,9 @@ void MFP_VectorReg_WriteByte(void)
 
 	if ( HATARI_TRACE_LEVEL ( HATARI_TRACE_MFP_WRITE ) )
 	{
-		int nFrameCycles = Cycles_GetCounter(CYCLES_COUNTER_VIDEO);;
+		int nFrameCycles = Cycles_GetCounter(CYCLES_COUNTER_VIDEO);
 		int nLineCycles = nFrameCycles % nCyclesPerLine;
-		HATARI_TRACE_PRINT ( "mfp write vector reg fa17=%x video_cyc=%d %d@%d pc=%x instr_cycle %d\n" ,
+		HATARI_TRACE_PRINT ( "mfp write vector reg fa17=0x%x video_cyc=%d %d@%d pc=%x instr_cycle %d\n" ,
 			MFP_VR, nFrameCycles, nLineCycles, nHBL, M68000_GetPC(), CurrentInstrCycles );
 	}
 
@@ -1244,7 +1445,7 @@ void MFP_TimerACtrl_WriteByte(void)
 		 * continue from where we left if the timer is restarted later
 		 * without writing to the data register. */
 		if ((new_tacr == 0) && (MFP_TACR >=1) && (MFP_TACR <= 7))
-			MFP_ReadTimerA();           /* Store result in 'MFP_TA_MAINCOUNTER' */
+			MFP_ReadTimerA(TRUE);	/* Store result in 'MFP_TA_MAINCOUNTER' */
 
 		MFP_TACR = new_tacr;            /* set to new value before calling MFP_StartTimer */
 		MFP_StartTimerA();              /* start/stop timer depending on control reg */
@@ -1270,7 +1471,7 @@ void MFP_TimerBCtrl_WriteByte(void)
 		 * continue from where we left if the timer is restarted later
 		 * without writing to the data register. */
 		if ((new_tbcr == 0) && (MFP_TBCR >= 1) && (MFP_TBCR <= 7))
-			MFP_ReadTimerB();           /* Store result in 'MFP_TB_MAINCOUNTER' */
+			MFP_ReadTimerB(TRUE);	/* Store result in 'MFP_TB_MAINCOUNTER' */
 
 		MFP_TBCR = new_tbcr;            /* set to new value before calling MFP_StartTimer */
 		MFP_StartTimerB();              /* start/stop timer depending on control reg */
@@ -1290,21 +1491,22 @@ void MFP_TimerCDCtrl_WriteByte(void)
 
 	new_tcdcr = IoMem[0xfffa1d];
 	old_tcdcr = MFP_TCDCR;
+//fprintf ( stderr , "write fa1d new %x old %x\n" , IoMem[0xfffa1d] , MFP_TCDCR );
 
-	if ((old_tcdcr & 0x70) != (new_tcdcr & 0x70))	/* Timer control changed */
+	if ((old_tcdcr & 0x70) != (new_tcdcr & 0x70))	/* Timer C control changed */
 	{
 		/* If we stop a timer which was in delay mode, we need to store
 		 * the current value of the counter to be able to read it or to
 		 * continue from where we left if the timer is restarted later
 		 * without writing to the data register. */
 		if ((new_tcdcr & 0x70) == 0)
-			MFP_ReadTimerC();			/* Store result in 'MFP_TC_MAINCOUNTER' */
+			MFP_ReadTimerC(TRUE);		/* Store result in 'MFP_TC_MAINCOUNTER' */
 
-		MFP_TCDCR = ( new_tcdcr & 0x70 ) | ( old_tcdcr & 0x07 );	/* we set TCCR and keep old TDDR */
+		MFP_TCDCR = ( new_tcdcr & 0x70 ) | ( old_tcdcr & 0x07 );	/* we set TCCR and keep old TDDR in case we need to read it below */
 		MFP_StartTimerC();			/* start/stop timer depending on control reg */
 	}
 
-	if ((old_tcdcr & 0x07) != (new_tcdcr & 0x07))	/* Timer control changed */
+	if ((old_tcdcr & 0x07) != (new_tcdcr & 0x07))	/* Timer D control changed */
 	{
 		Uint32 pc = M68000_GetPC();
 
@@ -1327,19 +1529,21 @@ void MFP_TimerCDCtrl_WriteByte(void)
 			 * (eg Paradroid, Speedball I) so we simply intercept the Timer-D
 			 * setup code in TOS and fix the numbers with more 'laid-back'
 			 * values. This still keeps 100% compatibility */
-			// FIXME [NP] : this seems wrong, this can set TDCR to 7 even if it was 0 !
-			new_tcdcr = IoMem[0xfffa1d] = (IoMem[0xfffa1d] & 0xf0) | 7;
-			bAppliedTimerDPatch = TRUE;
+			if ( new_tcdcr & 0x07 )			/* apply patch only if timer D is being started */
+			{
+				new_tcdcr = IoMem[0xfffa1d] = (IoMem[0xfffa1d] & 0xf0) | 7;
+				bAppliedTimerDPatch = TRUE;
+			}
 		}
 
 		/* If we stop a timer which was in delay mode, we need to store the current value */
 		/* of the counter to be able to read it or to continue from where we left if the timer is */
 		/* restarted later without writing to the data register. */
 		if ((new_tcdcr & 0x07) == 0)
-			MFP_ReadTimerD();			/* Stores result in 'MFP_TD_MAINCOUNTER' */
+			MFP_ReadTimerD(TRUE);	/* Stores result in 'MFP_TD_MAINCOUNTER' */
 
 		MFP_TCDCR = new_tcdcr;		/* set to new value before calling MFP_StartTimer */
-		MFP_StartTimerD();			/* start/stop timer depending on control reg */
+		MFP_StartTimerD();		/* start/stop timer depending on control reg */
 	}
 }
 
@@ -1358,6 +1562,14 @@ void MFP_TimerAData_WriteByte(void)
 		MFP_TA_MAINCOUNTER = MFP_TADR;  /* Timer is off, store to main counter */
 		TimerACanResume = FALSE;        /* we need to set a new int when timer start */
 	}
+
+	if ( HATARI_TRACE_LEVEL ( HATARI_TRACE_MFP_WRITE ) )
+	{
+		int nFrameCycles = Cycles_GetCounter(CYCLES_COUNTER_VIDEO);
+		int nLineCycles = nFrameCycles % nCyclesPerLine;
+		HATARI_TRACE_PRINT ( "mfp write data reg A fa1f=0x%x new counter=0x%x video_cyc=%d %d@%d pc=%x instr_cycle %d\n" ,
+			MFP_TADR, MFP_TA_MAINCOUNTER, nFrameCycles, nLineCycles, nHBL, M68000_GetPC(), CurrentInstrCycles );
+	}
 }
 
 /*-----------------------------------------------------------------------*/
@@ -1375,6 +1587,14 @@ void MFP_TimerBData_WriteByte(void)
 		MFP_TB_MAINCOUNTER = MFP_TBDR;  /* Timer is off, store to main counter */
 		TimerBCanResume = FALSE;        /* we need to set a new int when timer start */
 	}
+
+	if ( HATARI_TRACE_LEVEL ( HATARI_TRACE_MFP_WRITE ) )
+	{
+		int nFrameCycles = Cycles_GetCounter(CYCLES_COUNTER_VIDEO);
+		int nLineCycles = nFrameCycles % nCyclesPerLine;
+		HATARI_TRACE_PRINT ( "mfp write data reg B fa21=0x%x new counter=0x%x video_cyc=%d %d@%d pc=%x instr_cycle %d\n" ,
+			MFP_TBDR, MFP_TB_MAINCOUNTER, nFrameCycles, nLineCycles, nHBL, M68000_GetPC(), CurrentInstrCycles );
+	}
 }
 
 /*-----------------------------------------------------------------------*/
@@ -1392,6 +1612,14 @@ void MFP_TimerCData_WriteByte(void)
 		MFP_TC_MAINCOUNTER = MFP_TCDR;  /* Timer is off, store to main counter */
 		TimerCCanResume = FALSE;        /* we need to set a new int when timer start */
 	}
+
+	if ( HATARI_TRACE_LEVEL ( HATARI_TRACE_MFP_WRITE ) )
+	{
+		int nFrameCycles = Cycles_GetCounter(CYCLES_COUNTER_VIDEO);
+		int nLineCycles = nFrameCycles % nCyclesPerLine;
+		HATARI_TRACE_PRINT ( "mfp write data reg C fa23=0x%x new counter=0x%x video_cyc=%d %d@%d pc=%x instr_cycle %d\n" ,
+			MFP_TCDR, MFP_TC_MAINCOUNTER, nFrameCycles, nLineCycles, nHBL, M68000_GetPC(), CurrentInstrCycles );
+	}
 }
 
 /*-----------------------------------------------------------------------*/
@@ -1423,4 +1651,12 @@ void MFP_TimerDData_WriteByte(void)
 		MFP_TD_MAINCOUNTER = MFP_TDDR;  /* Timer is off, store to main counter */
 		TimerDCanResume = FALSE;        /* we need to set a new int when timer start */
 	}
+
+	if ( HATARI_TRACE_LEVEL ( HATARI_TRACE_MFP_WRITE ) )
+	{
+		int nFrameCycles = Cycles_GetCounter(CYCLES_COUNTER_VIDEO);
+		int nLineCycles = nFrameCycles % nCyclesPerLine;
+		HATARI_TRACE_PRINT ( "mfp write data reg D fa25=0x%x new counter=0x%x video_cyc=%d %d@%d pc=%x instr_cycle %d\n" ,
+			MFP_TDDR, MFP_TD_MAINCOUNTER, nFrameCycles, nLineCycles, nHBL, M68000_GetPC(), CurrentInstrCycles );
+	}
 }
