@@ -6,7 +6,7 @@
 
   Reset emulation state.
 */
-const char Reset_rcsid[] = "Hatari $Id: reset.c,v 1.27 2008/03/13 23:17:27 thothy Exp $";
+const char Reset_rcsid[] = "Hatari $Id: reset.c,v 1.29 2008-05-25 19:58:56 thothy Exp $";
 
 #include "main.h"
 #include "configuration.h"
@@ -18,8 +18,8 @@ const char Reset_rcsid[] = "Hatari $Id: reset.c,v 1.27 2008/03/13 23:17:27 thoth
 #include "ikbd.h"
 #include "int.h"
 #include "m68000.h"
-#include "misc.h"
 #include "mfp.h"
+#include "midi.h"
 #include "psg.h"
 #include "reset.h"
 #include "screen.h"
@@ -36,7 +36,7 @@ const char Reset_rcsid[] = "Hatari $Id: reset.c,v 1.27 2008/03/13 23:17:27 thoth
  * Reset ST emulator states, chips, interrupts and registers.
  * Return zero or negative TOS image load error code.
  */
-static int Reset_ST(BOOL bCold)
+static int Reset_ST(bool bCold)
 {
 	if (bCold)
 	{
@@ -70,6 +70,8 @@ static int Reset_ST(BOOL bCold)
 		Screen_Reset();               /* Reset screen */
 	M68000_Reset(bCold);          /* Reset CPU */
 
+	Midi_Reset();
+
 	/* Start HBL and VBL interrupts */
 	Video_StartInterrupts();
 
