@@ -19,7 +19,21 @@
   only convert the screen every 50 times a second - inbetween frames are not
   processed.
 */
-const char Screen_rcsid[] = "Hatari $Id: screen.c,v 1.79 2008/03/26 20:16:26 eerot Exp $";
+
+/*
+ Changes:
+  - 2008/06/01	[NP]
+    Although it seems possible to display 47 lines in the bottom border
+    with a second 60/50 Hz switch, most programs consider there are 45
+    visible lines in the bottom border, which gives a total of 274 lines
+    for a screen. This fixes garbage that could appear in the last two
+    lines when displaying 47 lines (Digiworld 2 by ICE, Tyranny by DHS).
+  - 2008/08/09 [ET] Add statusbar
+  - 2008-11-20 [ET] Move event selection to main.c
+*/
+
+
+const char Screen_rcsid[] = "Hatari $Id: screen.c,v 1.98 2008-11-20 21:46:08 eerot Exp $";
 
 #include <SDL.h>
 #include <SDL_endian.h>
@@ -29,13 +43,13 @@ const char Screen_rcsid[] = "Hatari $Id: screen.c,v 1.79 2008/03/26 20:16:26 eer
 #include "ikbd.h"
 #include "log.h"
 #include "m68000.h"
-#include "misc.h"
 #include "paths.h"
 #include "screen.h"
+#include "control.h"
 #include "convert/routines.h"
-#include "screenSnapShot.h"
 #include "sound.h"
 #include "spec512.h"
+#include "statusbar.h"
 #include "vdi.h"
 #include "video.h"
 #include "falcon/videl.h"
@@ -46,11 +60,11 @@ const char Screen_rcsid[] = "Hatari $Id: screen.c,v 1.79 2008/03/26 20:16:26 eer
 SDL_Surface *sdlscrn = NULL;                /* The SDL screen surface */
 int nScreenZoomX, nScreenZoomY;             /* Zooming factors, used for scaling mouse motions */
 int nBorderPixelsLeft, nBorderPixelsRight;  /* Pixels in left and right border */
-int nBorderPixelsBottom;                    /* Lines in bottom border */
+int nBorderPixelsTop, nBorderPixelsBottom;  /* Lines in top and bottom border */
 
 /* extern for shortcuts and falcon/hostscreen.c */
-BOOL bGrabMouse = FALSE;      /* Grab the mouse cursor in the window */
-BOOL bInFullScreen = FALSE;   /* TRUE if in full screen */
+bool bGrabMouse = FALSE;      /* Grab the mouse cursor in the window */
+bool bInFullScreen = FALSE;   /* TRUE if in full screen */
 
 /* extern for spec512.c */
 int STScreenLeftSkipBytes;
@@ -68,6 +82,7 @@ static Uint8 *pPCScreenDest;                       /* Destination PC buffer */
 static int STScreenEndHorizLine;                   /* End lines to be converted */
 static int PCScreenBytesPerLine;
 static int STScreenWidthBytes;
+static SDL_Rect STScreenRect;                      /* screen size without statusbar */
 
 static int STScreenLineOffset[NUM_VISIBLE_LINES];  /* Offsets for ST screen lines eg, 0,160,320... */
 static Uint16 HBLPalette[16], PrevHBLPalette[16];  /* Current palette for line, also copy of first line */
@@ -80,8 +95,8 @@ static void (*ScreenDrawFunctionsVDI[3])(void) =
 	ConvertVDIRes_2Colour
 };
 
-static BOOL bScreenContentsChanged;     /* TRUE if buffer changed and requires blitting */
-static BOOL bScrDoubleY;                /* TRUE if double on Y */
+static bool bScreenContentsChanged;     /* TRUE if buffer changed and requires blitting */
+static bool bScrDoubleY;                /* TRUE if double on Y */
 static int ScrUpdateFlag;               /* Bit mask of how to update screen */
 
 
@@ -185,7 +200,7 @@ static void Screen_CreatePalette(void)
  */
 static void Screen_Handle8BitPalettes(void)
 {
-	BOOL bPaletteChanged=FALSE;
+	bool bPaletteChanged=FALSE;
 	int i;
 
 	/* Do need to check for 8-Bit palette change? Ie, update whole screen */
@@ -271,16 +286,27 @@ static void Screen_SetSTScreenOffsets(void)
 	/* Determine border pixels */
 	if (bInFullScreen)
 	{
+		nBorderPixelsTop = ConfigureParams.Screen.nFullScreenBorderPixelsTop;
 		nBorderPixelsLeft = ConfigureParams.Screen.nFullScreenBorderPixelsLeft;
 		nBorderPixelsRight = ConfigureParams.Screen.nFullScreenBorderPixelsRight;
 		nBorderPixelsBottom = ConfigureParams.Screen.nFullScreenBorderPixelsBottom;
 	}
 	else
 	{
+		nBorderPixelsTop = ConfigureParams.Screen.nWindowBorderPixelsTop;
 		nBorderPixelsLeft = ConfigureParams.Screen.nWindowBorderPixelsLeft;
 		nBorderPixelsRight = ConfigureParams.Screen.nWindowBorderPixelsRight;
 		nBorderPixelsBottom = ConfigureParams.Screen.nWindowBorderPixelsBottom;
 	}
+	/* All screen widths need to be aligned to 16-bits.
+	 * 
+	 * TODO: Change VDI_Limit() to generic function and check
+	 * the limits when config values are set or changed (after 1.1).
+	 */
+	nBorderPixelsTop = VDI_Limit(nBorderPixelsTop, 1, 0, OVERSCAN_TOP);
+	nBorderPixelsLeft = VDI_Limit(nBorderPixelsLeft, 16, 0, 48);
+	nBorderPixelsRight = VDI_Limit(nBorderPixelsRight, 16, 0, 48);
+	nBorderPixelsBottom = VDI_Limit(nBorderPixelsBottom, 1, 0, MAX_OVERSCAN_BOTTOM);
 
 	/* Store offset to each horizontal line */
 	for (i = 0; i < NUM_VISIBLE_LINES; i++)
@@ -326,7 +352,7 @@ static void Screen_SetResolution(void)
 			int nZoom = ((Width == 640) ? 2 : 1);
 			/* Add in overscan borders (if 640x200 bitmap is double on Y) */
 			Width += (nBorderPixelsLeft+nBorderPixelsRight) * nZoom;
-			Height += (OVERSCAN_TOP+nBorderPixelsBottom) * nZoom;
+			Height += (nBorderPixelsTop+nBorderPixelsBottom) * nZoom;
 		}
 	}
 
@@ -368,22 +394,35 @@ static void Screen_SetResolution(void)
 	{
 		sdlVideoFlags  = SDL_SWSURFACE|SDL_HWPALETTE;
 	}
+	Height += Statusbar_SetHeight(Width, Height);
 
 	/* Check if we really have to change the video mode: */
 	if (!sdlscrn || sdlscrn->w != Width || sdlscrn->h != Height
 	    || sdlscrn->format->BitsPerPixel != BitCount
 	    || (sdlscrn->flags&SDL_FULLSCREEN) != (sdlVideoFlags&SDL_FULLSCREEN))
 	{
+		if (bInFullScreen)
+		{
+			/* unhide the Hatari WM window for fullscreen */
+			Control_ReparentWindow(Width, Height, bInFullScreen);
+		}
+		
 		/* Set new video mode */
 		//fprintf(stderr,"Requesting video mode %i %i %i\n", Width, Height, BitCount);
 		sdlscrn = SDL_SetVideoMode(Width, Height, BitCount, sdlVideoFlags);
 		//fprintf(stderr,"Got video mode %i %i %i\n", sdlscrn->w, sdlscrn->h, sdlscrn->format->BitsPerPixel);
-		/* We do not support 24 bpp (yet) */
+
+		/* By default ConfigureParams.Screen.nForceBpp and therefore
+		 * BitCount is zero which means "SDL color depth autodetection".
+		 * In this case the SDL_SetVideoMode() call might return
+		 * a 24 bpp resolution
+		 */
 		if (sdlscrn && sdlscrn->format->BitsPerPixel == 24)
 		{
 			fprintf(stderr, "Unsupported color depth 24, trying 32 bpp instead...\n");
 			sdlscrn = SDL_SetVideoMode(Width, Height, 32, sdlVideoFlags);
 		}
+
 		/* Exit if we can not open a screen */
 		if (!sdlscrn)
 		{
@@ -392,12 +431,26 @@ static void Screen_SetResolution(void)
 			exit(-2);
 		}
 
+		if (!bInFullScreen)
+		{
+			/* re-embed the new Hatari SDL window */
+			Control_ReparentWindow(Width, Height, bInFullScreen);
+		}
+		
 		/* Re-init screen palette: */
 		if (sdlscrn->format->BitsPerPixel == 8)
 			Screen_Handle8BitPalettes();    /* Initialize new 8 bit palette */
 		else
 			Screen_SetupRGBTable();         /* Create color convertion table */
 
+		Statusbar_Init(sdlscrn);
+		
+		/* screen area without the statusbar */
+		STScreenRect.x = 0;
+		STScreenRect.y = 0;
+		STScreenRect.w = sdlscrn->w;
+		STScreenRect.h = sdlscrn->h - Statusbar_GetHeight();
+		
 		/* Un-grab mouse pointer in windowed mode: */
 		if (!bGrabMouse)
 			SDL_WM_GrabInput(SDL_GRAB_OFF);
@@ -455,9 +508,6 @@ void Screen_Init(void)
 
 	/* Configure some SDL stuff: */
 	SDL_WM_SetCaption(PROG_NAME, "Hatari");
-	SDL_EventState(SDL_MOUSEMOTION, SDL_ENABLE);
-	SDL_EventState(SDL_MOUSEBUTTONDOWN, SDL_ENABLE);
-	SDL_EventState(SDL_MOUSEBUTTONUP, SDL_ENABLE);
 	SDL_ShowCursor(SDL_DISABLE);
 }
 
@@ -529,7 +579,7 @@ void Screen_SetFullUpdate(void)
  */
 static void Screen_ClearScreen(void)
 {
-	SDL_FillRect(sdlscrn,NULL, SDL_MapRGB(sdlscrn->format, 0, 0, 0) );
+	SDL_FillRect(sdlscrn, &STScreenRect, SDL_MapRGB(sdlscrn->format, 0, 0, 0));
 }
 
 /*-----------------------------------------------------------------------*/
@@ -540,7 +590,7 @@ void Screen_EnterFullScreen(void)
 {
 	if (!bInFullScreen)
 	{
-		Main_PauseEmulation();         /* Hold things... */
+		Main_PauseEmulation(FALSE);         /* Hold things... */
 		bInFullScreen = TRUE;
 
 		if ((ConfigureParams.System.nMachineType == MACHINE_FALCON
@@ -570,7 +620,7 @@ void Screen_ReturnFromFullScreen(void)
 {
 	if (bInFullScreen)
 	{
-		Main_PauseEmulation();        /* Hold things... */
+		Main_PauseEmulation(FALSE);        /* Hold things... */
 		bInFullScreen = FALSE;
 
 		if ((ConfigureParams.System.nMachineType == MACHINE_FALCON
@@ -584,7 +634,6 @@ void Screen_ReturnFromFullScreen(void)
 		{
 			Screen_SetResolution();
 		}
-
 		SDL_Delay(20);                /* To give monitor time to switch resolution */
 		Main_UnPauseEmulation();      /* And off we go... */
 	}
@@ -646,7 +695,7 @@ void Screen_ModeChanged(void)
  * Compare current resolution on line with previous, and set 'UpdateLine' accordingly
  * Return if swap between low/medium resolution
  */
-static BOOL Screen_CompareResolution(int y, int *pUpdateLine, int oldres)
+static bool Screen_CompareResolution(int y, int *pUpdateLine, int oldres)
 {
 	/* Check if wrote to resolution register */
 	if (HBLPaletteMasks[y]&PALETTEMASK_RESOLUTION)  /* See 'Intercept_ShifterMode_WriteByte' */
@@ -670,7 +719,7 @@ static BOOL Screen_CompareResolution(int y, int *pUpdateLine, int oldres)
  */
 static void Screen_ComparePalette(int y, int *pUpdateLine)
 {
-	BOOL bPaletteChanged = FALSE;
+	bool bPaletteChanged = FALSE;
 	int i;
 
 	/* Did write to palette in this or previous frame? */
@@ -706,7 +755,7 @@ static void Screen_ComparePalette(int y, int *pUpdateLine)
  */
 static int Screen_ComparePaletteMask(int res)
 {
-	BOOL bLowMedMix = FALSE;
+	bool bLowMedMix = FALSE;
 	int LineUpdate = 0;
 	int y;
 
@@ -815,7 +864,7 @@ static void Screen_SetConvertDetails(void)
 	PCScreenBytesPerLine = sdlscrn->pitch;        /* Bytes per line */
 	pHBLPalettes = pFrameBuffer->HBLPalettes;     /* HBL palettes pointer */
 	/* Not in TV-Mode? Then double up on Y: */
-	bScrDoubleY = !(ConfigureParams.Screen.MonitorType == MONITOR_TYPE_TV);
+	bScrDoubleY = !(ConfigureParams.Screen.nMonitorType == MONITOR_TYPE_TV);
 
 	if (bUseVDIRes)
 	{
@@ -831,17 +880,18 @@ static void Screen_SetConvertDetails(void)
 		{
 			/* Always draw to WHOLE screen including ALL borders */
 			STScreenLeftSkipBytes = 0;              /* Number of bytes to skip on ST screen for left (border) */
-			STScreenStartHorizLine = 0;             /* Full height */
 
 			if (bUseHighRes)
 			{
 				pFrameBuffer->OverscanModeCopy = OverscanMode = OVERSCANMODE_NONE;
+				STScreenStartHorizLine = 0;
 				STScreenEndHorizLine = 400;
 			}
 			else
 			{
 				STScreenWidthBytes = SCREENBYTES_LINE;  /* Number of horizontal bytes in our ST screen */
-				STScreenEndHorizLine = NUM_VISIBLE_LINES;
+				STScreenStartHorizLine = OVERSCAN_TOP - nBorderPixelsTop;
+				STScreenEndHorizLine = OVERSCAN_TOP + 200 + nBorderPixelsBottom;
 			}
 		}
 		else
@@ -860,7 +910,7 @@ static void Screen_SetConvertDetails(void)
 /**
  * Lock full-screen for drawing
  */
-static BOOL Screen_Lock(void)
+static bool Screen_Lock(void)
 {
 	if (SDL_MUSTLOCK(sdlscrn))
 	{
@@ -906,9 +956,8 @@ static void Screen_SwapSTBuffers(void)
 /*-----------------------------------------------------------------------*/
 /**
  * Blit our converted ST screen to window/full-screen
- * Note that our source image includes all borders so if have them disabled simply blit a smaller source rectangle!
  */
-static void Screen_Blit(BOOL bSwapScreen)
+static void Screen_Blit(void)
 {
 	unsigned char *pTmpScreen;
 
@@ -917,13 +966,11 @@ static void Screen_Blit(BOOL bSwapScreen)
 	{
 		Screen_SwapSTBuffers();
 		/* Swap screen */
-		if (bSwapScreen)
-			SDL_Flip(sdlscrn);
+		SDL_Flip(sdlscrn);
 	}
 	else
 	{
-		/* Blit image */
-		SDL_UpdateRect(sdlscrn, 0,0,0,0);
+		SDL_UpdateRects(sdlscrn, 1, &STScreenRect);
 	}
 
 	/* Swap copy/raster buffers in screen. */
@@ -936,12 +983,14 @@ static void Screen_Blit(BOOL bSwapScreen)
 /*-----------------------------------------------------------------------*/
 /**
  * Draw ST screen to window/full-screen framebuffer
+ * @param  bForceFlip  Force screen update, even if contents did not change
+ * @return  TRUE if screen contents changed
  */
-static void Screen_DrawFrame(BOOL bForceFlip)
+static bool Screen_DrawFrame(bool bForceFlip)
 {
 	int new_res;
 	void (*pDrawFunction)(void);
-	static BOOL bPrevFrameWasSpec512 = FALSE;
+	static bool bPrevFrameWasSpec512 = FALSE;
 
 	/* Scan palette/resolution masks for each line and build up palette/difference tables */
 	new_res = Screen_ComparePaletteMask(STRes);
@@ -953,15 +1002,23 @@ static void Screen_DrawFrame(BOOL bForceFlip)
 	if (pFrameBuffer->bFullUpdate)
 		Screen_SetFullUpdateMask();
 
+	/* restore area potentially left under overlay led
+	 * and saved by Statusbar_OverlayBackup()
+	 */
+	Statusbar_OverlayRestore(sdlscrn);
+	
 	/* Lock screen ready for drawing */
 	if (Screen_Lock())
 	{
 		bScreenContentsChanged = FALSE;      /* Did change (ie needs blit?) */
+
 		/* Set details */
 		Screen_SetConvertDetails();
+		
 		/* Clear screen on full update to clear out borders and also interleaved lines */
 		if (pFrameBuffer->bFullUpdate && !bUseVDIRes)
 			Screen_ClearScreen();
+		
 		/* Call drawing for full-screen */
 		if (bUseVDIRes)
 		{
@@ -995,22 +1052,29 @@ static void Screen_DrawFrame(BOOL bForceFlip)
 		}
 
 		if (pDrawFunction)
-			CALL_VAR(pDrawFunction)
+			CALL_VAR(pDrawFunction);
 
-			/* Unlock screen */
-			Screen_UnLock();
+		/* Unlock screen */
+		Screen_UnLock();
+
+		/* draw statusbar or overlay led(s) after unlock */
+		Statusbar_OverlayBackup(sdlscrn);
+		Statusbar_Update(sdlscrn);
+		
 		/* Clear flags, remember type of overscan as if change need screen full update */
 		pFrameBuffer->bFullUpdate = FALSE;
 		pFrameBuffer->OverscanModeCopy = OverscanMode;
 
 		/* And show to user */
 		if (bScreenContentsChanged || bForceFlip)
-			Screen_Blit(TRUE);
+		{
+			Screen_Blit();
+		}
 
-		/* Grab any animation */
-		if (bRecordingAnimation)
-			ScreenSnapShot_RecordFrame(bScreenContentsChanged);
+		return bScreenContentsChanged;
 	}
+
+	return false;
 }
 
 
@@ -1018,13 +1082,15 @@ static void Screen_DrawFrame(BOOL bForceFlip)
 /**
  * Draw ST screen to window/full-screen
  */
-void Screen_Draw(void)
+bool Screen_Draw(void)
 {
 	if (!bQuitProgram && VideoBase)
 	{
 		/* And draw (if screen contents changed) */
-		Screen_DrawFrame(FALSE);
+		return Screen_DrawFrame(false);
 	}
+
+	return false;
 }
 
 
