@@ -6,13 +6,15 @@
 
   Shortcut keys
 */
-const char ShortCut_rcsid[] = "Hatari $Id: shortcut.c,v 1.29 2008/02/24 20:10:47 thothy Exp $";
+const char ShortCut_rcsid[] = "Hatari $Id: shortcut.c,v 1.43 2008-11-22 18:21:33 eerot Exp $";
 
 #include <SDL.h>
 
 #include "main.h"
 #include "dialog.h"
 #include "audio.h"
+#include "file.h"
+#include "floppy.h"
 #include "joy.h"
 #include "m68000.h"
 #include "memorySnapShot.h"
@@ -21,7 +23,9 @@ const char ShortCut_rcsid[] = "Hatari $Id: shortcut.c,v 1.29 2008/02/24 20:10:47
 #include "screenSnapShot.h"
 #include "configuration.h"
 #include "shortcut.h"
+#include "debugui.h"
 #include "sound.h"
+#include "sdlgui.h"
 
 static SHORTCUTKEYIDX ShortCutKey = SHORTCUT_NONE;  /* current shortcut key */
 
@@ -105,8 +109,7 @@ static void ShortCut_RecordAnimation(void)
 	else
 	{
 		/* Start animation */
-		ScreenSnapShot_BeginRecording(ConfigureParams.Screen.bCaptureChange,
-		                              ConfigureParams.Screen.nFramesPerSecond);
+		ScreenSnapShot_BeginRecording(ConfigureParams.Screen.bCaptureChange);
 	}
 }
 
@@ -136,15 +139,15 @@ static void ShortCut_SoundOnOff(void)
 
 /*-----------------------------------------------------------------------*/
 /**
- * Shortcut to maximum speed
+ * Shortcut to fast forward
  */
-static void ShortCut_MaximumSpeed(void)
+static void ShortCut_FastForward(void)
 {
 	/* If already on max speed, switch back to normal */
-	if (ConfigureParams.System.nMinMaxSpeed == MINMAXSPEED_MAX)
+	if (ConfigureParams.System.bFastForward == TRUE)
 	{
 		/* Restore */
-		ConfigureParams.System.nMinMaxSpeed = MINMAXSPEED_MIN;
+		ConfigureParams.System.bFastForward = FALSE;
 
 		/* Reset the sound emulation variables: */
 		Sound_ResetBufferIndex();
@@ -152,7 +155,7 @@ static void ShortCut_MaximumSpeed(void)
 	else
 	{
 		/* Set maximum speed */
-		ConfigureParams.System.nMinMaxSpeed = MINMAXSPEED_MAX;
+		ConfigureParams.System.bFastForward = TRUE;
 	}
 }
 
@@ -171,12 +174,75 @@ static void ShortCut_BossKey(void)
 		SDL_WM_GrabInput(SDL_GRAB_OFF);
 		bGrabMouse = FALSE;
 	}
+	Main_PauseEmulation(TRUE);
 
 	/* Minimize Window and give up processing to next one! */
 	SDL_WM_IconifyWindow();
 }
 
 
+/*-----------------------------------------------------------------------*/
+/**
+ * Shorcut to pause or debug interface
+ */
+static void ShortCut_Pause(void)
+{
+	if (bEnableDebug)
+	{
+		int running;
+		
+		if (bInFullScreen)
+			Screen_ReturnFromFullScreen();
+
+		/* Call the debugger */
+		running = Main_PauseEmulation(TRUE);
+		DebugUI();
+		if (running)
+			Main_UnPauseEmulation();
+	}
+	else
+	{
+		if (!Main_UnPauseEmulation())
+			Main_PauseEmulation(TRUE);
+	}
+}
+
+
+/**
+ * Shorcut to load a disk image
+ */
+static void ShortCut_InsertDisk(int drive)
+{
+	char *selname, *zip_path = NULL;
+	const char *tmpname;
+
+	if (SDLGui_SetScreen(sdlscrn))
+		return;
+
+	if (ConfigureParams.DiskImage.szDiskFileName[drive][0])
+		tmpname = ConfigureParams.DiskImage.szDiskFileName[drive];
+	else
+		tmpname = ConfigureParams.DiskImage.szDiskImageDirectory;
+
+	Main_PauseEmulation(TRUE);
+	selname = SDLGui_FileSelect(tmpname, &zip_path, FALSE);
+	if (selname)
+	{
+		if (File_Exists(selname))
+			Floppy_SetDiskFileName(drive, selname, zip_path);
+		else
+			Floppy_SetDiskFileNameNone(drive);
+
+		if (zip_path)
+			free(zip_path);
+		free(selname);
+		
+		Floppy_InsertDiskIntoDrive(0);
+	}
+	Main_UnPauseEmulation();
+}
+
+
 /*-----------------------------------------------------------------------*/
 /**
  * Check to see if pressed any shortcut keys, and call handling function
@@ -198,9 +264,11 @@ void ShortCut_ActKey(void)
 		ShortCut_MouseMode();          /* Toggle mouse mode */
 		break;
 	 case SHORTCUT_COLDRESET:
+		Main_UnPauseEmulation();
 		Reset_Cold();                  /* Reset emulator with 'cold' (clear all) */
 		break;
 	 case SHORTCUT_WARMRESET:
+		Main_UnPauseEmulation();
 		Reset_Warm();                  /* Emulator 'warm' reset */
 		break;
 	 case SHORTCUT_SCREENSHOT:
@@ -212,8 +280,8 @@ void ShortCut_ActKey(void)
 	 case SHORTCUT_CURSOREMU:          /* Toggle joystick emu on/off */
 		Joy_ToggleCursorEmulation();
 		break;
-	 case SHORTCUT_MAXSPEED:
-		ShortCut_MaximumSpeed();       /* Toggle Min/Max speed */
+	 case SHORTCUT_FASTFORWARD:
+		ShortCut_FastForward();       /* Toggle Min/Max speed */
 		break;
 	 case SHORTCUT_RECANIM:
 		ShortCut_RecordAnimation();    /* Record animation */
@@ -224,6 +292,9 @@ void ShortCut_ActKey(void)
 	 case SHORTCUT_SOUND:
 		ShortCut_SoundOnOff();         /* Enable/disable sound */
 		break;
+	 case SHORTCUT_PAUSE:
+		ShortCut_Pause();              /* Invoke Pause or Debug UI */
+		break;
 	 case SHORTCUT_QUIT:
 		Main_RequestQuit();
 		break;
@@ -233,6 +304,9 @@ void ShortCut_ActKey(void)
 	 case SHORTCUT_SAVEMEM:
 		MemorySnapShot_Capture(ConfigureParams.Memory.szMemoryCaptureFileName, TRUE);
 		break;
+	 case SHORTCUT_INSERTDISKA:
+		ShortCut_InsertDisk(0);
+		break;
 	 case SHORTCUT_KEYS:
 	 case SHORTCUT_NONE:
 		/* ERROR: cannot happen, just make compiler happy */
@@ -242,6 +316,56 @@ void ShortCut_ActKey(void)
 }
 
 
+/*-----------------------------------------------------------------------*/
+/**
+ * Invoke shortcut identified by name. This supports only keys for
+ * functionality that cannot be invoked with command line options
+ * (for remote GUIs etc).
+ */
+bool Shortcut_Invoke(const char *shortcut)
+{
+	struct {
+		SHORTCUTKEYIDX id;
+		const char *name;
+	} shortcuts[] = {
+		{ SHORTCUT_MOUSEMODE, "mousemode" },
+		{ SHORTCUT_COLDRESET, "coldreset" },
+		{ SHORTCUT_WARMRESET, "warmreset" },
+		{ SHORTCUT_SCREENSHOT, "screenshot" },
+		{ SHORTCUT_BOSSKEY, "bosskey" },
+		{ SHORTCUT_RECANIM, "recanim" },
+		{ SHORTCUT_RECSOUND, "recsound" },
+		{ SHORTCUT_SAVEMEM, "savemem" },
+		{ SHORTCUT_QUIT, "quit" },
+		{ SHORTCUT_NONE, NULL }
+	};
+	int i;
+
+	if (ShortCutKey != SHORTCUT_NONE)
+	{
+		fprintf(stderr, "Shortcut invocation failed, shortcut already active\n");
+		return FALSE;
+	}
+	for (i = 0; shortcuts[i].name; i++)
+	{
+		if (strcmp(shortcut, shortcuts[i].name) == 0)
+		{
+			ShortCutKey = shortcuts[i].id;
+			ShortCut_ActKey();
+			ShortCutKey = SHORTCUT_NONE;
+			return TRUE;
+		}
+	}
+	fprintf(stderr, "WARNING: unknown shortcut '%s'\n\n", shortcut);
+	fprintf(stderr, "Hatari shortcuts are:\n");
+	for (i = 0; shortcuts[i].name; i++)
+	{
+		fprintf(stderr, "- %s\n", shortcuts[i].name);
+	}
+	return FALSE;
+}
+
+
 /*-----------------------------------------------------------------------*/
 /**
  * Check whether given key was any of the ones in given shortcut array.
@@ -264,7 +388,7 @@ static SHORTCUTKEYIDX ShortCut_CheckKey(int symkey, int *keys)
  * If press is set, store the key array index.
  * Return zero if key didn't match to a shortcut
  */
-int ShortCut_CheckKeys(int modkey, int symkey, BOOL press)
+int ShortCut_CheckKeys(int modkey, int symkey, bool press)
 {
 	SHORTCUTKEYIDX key;
 
