@@ -44,10 +44,19 @@
 /*			(readme.prg by TEX (in 1987))						*/
 /* 2008/01/24	[NP]	In Spec512_StartScanLine, use different values for LineStartCycle when	*/
 /*			running in 50 Hz or 60 Hz (TEX Spectrum Slideshow in 60 Hz).		*/
+/* 2008/12/14	[NP]	In Spec512_StoreCyclePalette, instead of approximating write position	*/
+/*			by Cycles_GetCounter+8, we use different cases for movem, .l acces and	*/
+/*			.w acces (similar to cycles.c). This gives correct results when using	*/
+/*			"move.w d0,(a0) or move.w (a0)+,(a1)+" for example, which were shifted	*/
+/*			8 or 4 pixels too late. Calibration was made using a custom program to	*/
+/*			compare the results with a real STF in different cases (fix Froggies	*/
+/*			Over The Fence Main Menu).						*/
+/* 2008/12/21	[NP]	Use BusMode to adjust Cycles_GetCounterOnReadAccess and		*/
+/*			Cycles_GetCounterOnWriteAccess depending on who is owning the	*/
+/*			bus (cpu, blitter).						*/
 
 
-
-const char Spec512_rcsid[] = "Hatari $Id: spec512.c,v 1.23 2008/03/19 00:24:44 thothy Exp $";
+const char Spec512_rcsid[] = "Hatari $Id: spec512.c,v 1.31 2008-12-14 18:02:03 npomarede Exp $";
 
 #include <SDL_byteorder.h>
 
@@ -55,11 +64,12 @@ const char Spec512_rcsid[] = "Hatari $Id: spec512.c,v 1.23 2008/03/19 00:24:44 t
 #include "cycles.h"
 #include "int.h"
 #include "m68000.h"
+#include "ioMem.h"
 #include "screen.h"
 #include "spec512.h"
 #include "video.h"
 #include "configuration.h"
-#include "trace.h"
+
 
 /* As 68000 clock multiple of 4 this mean we can only write to the palette this many time per scanline */
 #define MAX_CYCLEPALETTES_PERLINE  ((512/4) + 1)		/* +1 for the '-1' added as a line's terminator */
@@ -81,7 +91,7 @@ static int nPalettesAccesses;   /* Number of times accessed palette registers */
 static Uint16 CycleColour;
 static int CycleColourIndex;
 static int nScanLine, ScanLineCycleCount;
-static BOOL bIsSpec512Display;
+static bool bIsSpec512Display;
 
 #if SDL_BYTEORDER == SDL_BIG_ENDIAN
 static const int STRGBPalEndianTable[16] =
@@ -95,7 +105,7 @@ static const int STRGBPalEndianTable[16] =
 /**
  * Return TRUE if this frame is a Spectrum 512 style image(MUST be low res/non-mix)
  */
-BOOL Spec512_IsImage(void)
+bool Spec512_IsImage(void)
 {
 	/* Normal Low res screen? */
 	if (STRes == ST_LOW_RES && bIsSpec512Display)
@@ -145,7 +155,26 @@ void Spec512_StoreCyclePalette(Uint16 col, Uint32 addr)
 	/* To correct this, assume a delay of 8 cycles (should give a good approximation */
 	/* of a move.w or movem.l for example) */
 	//  FrameCycles = Cycles_GetCounterOnWriteAccess(CYCLES_COUNTER_VIDEO);
-	FrameCycles = Cycles_GetCounter(CYCLES_COUNTER_VIDEO) + 8;
+	if ( BusMode == BUS_MODE_BLITTER )
+	{
+		FrameCycles = Cycles_GetCounterOnWriteAccess(CYCLES_COUNTER_VIDEO);
+	}
+	else							/* BUS_MODE_CPU */
+	{
+#ifdef OLD_CYC_PAL
+		FrameCycles = Cycles_GetCounter(CYCLES_COUNTER_VIDEO) + 8;
+#else
+		if ( OpcodeFamily == i_MVMLE )
+		{
+			FrameCycles = Cycles_GetCounter(CYCLES_COUNTER_VIDEO) + 8;
+		}
+
+		else if ( nIoMemAccessSize == SIZE_LONG )	/* long access */
+			FrameCycles = Cycles_GetCounter(CYCLES_COUNTER_VIDEO) + CurrentInstrCycles - 8;
+		else						/* word/byte access */
+			FrameCycles = Cycles_GetCounter(CYCLES_COUNTER_VIDEO) + CurrentInstrCycles - 4;
+#endif
+	}
 
 
 	/* Find scan line we are currently on and get index into cycle-palette table */
@@ -218,16 +247,15 @@ void Spec512_StartFrame(void)
 		pCyclePalette->LineCycles = -1;          /* Term */
 	}
 
-	/* Copy first line palette, kept in 'HBLPalettes' and store to 'STRGBPalette' */
-	for (i = 0; i < 16; i++)
-	{
+       /* Copy first line palette, kept in 'HBLPalettes' and store to 'STRGBPalette' */
+       for (i = 0; i < 16; i++)
+       {
 #if SDL_BYTEORDER == SDL_BIG_ENDIAN
-		STRGBPalette[STRGBPalEndianTable[i]] = ST2RGB[pHBLPalettes[i]];
+               STRGBPalette[STRGBPalEndianTable[i]] = ST2RGB[pHBLPalettes[i]];
 #else
-		STRGBPalette[i] = ST2RGB[pHBLPalettes[i]];
+               STRGBPalette[i] = ST2RGB[pHBLPalettes[i]];
 #endif
-
-	}
+       }
 
 	/* Ready for first call to 'Spec512_ScanLine' */
 	nScanLine = 0;
@@ -280,7 +308,11 @@ void Spec512_StartScanLine(void)
 	/* Update palette entries until we reach start of displayed screen */
 	ScanLineCycleCount = 0;
 //	for(i=0; i<((SCREEN_START_CYCLE-16)/4); i++)  /* This '16' is as we've already added in the 'move' instruction timing */
+#ifdef OLD_CYC_PAL
 	for (i=0; i<((LineStartCycle-SCREENBYTES_LEFT*2)/4 + 6); i++)	/* [NP] '6' is required to align pixels and colors */
+#else
+	for (i=0; i<((LineStartCycle-SCREENBYTES_LEFT*2)/4 + 7); i++)	/* [NP] '7' is required to align pixels and colors */
+#endif
 		Spec512_UpdatePaletteSpan();              /* Update palette for this 4-cycle period */
 
 	/* And skip for left border is not using overscan display to user */
