@@ -15,7 +15,7 @@
   on boot-up which (correctly) cause a bus-error on Hatari as they would in a
   real STfm. If a user tries to select any of these images we bring up an error.
 */
-const char TOS_rcsid[] = "Hatari $Id: tos.c,v 1.54 2007/12/18 18:56:19 thothy Exp $";
+const char TOS_rcsid[] = "Hatari $Id: tos.c,v 1.58 2008-09-05 21:29:01 eerot Exp $";
 
 #include <SDL_endian.h>
 
@@ -33,10 +33,11 @@ const char TOS_rcsid[] = "Hatari $Id: tos.c,v 1.54 2007/12/18 18:56:19 thothy Ex
 #include "vdi.h"
 
 
-Uint16 TosVersion;                      /* eg, 0x0100, 0x0102 */
+bool bIsEmuTOS;
+Uint16 TosVersion;                      /* eg. 0x0100, 0x0102 */
 Uint32 TosAddress, TosSize;             /* Address in ST memory and size of TOS image */
-BOOL bTosImageLoaded = FALSE;           /* Successfully loaded a TOS image? */
-BOOL bRamTosImage;                      /* TRUE if we loaded a RAM TOS image */
+bool bTosImageLoaded = FALSE;           /* Successfully loaded a TOS image? */
+bool bRamTosImage;                      /* TRUE if we loaded a RAM TOS image */
 unsigned int ConnectedDriveMask = 0x03; /* Bit mask of connected drives, eg 0x7 is A,B,C */
 int nNumDrives = 2;                     /* Number of drives, default is 2 for A: and B: */
 
@@ -170,7 +171,7 @@ static const TOS_PATCH TosPatches[] =
 /**
  * Save/Restore snapshot of local variables ('MemorySnapShot_Store' handles type)
  */
-void TOS_MemorySnapShot_Capture(BOOL bSave)
+void TOS_MemorySnapShot_Capture(bool bSave)
 {
 	/* Save/Restore details */
 	MemorySnapShot_Store(&TosVersion, sizeof(TosVersion));
@@ -260,7 +261,7 @@ static void TOS_CheckSysConfig(void)
 {
 	if ((TosVersion == 0x0106 || TosVersion == 0x0162) && ConfigureParams.System.nMachineType != MACHINE_STE)
 	{
-		Log_AlertDlg(LOG_INFO, "TOS versions 1.06 and 1.62 are for Atari STE only.\n"
+		Log_AlertDlg(LOG_ERROR, "TOS versions 1.06 and 1.62 are for Atari STE only.\n"
 		             " ==> Switching to STE mode now.\n");
 		IoMem_UnInit();
 		ConfigureParams.System.nMachineType = MACHINE_STE;
@@ -268,7 +269,7 @@ static void TOS_CheckSysConfig(void)
 	}
 	else if ((TosVersion & 0x0f00) == 0x0300 && ConfigureParams.System.nMachineType != MACHINE_TT)
 	{
-		Log_AlertDlg(LOG_INFO, "TOS versions 3.0x are for Atari TT only.\n"
+		Log_AlertDlg(LOG_ERROR, "TOS versions 3.0x are for Atari TT only.\n"
 		             " ==> Switching to TT mode now.\n");
 		IoMem_UnInit();
 		ConfigureParams.System.nMachineType = MACHINE_TT;
@@ -278,7 +279,7 @@ static void TOS_CheckSysConfig(void)
 	}
 	else if ((TosVersion & 0x0f00) == 0x0400 && ConfigureParams.System.nMachineType != MACHINE_FALCON)
 	{
-		Log_AlertDlg(LOG_INFO, "TOS versions 4.x are for Atari Falcon only.\n"
+		Log_AlertDlg(LOG_ERROR, "TOS versions 4.x are for Atari Falcon only.\n"
 		             " ==> Switching to Falcon mode now.\n");
 		IoMem_UnInit();
 		ConfigureParams.System.nMachineType = MACHINE_FALCON;
@@ -289,7 +290,7 @@ static void TOS_CheckSysConfig(void)
 	else if ((TosVersion < 0x0300 && ConfigureParams.System.nMachineType == MACHINE_FALCON)
 	         || (TosVersion < 0x0200 && ConfigureParams.System.nMachineType == MACHINE_TT))
 	{
-		Log_AlertDlg(LOG_INFO, "This TOS versions does not work in TT/Falcon mode.\n"
+		Log_AlertDlg(LOG_ERROR, "This TOS versions does not work in TT/Falcon mode.\n"
 		             " ==> Switching to STE mode now.\n");
 		IoMem_UnInit();
 		ConfigureParams.System.nMachineType = MACHINE_STE;
@@ -300,6 +301,13 @@ static void TOS_CheckSysConfig(void)
 			M68000_CheckCpuLevel();
 		}
 	}
+	else if (TosVersion >= 0x0300 && ConfigureParams.System.nCpuLevel < 2)
+	{
+		Log_AlertDlg(LOG_ERROR, "This TOS versions requires a CPU >= 68020.\n"
+		             " ==> Switching to 68020 mode now.\n");
+		ConfigureParams.System.nCpuLevel = 2;
+		M68000_CheckCpuLevel();
+	}
 }
 
 
@@ -312,7 +320,6 @@ int TOS_LoadImage(void)
 {
 	Uint8 *pTosFile = NULL;
 	long nFileSize;
-	BOOL bIsEmuTOS;
 
 	bTosImageLoaded = FALSE;
 
@@ -396,7 +403,7 @@ int TOS_LoadImage(void)
 	if (TosVersion == 0x0100 && bUseVDIRes)
 	{
 		/* Warn user */
-		Log_AlertDlg(LOG_WARN, "To use GEM extended resolutions, you must select a TOS >= 1.02.");
+		Log_AlertDlg(LOG_ERROR, "To use GEM extended resolutions, you must select a TOS >= 1.02.");
 		/* And select non VDI */
 		bUseVDIRes = ConfigureParams.Screen.bUseExtVdiResolutions = FALSE;
 	}
