@@ -9,21 +9,20 @@
   We intercept and direct some XBios calls to handle the RS-232 etc. and help
   with floppy debugging.
 */
-const char XBios_rcsid[] = "Hatari $Id: xbios.c,v 1.14 2008/01/23 19:32:36 thothy Exp $";
+const char XBios_rcsid[] = "Hatari $Id: xbios.c,v 1.18 2008-11-03 20:24:25 thothy Exp $";
 
 #include "main.h"
 #include "configuration.h"
 #include "floppy.h"
 #include "log.h"
 #include "m68000.h"
-#include "misc.h"
 #include "rs232.h"
 #include "screenSnapShot.h"
 #include "stMemory.h"
 #include "xbios.h"
 
 
-#define XBIOS_DEBUG 0
+#define XBIOS_DEBUG 0	/* for floppy read/write */
 
 
 /* List of Atari ST RS-232 baud rates */
@@ -53,7 +52,7 @@ static const int BaudRates[] =
  * XBIOS Floppy Read
  * Call 8
  */
-static BOOL XBios_Floprd(Uint32 Params)
+static bool XBios_Floprd(Uint32 Params)
 {
 #if XBIOS_DEBUG
 	char *pBuffer;
@@ -80,7 +79,7 @@ static BOOL XBios_Floprd(Uint32 Params)
  * XBIOS Floppy Write
  * Call 9
  */
-static BOOL XBios_Flopwr(Uint32 Params)
+static bool XBios_Flopwr(Uint32 Params)
 {
 #if XBIOS_DEBUG
 	char *pBuffer;
@@ -107,7 +106,7 @@ static BOOL XBios_Flopwr(Uint32 Params)
  * XBIOS RsConf
  * Call 15
  */
-static BOOL XBios_Rsconf(Uint32 Params)
+static bool XBios_Rsconf(Uint32 Params)
 {
 	short int Baud,Ctrl,Ucr,Rsr,Tsr,Scr;
 
@@ -151,10 +150,8 @@ static BOOL XBios_Rsconf(Uint32 Params)
  * XBIOS Scrdmp
  * Call 20
  */
-static BOOL XBios_Scrdmp(Uint32 Params)
+static bool XBios_Scrdmp(Uint32 Params)
 {
-	fprintf(stderr, "XBIOS screendump!\n");
-
 	ScreenSnapShot_SaveScreen();
 
 	/* Correct return code? */
@@ -164,27 +161,11 @@ static BOOL XBios_Scrdmp(Uint32 Params)
 }
 
 
-/*----------------------------------------------------------------------- */
-/**
- * XBIOS Prtblk
- * Call 36
- */
-static BOOL XBios_Prtblk(Uint32 Params)
-{
-	fprintf(stderr, "Intercepted XBIOS Prtblk()\n");
-
-	/* Correct return code? */
-	Regs[REG_D0] = 0;
-
-	return TRUE;
-}
-
-
 /*-----------------------------------------------------------------------*/
 /**
  * Check if we need to re-direct XBios call to our own routines
  */
-BOOL XBios(void)
+bool XBios(void)
 {
 	Uint32 Params;
 	Uint16 XBiosCall;
@@ -193,22 +174,23 @@ BOOL XBios(void)
 	Params = Regs[REG_A7];
 	XBiosCall = STMemory_ReadWord(Params);
 
-	/*Log_Printf(LOG_DEBUG, "XBIOS %d\n",XBiosCall);*/
-
 	switch (XBiosCall)
 	{
 	 case 8:
+		HATARI_TRACE ( HATARI_TRACE_OS_XBIOS, "XBIOS Floprd()\n" );
 		return XBios_Floprd(Params);
 	 case 9:
+		HATARI_TRACE ( HATARI_TRACE_OS_XBIOS, "XBIOS Flopwr()\n" );
 		return XBios_Flopwr(Params);
 	 case 15:
+		HATARI_TRACE ( HATARI_TRACE_OS_XBIOS, "XBIOS Rsconf()\n" );
 		return XBios_Rsconf(Params);
 	 case 20:
+		HATARI_TRACE ( HATARI_TRACE_OS_XBIOS, "XBIOS Scrdmp()\n" );
 		return XBios_Scrdmp(Params);
-	 case 36:
-		return XBios_Prtblk(Params);
 
 	 default:  /* Call as normal! */
+		HATARI_TRACE ( HATARI_TRACE_OS_XBIOS, "XBIOS %d\n", XBiosCall );
 		return FALSE;
 	}
 }
